#!/bin/bash

    ###########################
    #                         #
    # Функции для установщика #
    #                         #
    ###########################
pre_init()
{
    TITLE=$DEF_TITLE

    if [[ $network_flg == 1 ]]
    then
        BACKTITLE="Сетевая установка ОС Эльбрус"
    elif [[ $cross_flg == 1 ]]
    then
        BACKTITLE="Установка ОС Эльбрус в кросс режиме"
    else
        BACKTITLE="Установка ОС Эльбрус"
    fi

    DEV_SELECT_INPUT=cd
    DEV_SELECT_OUTPUT=hdd

    if [[ $debug_flg == 1 ]]
    then
        # поток ошибок в дебажный лог
        exec 2>>$DEBUG_LOG
    else
        # необработанные ошибки скидываются в лог
        exec 2> >(tee -a $INST_LOG)
    fi

    # Обязана быть, используется для временных файлов
    [ ! -d /tmp ] && mkdir /tmp

    # Убираем вывод сообщений ядра из консоли в лог
    echo 2 > /proc/sys/kernel/printk

    if [ -f ${INST_DIR}/VERSION ]
    then
        echo "Версия инсталлятора:" >> $INST_LOG
        cat ${INST_DIR}/VERSION     >> $INST_LOG
    else
        echo "Версия инсталлятора не определена"    >> $INST_LOG
    fi

    # Добавление информации о машине в шапку лога
    echo -n -e "\nЗапуск установки в " >> $INST_LOG
    date        >> $INST_LOG
    echo        >> $INST_LOG

    echo "Данные о хостовой машине:" >> $INST_LOG
    uname -a    >> $INST_LOG
    echo        >> $INST_LOG

    # Добавление информации о запущенном процессе 'einst'
    # в файловой системе '/proc'
    echo "Process id (в файловой системе /proc): $$" >> $INST_LOG

    echo "Загрузочные параметры хоста:"     >> $INST_LOG
    cat $cmdline   >> $INST_LOG
    echo                >> $INST_LOG
}

# имитирует деятельность для прогресс бара
do_progress ()
{
    local progress_count=0

    while [ ! -s $progress ]; do
        [ $progress_count -ge 3 ] && progress_count=0 || progress_count=$[$progress_count+1]

cat <<EOF
XXX
$1
$2 ${progress_symbol[$progress_count]}
XXX
EOF
        sleep 1

        [[ `cat $RESULT` == 1 ]] && break
    done

    rm -f $progress
}


do_progress_apt()
{
    local progress_count=0
    local tmp=0
    local text=
    local string=
    local string_prev=

    while true
    do
        [ $progress_count -ge 3 ] && progress_count=0 || progress_count=$[$progress_count+1]

        string=`tail -1 $TARGET_DIR/$PIPE_FILE`

        # Загрузка
        if [[ `echo $string | grep "dlstatus" | grep "Retrieving"` ]]
        then
            # Сколько готово
            all_files=`echo $string | awk '{gsub(/ \(.* remaining\)$/,"", $0); print $NF}'`
            done_files=`echo $string | awk '{gsub(/ \(.* remaining\)$/,"", $0); print $(NF-2)}'`

            if [[ -z "$all_files" || -z "$done_files" ]]
            # Если что-то пустое
            then
                echo "Пустые значеня all_files или done_files  прогрессбара в do_progress_apt" #>> $INST_LOG
                echo $string
            else
                # Подсчет процентов "tmp"
                [ $all_files -ge $done_files ] && tmp=$((100*($done_files)/$all_files))
            fi
            text="                     Проверка файлов"

        # Установка
        # Пишем название текущего пакета
        elif [[ `echo $string | grep -o "pmstatus"` ]]
        then
            text="  Текущий пакет: `echo $string | awk -F ':' '{print $2;}'`"
            tmp=`echo $string  | awk -F ':' '{print $3;}' | cut -d'.' -f1`

            # сброс счетчика для нового пакета
            if [[ $string != $string_prev ]]
            then
                progress_count=1
                string_prev=$string
            fi

        # Конфигурирование
        elif [[ `echo $string | grep -o "pmconffile"` ]]
        then
            text="          `echo $string | awk -F ':' '{print $2;}'`"
        # Ошибка
        elif [[ `echo $string | grep -o "pmerror"` ]]
        then
            apt_error_flg=1
            text="`echo $string | awk -F ':' '{print $4;}'`"
        else
            text="                      Инициализация"
        fi

cat <<EOF
XXX
$tmp

$text ${progress_symbol_dots[$progress_count]}
XXX
EOF

        remote_message "$text"
        sleep 1

        # Проверка на наличие процесса apt-get
        chroot $TARGET_DIR /bin/bash -c "ps afx | grep 'apt-get' | grep -v 'grep' | grep install" &>/dev/null
        if [[ $? != 0 ]] || [[ $apt_error_flg == 1 ]]
        then
#           echo 1 > $RESULT
            return 1;   ### Диалог зависает в пайпе?
        fi
    done

#        rm -f $progress
}

do_progress_apt_no_dialog()
{
    local text=
    local text_prev=
    local string=
    local string_prev=
    local progress_count=0

    # Время на инициализацию в chroot'е
    sleep 2

    while true
    do
        # Проверка на наличие процесса apt-get
        chroot $TARGET_DIR /bin/bash -c "ps afx | grep 'apt-get' | grep -v 'grep' | grep install" &>/dev/null
        if [[ $? != 0 ]] || [[ $apt_error_flg == 1 ]]
        then
            return 1;   ### Диалог зависает в пайпе?
        fi

        string=`tail -1 $TARGET_DIR/$PIPE_FILE`

        if [[ "$string_prev" == "$string" ]]
        then
            sleep 0.5
            continue
        else
            string_prev="$string"

            # Загрузка
            if [[ `echo $string | grep "dlstatus" | grep "Retrieving"` ]]
            then
                text="Проверка файлов `echo $string | awk '{print $3 " из " $5}'`..."
            # Установка
            # Пишем название текущего пакета
            elif [[ `echo $string | grep -o "pmstatus"` ]]
            then
                # BUG вывод не с начала строки
                text="Установка `echo $string | awk -F ':' '{print $2;}'`..."
            # Конфигурирование
            elif [[ `echo $string | grep -o "pmconffile"` ]]
            then
                text="Конфигурирование `echo $string | awk -F ':' '{print $2;}'`..."
            # Ошибка
            elif [[ `echo $string | grep -o "pmerror"` ]]
            then
                apt_error_flg=1
                text="Ошибка `echo $string | awk -F ':' '{print $4;}'`"
            else
                text="Инициализация..."
            fi

            [[ "$text_prev" != "$text" ]] && dialog_infobox "$text"
            text_prev="$text"

            sleep 0.5
        fi
    done
}

do_progress_dpkg ()
{
    local progress_count=0

    while [ ! -s $progress ]; do
        [ $progress_count -ge 3 ] && progress_count=0 || progress_count=$[$progress_count+1]

cat <<EOF
XXX
$1
$2 ${progress_symbol_dots[$progress_count]}
XXX
EOF
        remote_message "$2"
        sleep 1

        [[ `cat $RESULT` == 1 ]] && break
    done

    rm -f $progress
}

do_progress_cp()
{
cat <<EOF
XXX
$1
$2
XXX
EOF
    remote_message "$2"

    [[ `cat $RESULT` == 1 ]] && break
}

do_progress_sync()
{
    local progress_count=0
    local tmp=0
    local dirty=
    local dirty_all=

    # Сколько всего памяти не синхронизировано
    dirty_all=`cat /proc/meminfo | grep Dirty | grep -o -E '[0-9]{1,}'`

    while [ ! -s $progress ]
    do
        [ $progress_count -ge 3 ] && progress_count=0 || progress_count=$[$progress_count+1]

         # Сколько осталось не синхронизировано
         dirty=`cat /proc/meminfo | grep Dirty | grep -o -E '[0-9]{1,}'`

        if [[ -z "$dirty_all" || -z "$dirty" ]]
        # Если что-то пустое
        then
            tmp=50
        elif [[ $dirty_all == 0 ]] || [[ $dirty == 0 ]]
        then
            tmp=100
        else
            # Подсчет процентов "tmp"
            [ $dirty_all -ge $dirty ] && tmp=$((100*($dirty_all-$dirty)/$dirty_all))
        fi

cat <<EOF
XXX
$tmp
$1 ${progress_symbol[$progress_count]}
XXX
EOF
        sleep 1

        if [[ `cat $RESULT` == 1 ]]
        then
            echo "Ошибка синхронизации" >> $INST_LOG
            break
        fi
    done

    rm -f $progress
}

check_vm_inst()
{
    [[ "$vm_flg" == 1 ]] && return 0

    local linux_version=

    linux_version=`cat ${SOURCE_DIR}/INFO | grep 'linux version=' | awk -F'=' '{print $NF}' 2>>$INST_LOG`
    if [[ $? != 0 ]] || [ -z "$linux_version" ]
    then
        do_fatal "Не удалось определить версию целевого ядра ОС"
    else
        target_kern_ver=${linux_version}-${img_arch}
        echo "Версия целевого ядра $target_kern_ver" >> $INST_LOG
    fi

    # mcstbug 142509 + rm18485
    host_kern_ver=$(echo $(uname -r)) 2>>$INST_LOG

    local kern_arch=$(echo $host_kern_ver | awk -F '-' '{print $NF}')
    if [[ "$kern_arch" != "amd64" ]] && [[ "$kern_arch" != "x86" ]]
    then
	local new_kern_arch=$(arch_convert_e2k ${kern_arch})
	if [[ $? == 0 ]]
	then
	    host_kern_ver=$(echo $host_kern_ver | sed "s/${kern_arch}/${new_kern_arch}/g")
	fi
    fi

    if [[ $? != 0 ]] || [ -z "$host_kern_ver" ]
    then
        do_fatal "Не удалось определить версию хостового ядра ОС"
    else
        echo "Версия хостового ядра $host_kern_ver" >> $INST_LOG
    fi

    if [[ "$target_kern_ver" != "$host_kern_ver" ]]
    then
        set_virt_mode
    fi
}

set_virt_mode()
{
        vm_flg=1
        echo "Включен режим установки виртуальной машины" >> $INST_LOG
}

check_cross_inst()
{
   # Проверка монтирования корневой ФС инструментального ВК
   # исключая rootfs в инсталляционном initrd
   mount | grep -v rootfs | grep ' / ' > /dev/null
   if [[ $? == 0 ]]
   then
    cross_flg=1
    echo "Обнаружен примонтированный корневой раздел" >> $INST_LOG
    BACKTITLE="Установка ОС Эльбрус в кросс режиме"
   fi

   # В кроссе по сети не устанавливаем
   if [[ $cross_flg == 1 ]] && [[ $network_flg == 1 ]]
   then
    msg="\n   Установка по сети в кросс режиме не поддерживается"
    dialog_msgbox "$msg" 7 60
    exit 0
   fi
}

# Определение целевой архитектуры из установочного диска
check_img_arch()
{
    img_arch=`cat ${SOURCE_DIR}/INFO | grep 'arch=' | awk -F'=' '{print $NF}' 2>>$INST_LOG`
    if [[ $? != 0 ]] || [ -z "$img_arch" ]
    then
        echo "Не удалось определить архитектуру установочного образа" >> $INST_LOG
        do_fatal
    fi
}

set_arch()
{
    # Если процедура уже вызывалась
    [[ -n "$cur_arch" ]] && return 0

    cur_arch=`uname -m`

    if [ -z "$cur_arch" ]
    then
        echo "Не удалось определить архитектуру ВК" >> $INST_LOG
        do_fatal
    fi

    if [[ $cur_arch == *e90s* || $cur_arch == *sparc64* ]]
    then cur_arch=e90s

    elif [[ $cur_arch == *e90* || $cur_arch == *sparc* ]]
    then cur_arch=e90

    elif [[ $cur_arch == *e2k* ]]
    then
        local cur_processor=$(uname -p)
        cur_arch=$(arch_convert_e2k ${cur_processor})
        if [[ $? != 0 ]]
        then
            dialog_msgbox "\n   Ошибка задания архитектуры в set_arch() "  7 50
            do_fatal
        fi

    elif [[ $cur_arch == *i*86* || $cur_arch == x86 ]]
    then cur_arch="x86"

    elif [[ $cur_arch == x86_64 || $cur_arch == amd64 ]]
    then cur_arch="amd64"

    else cur_arch="unknown"
    fi

    check_img_arch

   # Проверка задания сетевой установки
   # TODO может перенести отсюда в init_prog ?
   if [[ $network_flg == 1 ]]
      then
           img_arch=$cur_arch
           BACKTITLE="Сетевая установка ОС Эльбрус ${img_arch}"
           return 0
   fi

   # Проверка соответствия архитектуры хоста и установочного диска
   if [[ $cur_arch != $img_arch ]]
   then
       cross_flg=1
   fi

   if [[ $cross_flg == 1 ]] && [[ $img_arch == x86* ]]
   then
#        msg="Предупреждение: кросс установка загрузчика grub для $img_arch не поддержана!\nУстановка создаёт окружение для бинарного транслятора уровня приложений (RTC)"
        msg="Кросс установка загрузчика grub для $img_arch не поддержана!\n\nУстановка создаёт окружение для бинарного транслятора\nуровня приложений (RTC)."
        if [[ $dialog_min == 0 ]]
        then
            $DIALOG $D_OPT  --backtitle "$BACKTITLE" --yes-label 'Продолжить' --no-label 'Прервать' --title "$TITLE"  --yesno "$msg" 8 65
            [[ $? != 0 ]] && exit 0
        else
            dialog_msgbox "$msg"
        fi
   fi
}

arch_convert_e2k()
{
    if [ -z "$1" ]
    then
        dialog_msgbox "\n     Ошибка: пустой аргумент в arch_convert()" 7 55
        return 1
    fi

    local old_arch=$1
    local new_arch

    case $old_arch in
    "E1C" |"e1c" )     new_arch="e2k-1c"  ;;
    "E1C+"|"e1cp")     new_arch="e2k-1c"  ;;
    "E2S" |"e2s" )     new_arch="e2k-4c"  ;;
    "E8C" |"e8c" )     new_arch="e2k-8c"  ;;
    "E8C2"|"e8c2")     new_arch="e2k-8c2" ;;
    "E2C3"|"e2c3")     new_arch="e2k-2c3" ;;
    "E12C"|"e12c")     new_arch="e2k-12c" ;;
    "E16C"|"e16c")     new_arch="e2k-16c" ;;
    "E2K-VIRT"   )     new_arch=$img_arch ;;  # virtual machine
    *)  echo "Неподдержанный тип процессора $old_arch" >> $INST_LOG; return 1 ;;
    esac

    echo $new_arch
    return 0
}

set_backtitle()
{
    BACKTITLE="Установка ОС Эльбрус"

    [ -n ${distr_vers} ] && BACKTITLE="$BACKTITLE v${distr_vers}"

    if [[ $cross_flg == 1 ]]
        then BACKTITLE="$BACKTITLE в кросс режиме ${cur_arch} -> ${img_arch}"
        else BACKTITLE="$BACKTITLE ${img_arch}"
    fi

    if [[ $vm_flg == 1 ]]
    then
        BACKTITLE="$BACKTITLE (на виртуальную машину)"
    else
        [ -n "${mb_name}" ] && BACKTITLE="$BACKTITLE ${mb_name}"
    fi

    [[ $network_flg == 1 ]] && BACKTITLE="$BACKTITLE $server_ip -> $local_ip"

    TITLE=$DEF_TITLE
}

cross_warn()
{
    if [[ $cross_flg == 1 ]]
    then
        msg="\n     Установка ОС Эльбрус в кросс режиме\n           хост ${cur_arch} -> цель ${img_arch}"
        dialog_msgbox "$msg" 7 50
    fi
}

version()
{
    if [ -f $INST_DIR/VERSION ]
    then
        cat $INST_DIR/VERSION
    else
        echo "Не удалось определить версию инсталлятора"
    fi
}

usage()
{
    echo "Использование: $0 [-$OPTIONS]"
    echo
    echo "   -h        (help)     помощь"
    echo "   -v        (version)  версия инсталлятора"
    echo "   -N        (network)  инсталляция по сети"
    echo "   -i src    (input)    инсталляционное устройство, директория или iso-образ"
    echo "   -o tgt    (output)   целевое устройство или директория для инсталляции"
    echo "   -m        (minimize) установка без диалога"
    echo "   -M        (mute)     установка без сообщений, только лог"
    echo "   -R        (no repo)  НЕ копировать репозиторий на целевой диск"
    echo "                        по умолчанию копируется"
    echo "   -p file   (config file)  конфигурация + разметка диска"
    echo "             пример $INST_DIR/examples/install.cfg"
    echo "   -l file   (package list) список пакетов для установки"
    echo "             пример $INST_DIR/examples/package.list"
    echo "   -g        (graphic)  загрузка целевой ОС в графическом режиме"
    echo "             (требуется установка X-сервера)"
    echo "   -Z serial задать вывод на последовательный порт при кросс-установке"
    echo "             например: ttyS1,115200"
    echo
    echo "Дополнительные опции для инсталляции по сети:"
    echo "   -e iface  (ethernet) интерфейс"
    echo "   -L ip     (local)    ip локальный"
    echo "   -S ip     (server)   ip сервера"
    echo
    echo "Отладочные опции:"
    echo "   -D        (debug) отладка: set -vx"
    echo "   -f        (force) игнорировать ошибки установки dpkg и apt"
    echo
    echo "Примеры:"
    echo "   задание установочного источника (устройства/директории)"
    echo "       $EINSTNAME -i <src>"
    echo
    echo "   отладка"
    echo "       $EINSTNAME -D 2>debug_log.txt"
}

opt_prog()
{
    while getopts $OPTIONS opt
    do
        case $opt in
        h ) usage; exit 0;;
        v ) version; exit 0;;
        N ) network_flg=1;;
        e ) eth="$OPTARG"; network_flg=1;;
        L ) local_ip=$OPTARG; type="static"; network_flg=1;;
        S ) server_ip=$OPTARG; network_flg=1;;
        i ) source_arg=$OPTARG;;
        o ) target_arg=$OPTARG;;
        n ) true;;          # осталось, для поддержки тонких клиентов - игнорируем
        R ) copy_repo_flg=0;;
        m ) dialog_min=1;;
        M ) dialog_min=2;;
        D ) debug_flg=1; set -vx;;
        p ) CONF_FILE=$OPTARG;;
        l ) PACKAGE_LIST=$OPTARG;;
        f ) force_flg=1;;
        g ) graphic_flg=1;;
        Z ) serial_port=$OPTARG;;
        * ) echo "Для вызова справки выполните $EINSTNAME -h"; exit 1;;
        esac
    done

    shift $((OPTIND-1))
    if [[ $# != 0 ]]
    then
        echo "Аргументы не нужны"
        exit 1
    fi
}

cmd_flag_check()
{
    grep 'SERVER_IP=' $cmdline > /dev/null
    if [[ $? = 0 ]]
    then
        network_flg=1
        echo "Задан SERVER_IP в cmdline - сетевая установка" >> $INST_LOG
    fi

    grep 'AUTO_NET_INST' $cmdline > /dev/null
    if [[ $? = 0 ]]
    then
        auto_net_flg=1
        dialog_min=1
        network_flg=1
        echo "Включен режим автоматической сетевой установки без диалога" >> $INST_LOG
    fi

    grep 'INST_DEBUG' $cmdline > /dev/null
    if [[ $? = 0 ]]
    then
        debug_flg=1; set -vx
        echo "Включен режим отладки" >> $INST_LOG
        # поток ошибок в дебажный лог
        exec 2>>$DEBUG_LOG
    fi

    grep 'INST_VM' /proc/cmdline > /dev/null
    if [[ $? = 0 ]]
    then
        set_virt_mode
    fi
}

remote_log_init()
{
    if [[ $network_flg == 1 ]]
    then
        REMOTE_LOG="${REMOTE_LOG_DIR}/${local_ip}_state"
        mkdir $REMOTE_LOG_DIR

        msg="\n    Монтирование служебных директорий сервера..."
        dialog_infobox "$msg"
#        mount -v -n -o rw,nolock,vers=3 $server_ip:/inst.server/client_state  ${REMOTE_LOG_DIR} >> $INST_LOG 2>&1
        mount -v -n -o rw,nolock $server_ip:/inst.server/client_state  ${REMOTE_LOG_DIR} >> $INST_LOG 2>&1
        [[ $? != 0 ]] && do_fatal

#        if [[ $auto_net_flg == 1 ]]
#        then
            msg="\n      Инициализация удаленного логирования..."
            dialog_infobox "$msg"

            ### Удаляем предыдущие логи
            rm -f ${REMOTE_LOG_DIR}/${local_ip}_state
            rm -f ${REMOTE_LOG_DIR}/${local_ip}_full_log
            rm -f ${REMOTE_LOG_DIR}/${local_ip}_error_log
            rm -f ${REMOTE_LOG_DIR}/${local_ip}_reboot      # признак перезагрузки

            cp $INST_LOG $REMOTE_LOG
            if [[ $? != 0 ]]
            then
                msg="\n    Ошибка создания $REMOTE_LOG"
                dialog_infobox "$msg"
                do_fatal
            fi
#        fi
    fi
}

#################################################
# Выбор режима установки
# обычая/сетевая
#################################################
select_install_variant ()
{
    while true; do
        rm -f $tmp
        $DIALOG $D_OPT --backtitle "$BACKTITLE" --title "$TITLE" \
        --ok-label "Выбрать" --cancel-label "Прервать" \
        --output-fd 5 --radiolist "\n                  Выберите вариант установки: \n " 13 70 3 \
        "1" "Установка по умолчанию" on \
        "2" "Установка по сети" off 5> $tmp
        ret=$?
        [ $ret -eq 0 -a -s $tmp ] && break
        [ $ret -eq -1 -o $ret -eq 1 ] && return 1
    done

    if [[ `cat $tmp` == 2 ]]
    then
        network_flg=1
        echo -e "\nВыбран вариант установки по сети" >> $INST_LOG
    else
        echo -e "\nВыбран вариант установки по умолчанию" >> $INST_LOG
    fi

    return 0
}

check_install_conf_on_source()
{
    # возврат 1 устанавливает начальный режим диалога
    [ -n "$CONF_FILE" ] && return 1

    if [ -f "${SOURCE_DIR}/install.cfg" ]
    then
        echo "Обнаружен конфигурационный файл $CONF_FILE" >> $INST_LOG
        CONF_FILE="${SOURCE_DIR}/install.cfg"

        set_dialog_mode
        return $?
    else
        echo "Конфигурационный файл в корне источника не обнаружен " >> $INST_LOG
        return 1
    fi
    return 0
}

set_dialog_mode()
{
    [[ $dialog_min != 0 ]] && return

    local silent_mode=
    silent_mode=$(cat $CONF_FILE | awk -F'#' '{print $1}' | grep -E "silent_mode[ \t]{0,}=" | awk -F'=' '{print $2}' | sed -e 's/^[ \t]*//;s/[ \t]*$//')

    if [ -n "$silent_mode" ] && [[ $silent_mode == 1 ]]
    then
        dialog_min=1
        dialog_infobox "\n      Задан режим установки без диалога"
        return 0
    elif [ -n "$silent_mode" ] && [[ $silent_mode == 0 ]]
    then
        dialog_min=0
        return 0
    else
        echo "silent_mode не задан в $CONF_FILE" >> $INST_LOG
        return 1
    fi
}

check_package_list_on_source()
{
    if [ -z "$PACKAGE_LIST" ] && [ -f "${SOURCE_DIR}/package.list" ]
    then
        PACKAGE_LIST="${SOURCE_DIR}/package.list"

        if [[ $dialog_min == 0 ]]
        then
            # Зададим вопрос
            $DIALOG $D_OPT --backtitle "$BACKTITLE" --title "$TITLE" --ok-label "Да" --no-label "Нет" \
            --yesno "\n    Установка пакетов по списку package.list\n               в корне источника?" 7 55
            ret=$?
            if [[ $ret == 1 ]]    # cancel button
            then
                PACKAGE_LIST=
                package_list=
                return 1
            else
                set_package_list
            fi
        else # dialog_min == 1
            dialog_infobox "Установка пакетов по списку package.list в корне источника"
            set_package_list
        fi
    fi
    return 0
}


check_disk2_iso_on_source()
{
    # второй диск опционально
    # формат имени образа el-6.0-rc1-e8c-disk2.iso
    if [ -f "${SOURCE_DIR}/el-${distr_vers}-${img_arch}-disk2.iso" ]
    then
        DISK2_ISO="${SOURCE_DIR}/el-${distr_vers}-${img_arch}-disk2.iso"
    elif [ -f "${SOURCE_DIR}/disk2.${img_arch}.iso" ]
    then
        DISK2_ISO="${SOURCE_DIR}/disk2.${img_arch}.iso"
    else
        echo "ISO-образ второго диска не найден" >> $INST_LOG
        DISK2_ISO=
        return 1
    fi

    echo "Найден образ второго диска $DISK2_ISO в корне источника" >> $INST_LOG
    return 0
}

set_package_list()
{
    # убираем комменты, переносы, пробелы и табуляции
    package_list=$(cat $PACKAGE_LIST | awk -F'#' '{print $1}' | sed '/^$/d' | tr '\n' ' ' | sed -e 's/^[ \t]*//;s/[ \t]*$//')

    if [ -z "$package_list" ] || [ ! `echo "$package_list" | wc -w` -gt 0 ]
    then
        dialog_msgbox "\n     Ошибка! Пустой список пакетов $PACKAGE_LIST" 7 60
        do_fatal
    fi

    echo "Задан список пакетов для установки $PACKAGE_LIST:" >> $INST_LOG
    echo $package_list >> $INST_LOG
    echo >> $INST_LOG

    if [ -n $DISK2_ISO ]
    then
        msg="Используется $DISK2_ISO"
        dialog_infobox "$msg" 7 60
    fi

    return 0
}


get_graphic_mode_from_conf_file()
{
    # загрузка целевой ОС в графику
    if [ -n "$CONF_FILE" ]
    then
        local gm=$(awk -F'#' '{print $1}' $CONF_FILE | grep -E "graphic_mode=" | awk -F'=' '{print $2}'| sed -e 's/^[ \t]*//;s/[ \t]*$//')
        if [[ $gm == 1 ]]
        then
            graphic_flg=1
        elif [[ $gm == 0 ]] && [[ $graphic_flg == 1 ]]
        then
            dialog_msgbox "\nОшибка: одновременно задан графический режим опцией или в меню\nи выключен в конфигурационном файле $CONF_FILE" 8 60
        fi
    fi
}

check_args()
{
    # обновление кэша динамических библиотек
    [[ $cross_flg == 0 ]] && ldconfig &>>$INST_LOG

    # Проверка опции -p
    if [ -n "$CONF_FILE" ] && [ ! -f $CONF_FILE ]
    then
        msg="\nФайл конфигурации не найден:\n$CONF_FILE"
        dialog_msgbox "$msg" 8 50
        do_fatal
    elif [ -n "$CONF_FILE" ]
    then
        echo "Задан конфигурационный файл $CONF_FILE" >> $INST_LOG
        set_dialog_mode
    fi

    # Проверка опции -l
    if [ -n "$PACKAGE_LIST" ] && [ ! -f $PACKAGE_LIST ]
    then
        msg="\nФайл со списком пакетов не найден:\n$PACKAGE_LIST"
        dialog_msgbox "$msg" 8 50
        do_fatal
    elif [ -n "$PACKAGE_LIST" ]
    then
        set_package_list
    fi

    ### Проверка на корректность ###
    if [[ -n "$serial_port" ]]
    then
        echo $serial_port | grep -o -E 'ttyS[0-9]{1,},[0-9]{1,}' &>/dev/null
        ret=$?
        if [[ $ret != 0 ]]
        then
            msg="\nОшибка: некорректно задан последовательный порт \"$serial_port\""
            dialog_msgbox "$msg" 8 60
            do_fatal
        fi
    fi

    # Дополнение до полного пути
    [[ -n $source_arg && $source_arg != /* ]] && source_arg=$CWD/$source_arg
    [[ -n $target_arg && $target_arg != /* ]] && target_arg=$CWD/$target_arg

}

mount_source_by_uuid()
{
    if [ -z "$source_arg" ] && [ -z "$SOURCE_DEV" ] && [[ $cross_flg == 0 ]] && [[ $network_flg == 0 ]]
    then
        local prev_dialog_min=$dialog_min
        dialog_min=2
        find_in_dev_by_uuid
        if [[ $? == 0 ]]; then
            # монтирование и проверка образа
            # проверка наличия конфигов для автоматической бездиалоговой установки
            in_dev_mount
            if [[ $? != 0 ]]; then
                dialog_min=$prev_dialog_min
                return 1
            fi

            in_dev_check
            if [[ $? != 0 ]]; then
                dialog_min=$prev_dialog_min
                return 1
            fi

            set_arch

            # без конфига или параметра silent_mode в нём возвращаем заданный режим диалога
            check_install_conf_on_source || dialog_min=$prev_dialog_min

            check_package_list_on_source

            # TODO задавать disk2 опцией ?
            check_disk2_iso_on_source
        else # не нашли / не смонтировали источник по uuid
            dialog_min=$prev_dialog_min # продолжаем в обычном режиме
        fi
    fi
}

start_dialog()
{
    if [[ $dialog_min == 0 ]]
    then
        if [[ $network_flg == 1 ]] || [ -n "$source_arg" ] || [[ $cross_flg == 1 ]] || [[ $source_mounted == 1 ]]
        then
            $DIALOG $D_OPT --backtitle "$BACKTITLE" --yes-label 'Начать' --no-label 'Прервать' --title "$TITLE" \
                    --yesno "\n                   Начать установку?" 7 60
            if [[ $? != 0 ]]; then cleanup; exit 0; fi
        else
            select_install_variant
            if [[ $? != 0 ]]; then cleanup; exit 0; fi
        fi
   fi
}

source_init()
{
    if [[ $source_mounted == 1 ]]
    then
        true # заглушка
    elif [[ $network_flg == 1 ]]
    then
        BACKTITLE="$BACKTITLE по сети"
        network_init
        remote_log_init

        # чтение списка пакетов
        if [ -z "$PACKAGE_LIST" ] && [ -f "${REMOTE_LOG_DIR}/conf/package.list" ]
        then
            PACKAGE_LIST=${REMOTE_LOG_DIR}/conf/package.list
            set_package_list
        fi

        if [[ -z $source_arg ]]
        then
            source_arg="$server_ip:/inst.server/input"
        fi
        source_type=device
        SOURCE_DEV=$source_arg
        SOURCE_DIR=

    elif [[ -z $source_arg ]]
    then
        source_type=default
        SOURCE_DEV=
        SOURCE_DIR=
    elif [[ $source_arg == *.iso ]]
    then
        source_type=iso
        SOURCE_DEV=$source_arg
        SOURCE_DIR=

    elif [[ $source_arg == /dev/* ]]
    then
        source_type=device
        SOURCE_DEV=$source_arg
        SOURCE_DIR=

    elif [[ -d $source_arg ]]
    then
        source_type=directory
        SOURCE_DEV=$source_arg
        SOURCE_DIR=$source_arg
    else
        echo "Нет доступа к $source_arg" >> $INST_LOG
        do_fatal
    fi
}

target_init()
{
    if [[ -z "$target_arg" ]]
    then
        target_type=device
        TARGET_DEV=
        PART_PREFIX=
        TARGET_DIR=$TARGET_DIR_DEF
    elif [[ $target_arg == /dev/* ]]
    then
        target_type=device
        TARGET_DEV=$target_arg
        TARGET_DIR=$TARGET_DIR_DEF
    else
        target_type=directory
        TARGET_DEV=$target_arg
        PART_PREFIX=
        TARGET_DIR=$target_arg
    fi

    [[ "$TARGET_DEV" == *nvme* ]] && PART_PREFIX="p"

    # Уделение / в конце строки
    TARGET_DIR=`echo $TARGET_DIR | sed -E 's%/$%%g'`
}

clean_target_dir()
{
    if [ "$target_type" == directory ]
       then
        # Есть такая директория и не пустая?
        if [[ -d $TARGET_DIR ]] && [ "`ls -A $TARGET_DIR`" ]
        then
            if [[ $dialog_min == 0 ]]
            then
                while true; do
                    # Зададим вопрос
                    $DIALOG $D_OPT --title "Установка ОС" --ok-label "Да" --cancel-label "Выход" \
                            --extra-button --extra-label "Очистить" \
                    --yesno "\n    Целевая директория $TARGET_DIR не пустая.\n               Продолжить без очистки?" 8 60
                    ret=$?
                    if [[ $ret == 0 ]]    # Да
                    then
                        break
                    elif [[ $ret == 1 ]]    # cancel button
                    then
                        exit 0
                    elif [[ $ret == $DIALOG_ESC ]]    # ESC button
                    then
                        do_exit
                        continue
                    elif [[ $ret == 3 ]]    # extra button / очистить
                    then
                        dialog_infobox "\nОчистка ${TARGET_DIR}..." 5 50
                        rm -rf ${TARGET_DIR}/* &>> $INST_LOG
                        break
                    elif [[ $ret != 0 ]]
                    then
                        msg="\n      Ошибка диалога"
                        dialog_msgbox "$msg" 7 40
                        exit 1
                    fi
                done
            else # $dialog_min != 0
                dialog_infobox "ВНИМАНИЕ! Целевая директория $TARGET_DIR не пустая!\nБудет очищена через 10 секунд. Для отмены нажмите Ctrl+C"
                timeout=10
                while [ $timeout -gt 0 ]
                do
                    [[ $dialog_min != 2 ]] && echo -n '. '
                    timeout=$(($timeout - 1))
                    sleep 1
                done
                dialog_infobox "Очистка $TARGET_DIR..."
                rm -rf ${TARGET_DIR}/*
            fi
        fi

        mkdir -p $TARGET_DIR >> $INST_LOG 2>&1
        [[ $? != 0 ]] && do_fatal
    fi

    INSTALLED_LIST=${TARGET_DIR}/var/log/einst_package.list
    return 0
}

dev_select()
{
    $DIALOG $D_OPT --backtitle "$BACKTITLE" --title "$TITLE" \
    --infobox "\n             Поиск запоминающих устройств..." 5 60

    is_first=
    n_count=0

    rm -f $DEV_LOG
    rm -f $DEV_SELECT_DIALOG

    # Источник исключить из выбора если задан
    if [ -n ${SOURCE_DEV} ] && [[ $1 = "hdd" ]]
    then
        BLOCK_DEV_LIST=`ls /sys/block | grep -v "${SOURCE_DEV#/dev/}"`
    else
        BLOCK_DEV_LIST=`ls /sys/block`
    fi

    [ -z "$BLOCK_DEV_LIST" ] && do_fatal "Блочные устройства не найдены"

    echo "--backtitle \"$BACKTITLE\"  --title \"$TITLE\"" \
    "--ok-label \"Выбрать\" --cancel-label \"Назад\"" \
    "--extra-button --extra-label \"Обновить\"" >> $DEV_SELECT_DIALOG

    if [[ $1 = "cd" ]]
    then
        echo $BLOCK_DEV_LIST | grep -o -E 'sr[0-9]|sd[a-z]|hd[a-z]|scd[0-5]|nvme[0-9]{1,}n[0-9]{1,}|vd[a-z]' >> $DEV_LOG
        echo "--radiolist \"\n        Выберите устройство-источник установки: \n \" 18 60 8 "  >> $DEV_SELECT_DIALOG
    elif [[ $1 = "hdd" ]]
    then
        echo $BLOCK_DEV_LIST | grep -o -E 'sd[a-z]|hd[a-z]|nvme[0-9]{1,}n[0-9]{1,}|vd[a-z]' >> $DEV_LOG
        echo "--radiolist \"\n          Выберите целевое устройство установки\n"            >> $DEV_SELECT_DIALOG
        if [ "$format_mode" == 1 ]; then
            echo "               для разметки по умолчанию:\" 19 60 9 " >> $DEV_SELECT_DIALOG
        elif [ "$format_mode" == 2 ]; then
            echo "                  для ручной разметки:\" 19 60 9 "    >> $DEV_SELECT_DIALOG
        else
            echo "\" 18 60 8 " >> $DEV_SELECT_DIALOG
        fi
    else
        echo "Внутренняя ошибка: dev_select: invalid arg=$1" >> $INST_LOG
        do_fatal
    fi

    is_first=


    for dev in `cat $DEV_LOG`
    do
        rm -f $DD_LOG
        dd if=/dev/$dev of=$DD_LOG bs=1k count=1 >&/dev/null
        if [ -s $DD_LOG ]
        then
            size_b=
            size_gb=
            size_line=
            model=
            num=
            local pci_slot=

#           Вычисление размера диска
            size_b=`blockdev --getsize64 /dev/$dev`
            if [ -z $size_b ]
            then
                echo "Не удалось определить размер устройства /dev/${dev}" >> $INST_LOG
                size_line="unknown"
            else
                let size_gb=$[$size_b/(1024*1024*1024)] # Gb
                if [ $size_gb -ge 10 ]
                then
                    size_line="$size_gb Gb"
                else
                    let size_mb=$[$size_b/(1024*1024)] # Mb
                    size_line="$size_mb Mb"
                fi
            fi

#           Определение модели IDE/SATA, не подходит для USB - ядро грязно ругается на системный вызов
            #hdparm -h > /dev/null 2>&1 # есть ли утилита?
#           [[ $? == 0 ]] && model=`hdparm -I /dev/$dev | grep 'Model Number' | cut -d':' -f2 | sed 's/^ *//;s/ *$//' ` > /dev/null 2>&1

            if [ -f /sys/block/${dev}/device/model ]
            then
                # Удаление опасных для диалога символов
                model=$(cat /sys/block/${dev}/device/model | tr -d \' | tr -d \" | tr -d \` | tr -d '\\' )
            elif [[ $dev == nvme* ]] && [ -f /sys/block/${dev}/device/uevent ]
            then
                pci_slot=$(cat /sys/block/${dev}/device/uevent | grep 'PCI_SLOT_NAME=' | grep -o -E '[0-9]{1,}:[0-9]{1,}\.[0-9]{1,}')
                model=$(lspci -k -s "$pci_slot" | grep 'Subsystem:' | cut -d':' -f2)
                [[ -z "$model" ]] && model="unknown"
            else
                echo "Не удалось определить модель устройства /dev/${dev}" >> $INST_LOG
                model="unknown"
            fi

            echo "\"$dev\" \"$size_line $model\"  off" >> $DEV_SELECT_DIALOG
            let n_count++
        fi
    done

    echo >> $DEV_SELECT_DIALOG

    if [[ $n_count == 0 ]]
    then
        echo "Устройство $1 не обнаружено!!!" >> $INST_LOG
        do_fatal
    else
        $DIALOG $D_OPT --output-fd 5 --file $DEV_SELECT_DIALOG 5> $RESPONSE
    fi
}


################################################
# function mount_fs()
# note: монтирование целевой файловой системы
#################################################
mount_fs()
{
    [[ "$target_type" != "device" ]] && return 0

    TITLE=$DEF_TITLE
    dialog_infobox "\n        Монтирование целевой ФС..." 5 45

    sort_sequence=$(for ((pnum=0; pnum<${#part_mountpoint[@]}; pnum=pnum+1)); \
        do echo $pnum ${part_mountpoint[$pnum]}; done  | grep -v "^[0-9]$" | sort -k 2 | cut -d' ' -f1)

    for pnum in $sort_sequence; do
        [ "${part_mountpoint[$pnum]}" == swap ] && continue
        dialog_infobox "\n      Монтирование целевой ФС (${TARGET_DEV}${part_n[$pnum]})" 5 60
        mkdir -p ${TARGET_DIR}${part_mountpoint[$pnum]}
        mount -n -t "${part_fs_type[$pnum]}" "${TARGET_DEV}${part_n[$pnum]}" "$TARGET_DIR${part_mountpoint[$pnum]}" >> $INST_LOG 2>&1
        if [ $? -ne 0 ]; then
            msg="\nОшибка: не удалось смонтировать ${TARGET_DEV}${part_n[$pnum]}(${part_fs_type[$pnum]})\nв $TARGET_DIR/${part_mountpoint[$pnum]}"
            dialog_msgbox "$msg" 8 50
            return 1
        fi
    done
}

### отмонтирование целевой файловой системы
umount_fs()
{
    [ -z "$TARGET_DEV" ] && return 0
    #cначала находим root (/)
    sort_sequence=$(for ((pnum=0; pnum<${#part_mountpoint[@]}; pnum=pnum+1)); \
                    do echo $pnum ${part_mountpoint[$pnum]}; done | grep -v "^[0-9]$" | sort -k 2 -r | cut -d' ' -f1)

    for pnum in $sort_sequence
    do
        [ "${part_mountpoint[$pnum]}" == swap ] && continue
        [ -z "${part_mountpoint[$pnum]}" ] && continue

        msg="\n     Размонтирование ${part_mountpoint[$pnum]} (${TARGET_DEV}${part_n[$pnum]})"
        dialog_infobox "$msg" 5 60

        if [ "${part_mountpoint[$pnum]}" = "/" ] && [ -d "$TARGET_DIR" ]
        then
            [ -d $TARGET_DIR/mnt/cdrom2 ] && umount -v ${TARGET_DIR}/mnt/cdrom2 &>> $INST_LOG

            [[ $cross_flg == 0 ]] && [ -d ${TARGET_DIR}/lib/modules/`uname -r` ] && umount -v ${TARGET_DIR}/lib/modules/`uname -r` &>> $INST_LOG

            [ -d $TARGET_DIR/sys -o -d $TARGET_DIR/proc -o -d $TARGET_DIR/dev  -o -d $TARGET_DIR/run -o -d $TARGET_DIR/mnt/cdrom ] \
            && umount -lv $TARGET_DIR/sys $TARGET_DIR/proc $TARGET_DIR/dev/pts $TARGET_DIR/dev $TARGET_DIR/mnt/cdrom $TARGET_DIR/run &>>$INST_LOG 2>&1

            [ -d $TARGET_DIR/lib/modules ] && umount -v $TARGET_DIR/lib/modules &>> $INST_LOG 2>&1

            umount -v $TARGET_DIR &>> $INST_LOG 2>&1
            [[ $? != 0 ]] && echo "Не удалось отмонтировать $TARGET_DIR" >> $INST_LOG
        else
            umount -v "$TARGET_DIR/${part_mountpoint[$pnum]}" &>> $INST_LOG 2>&1
            [[ $? != 0 ]] && echo "Не удалось отмонтировать $TARGET_DIR/${part_mountpoint[$pnum]}" >> $INST_LOG
        fi
    done

    return 0
}

cross_boot_conf()
{
#   [[ $cross_flg == 0 ]] && return 0
    [[ $cur_arch == $img_arch ]] && return 0
    [[ $img_arch == x86 || $img_arch == amd64 ]] && return 0

    dialog_infobox "\n      Создание загрузочной конфигурации (boot.conf) ..." 5 60

    ### IMAGE Detect ###
    if [ -n "$default_kernel_meta" ]
    then
        image_package=`dpkg --info ${SOURCE_DIR}/images/debs/${default_kernel_meta}_*.deb | grep 'Depends:' | sed -E 's/ /\n/g; s/,//g' | grep 'linux-image-'`
        [ -n "$image_package" ] && image=`dpkg -c ${SOURCE_DIR}/images/debs/${image_package}_*.deb | grep 'image-' | sed q | awk -F'/' '{print $NF}'`
    else
        image=`ls ${TARGET_DIR}/boot/image-* | grep -v 'nn' | grep -v 'rt' | sed q | sed "s%${TARGET_DIR}/boot/%%" 2>> $INST_LOG`
    fi
    if [ -z "$image" ]
    then
        image=`ls ${TARGET_DIR}/boot/image-* | sed q | sed "s%${TARGET_DIR}/boot/%%" 2>> $INST_LOG`
        if [[ -z "$image" ]]
        then
            msg="\n        Не удалось обнаружить ядро ОС в /boot\n        boot.conf не будет создан"
            dialog_msgbox "$msg" 8 60
            rm -v ${TARGET_DIR}/boot/boot.conf >> $INST_LOG
            return 1
        fi
    fi

    label="linux-`echo $image | sed "s/image-//"`"

    echo "default=$label"   >  ${TARGET_DIR}/boot/boot.conf
    echo "timeout=3"        >> ${TARGET_DIR}/boot/boot.conf
    echo "      "           >> ${TARGET_DIR}/boot/boot.conf
    echo "#########"        >> ${TARGET_DIR}/boot/boot.conf
    echo "      "           >> ${TARGET_DIR}/boot/boot.conf
    echo "label=$label"     >> ${TARGET_DIR}/boot/boot.conf

    ### BOOT PARTITION ###
    pn=0
    if [ -n "$is_root" ] && [ -n "$is_boot" ]
    then
        if [ "$is_root" -eq 1 -a "$is_boot" -eq 0 ]     # Есть рут и нет бута
        then
            pn=$root_id     # root partition number
        elif [ "$is_root" -eq 1 -a "$is_boot" -eq 1 ]     # Есть рут и бут
        then
            pn=$boot_id     # root partition number
        fi
    fi
    echo "  partition=${pn}"    >> ${TARGET_DIR}/boot/boot.conf

    ### IMAGE ###
    if [[ "$is_root" == 1 ]] && [[ "$is_boot" == 0 ]]     # Есть рут и нет бута
    then
        echo "  image=/boot/${image}"   >> ${TARGET_DIR}/boot/boot.conf
    else
        echo "  image=/${image}"        >> ${TARGET_DIR}/boot/boot.conf
    fi

    # генерация modules.dep, начиная с ядра 4.9 не входит в поставку
    if $(depmod -V &>>$INST_LOG)
    then
        DEPMOD=depmod
    else
        DEPMOD=
        msg="\n         Команда depmod не найдена\n         boot.conf не будет создан"
        dialog_msgbox "$msg" 8 50
        rm -v ${TARGET_DIR}/boot/boot.conf >> $INST_LOG
        return 1
    fi

    dialog_infobox "\n       Генерация modules.dep ..." 5 45
    $DEPMOD -b ${TARGET_DIR} ${image##image-}  >> $INST_LOG 2>&1
    if [ ! -f ${TARGET_DIR}/lib/modules/${image##image-}/modules.dep ]
    then
        msg="\n     Не удалось сгенерировать modules.dep\n          boot.conf не будет создан"
        dialog_msgbox "$msg" 8 50
        rm -v ${TARGET_DIR}/boot/boot.conf >> $INST_LOG
        return 1
    fi

    ### INITRD ###
    # Генерим initrd
    msg="\n           Создание initrd ..."
    dialog_infobox "$msg" 5 45

    # опция -t в mkinitramfs поддержана для дистрибутивов >= 5.0
    local mkramfs_opts=

    # версии дистрибутивов обозначаются как "4.0.1" или "4.0-rc5"
    local distr_vers_major=$(echo ${distr_vers} | awk -F'-' '{print $1}')
    local bigest_vers=`echo -e "5.0\n${distr_vers_major}" | sort -V | tail -1` # бОльшая версия в последней строке
    # если в кросе хост интеловский
    if [[ $cur_arch == x86 ]] || [[ $cur_arch == amd64 ]]
    then
        if [ -n "$distr_vers_major" ] && [[ $distr_vers_major == $bigest_vers ]] # т.е. больше или равна 5.0
        then # используем новую опцию
            # подходит для всех целевых архитектур, кроме интела
            mkramfs_opts="-t e2k"
        else # обойти ссылкой на библиотеку
            cd ${TARGET_DIR}/lib64 && ln -s ld-linux.so.2 ld-linux-x86-64.so.2 2>&1 >> $INST_LOG
            cd /
        fi
    fi

    ${TARGET_DIR}/usr/sbin/mkinitramfs $mkramfs_opts -k `echo $image | \
        sed 's/.*image-//'` -r ${TARGET_DIR} -o ${TARGET_DIR}/boot -b ${TARGET_DIR}/usr/busybox >> $INST_LOG 2>&1
    if [[ $? != 0 ]]
    then
        msg="\n        Не удалось создать загрузочный initrd\n        boot.conf не будет создан"
        dialog_msgbox "$msg" 8 60
        rm -v ${TARGET_DIR}/boot/boot.conf >> $INST_LOG
        return 1
    fi

    initrd="initrd-`echo $image | sed 's/.*image-//'`.img"
    if [ ! -f ${TARGET_DIR}/boot/${initrd} ]
    then
        msg="\n      Не удалось обнаружить $initrd\n          boot.conf не будет создан"
        dialog_msgbox "$msg" 8 60
        rm -v ${TARGET_DIR}/boot/boot.conf >> $INST_LOG
        return 1
    fi
    # Есть рут и нет бута
    if [ "$is_root" -eq 1 -a "$is_boot" -eq 0 ] 2>/dev/null    # Ругается на пустые аргументы, но переходит в else
    then
        echo "  initrd=/boot/${initrd}"  >> ${TARGET_DIR}/boot/boot.conf
        # Создаем ссылку на boot.conf в корень
        if [[ ! -e ${TARGET_DIR}/boot.conf ]]
            then ln -s -r ${TARGET_DIR}/boot/boot.conf ${TARGET_DIR}/ &> $ERR_MSG
            [[ $? != 0 ]] && dialog_msgbox "`cat $ERR_MSG`" 7 60
        fi
    else
        echo "  initrd=/${initrd}"  >> ${TARGET_DIR}/boot/boot.conf
    fi

   ### CMDLINE ###
   # cmdline create, root partition detect
    if [ "$target_type" == directory ]
    then
        echo "  cmdline="   >> ${TARGET_DIR}/boot/boot.conf
    else
        if [[ $old_fstab_flg == 1 ]]
        then
            echo "  cmdline=root=${CROSS_TARGET_DEV}${PART_PREFIX}${part_n[$var_root]} console=tty0"  >> ${TARGET_DIR}/boot/boot.conf
        else
            echo "  cmdline=root=UUID=${partition_uuid[$root_id]} console=tty0"  >> ${TARGET_DIR}/boot/boot.conf
        fi
    fi

   return 0
}

# Если порт задан пользователем прописываем в boot.conf
serial_port_to_boot_conf()
{
    if [ -n "$serial_port" ] && [ -f ${TARGET_DIR}/boot/boot.conf ]
    then
        sed -i "/cmdline=/ s/$/ console=${serial_port}/" ${TARGET_DIR}/boot/boot.conf &>>$INST_LOG
        if [[ $? == 0 ]]
        then
            echo "${serial_port} добавлен в boot.conf" >> $INST_LOG
            return 0
        else
            echo "Не удалось добавить ${serial_port} в boot.conf" >> $INST_LOG
            return 1
        fi
    fi
}

print_boot_conf()
{
   echo -e "\n########## boot.conf: ##########" >> $INST_LOG
   cat $TARGET_DIR/boot/boot.conf >> $INST_LOG
   echo -e "####### End of boot.conf ####### \n" >> $INST_LOG
}

edit_boot_conf()
{
    [[ $cmdline_flg == 0 ]] && return 0

    [[ $img_arch == x86 || $img_arch == amd64 ]] && return 0
    [[ $dialog_min != 0 ]] && return 0
    [ ! -f ${TARGET_DIR}/boot/boot.conf ] && return 0


    TITLE="Настройка параметров запуска ядра ОС (boot.conf)"

    $DIALOG $D_OPT --stdout --no-collapse --backtitle "$BACKTITLE" --title "$TITLE" --yes-label "Да" --no-label "Пропустить" \
            --yesno "\n     Настроить параметры запуска ядра ОС вручную?" 7 60
    local ret=$?
    if [[ $ret == 1 ]]    # cancel button
    then
        return 0
    elif [[ $ret != 0 ]]
    then
        dialog_msgbox "            Ошибка диалога"
        return 1
    fi

    local cmdline=
    local cmdline_mod=
    local label=
    local l=
    local label_list=
    local BC=$TARGET_DIR/boot/boot.conf

    # выбор метки для правки, если больше одной
    label_list=$(awk -F'#' '{print $1}' $BC | grep "label=" | sed -e 's/label=//g')
    if [ -z "$label_list" ]
    then
        dialog_msgbox "\n     Ошибка: загрузочные метки не найдены в\n     $BC" 8 60
        return 1
    fi

    # default label
    local def_boot_label=
    def_boot_label=$(awk -F'#' '{print $1}' $BC  | grep 'default=' | awk -F '=' '{print $2}')
    [ -z "$def_boot_label" ] && dialog_msgbox "\n    Не найдена загрузочная метка по умолчанию default=" 7 60

    while true
    do
        TITLE="Настройка параметров запуска ядра ОС (boot.conf)"
        while true
        do
            if [[ $(echo $label_list | wc -w) > 1 ]]
            then
                unset dialog_opt dialog_extra dialog_init

                for l in $label_list
                do
                    if [[ $l == $def_boot_label ]]
                    then
                        dialog_opt+=("$l" "$l" "on")
                    else
                        dialog_opt+=("$l" "$l" "off")
                    fi
                done

                label=$(dialog_call --radiolist "Выберите загрузочную метку для изменения командной строки ядра" "Редактировать" "Готово")
                ret=$?

                if [[ $ret == 0 ]] && [ -n "$label" ]    # корректный выбор
                then
                    break
                elif [[ $ret == 0 ]] && [ -z "$label" ]    # Нажат enter без выбора
                then
                    continue
                elif [[ $ret == 1 ]]    # cancel button
                then
                    echo "Выбор загрузочной метки прерван" >> $INST_LOG
                    return 0
                elif [[ $ret != 0 ]]
                then
                    msg="\n      Ошибка диалога настройки командной строки ядра"
                    dialog_msgbox "$msg" 7 50
                    return $ret
                else
                    echo "как сюда попали" >> $INST_LOG
                    return 1
                fi
            else
                # в списке одна метка
                label=$(echo $label_list | cut -d' ' -f1)
                break
            fi
        done

        cmd_line_num=$(sed -n  "/\<label=${label}\>$/,/cmdline=/=" $BC | tail -1)
        if [ -z "$cmd_line_num" ]
        then
            dialog_msgbox "\n      Не найдена строка cmdline= по метке ${label}" 7 50
            return 1
        fi

        cmdline=$(sed "${cmd_line_num}!d; s%^.*cmdline=%%" $BC)
        ret=$?
        if [[ $ret == 0 ]] && [ -z "$cmdline" ]
        then
            dialog_msgbox "\n      Внимание! Пустая строка cmdline" 7 50
        elif [[ $ret == 1 ]]
        then
            dialog_msgbox "\n  Ошибка чтения cmdline= по метке ${label}" 7 65
            return 1
        fi

        while true
        do
            # правка cmdline
            TITLE="Метка ${label}"
            rm -f $RESPONSE
            $DIALOG $D_OPT --backtitle "$BACKTITLE" --title "$TITLE" --ok-label "Да" --cancel-label "Пропустить" \
                    --extra-button --extra-label "Восстановить" \
                    --output-fd 5 --inputbox "Откорректируйте загрузочную строку ядра ОС" 9 70 "$cmdline" 5>$RESPONSE
            local ret=$?
            if [[ $ret == 3 ]]    # extra
            then
                continue
            elif [[ $ret == 0 ]] && [[ -s $RESPONSE ]]
            then
                cmdline_mod=$(cat $RESPONSE)
                # случайно нажат ввод без изменения строки
                if [[ $cmdline == $cmdline_mod ]]
                then
                    msg="Продолжить без изменений?"
                    unset dialog_opt dialog_extra dialog_init
                    dialog_call --yesno "$msg" "Да" "Редактировать"
                    [[ $? == 0 ]] && break || continue
                else
                    break
                fi
            elif [[ $ret == 0 ]] && [[ ! -s $RESPONSE ]]
            then
                continue
            elif [[ $ret == 1 ]]    # cancel button
            then
                break
            elif [[ $ret != 0 ]]
            then
                dialog_msgbox "            Ошибка диалога"
                return 1
           else
               echo "как сюда попали" >> $INST_LOG
               return 1
            fi
        done

        # модификация boot.conf
        sed -i "${cmd_line_num}s%cmdline=.*%cmdline=${cmdline_mod}%" $BC &>> $INST_LOG
        ret=$?
        if [[ $ret == 0 ]]
        then
            # если одна метка, выходим из диалога
            if [[ $(echo $label_list | wc -w) -le 1 ]]
            then
                break
            # иначе возвращаемся к выбору меток
            else
                continue
            fi
        else
            dialog_msgbox "Ошибка: не удалось модифицировать $BC"
            return 1
        fi
    done

    return 0
}

### сосздание fstab
generate_fstab ()
{
    rm -f $fstab

    #параметры файла fstab (man fstab)
    fstab_dump=
    fstab_fsck=
    fstab_option=

    for ((pnum=0; pnum < ${#part_name[@]}; pnum=pnum+1)); do
        if [  "${part_mountpoint[$pnum]}" == "" -o "${part_mountpoint[$pnum]}" == "spare" ]; then
            continue
        elif [ ${part_mountpoint[$pnum]} == "/" ]; then
            fstab_dump="1"
            fstab_fsck="1"
            fstab_option="defaults"
        elif [ ${part_mountpoint[$pnum]} == "swap" ]; then
            fstab_dump="0"
            fstab_fsck="0"
            fstab_option="pri=-1"   # -1 приоритет не установлен
        else
            fstab_dump="1"
            fstab_fsck="2"
            fstab_option="defaults"
        fi

        if [[ $old_fstab_flg == 1 ]]
        then
            if [[ $cross_flg == 1 ]] && [ -n "$CROSS_TARGET_DEV" ]
                then echo "${CROSS_TARGET_DEV}${part_n[$pnum]}       ${part_mountpoint[$pnum]}      ${part_fstype[$pnum]} $fstab_option   $fstab_dump $fstab_fsck" \
                        >> $fstab
                else echo "${TARGET_DEV}${part_n[$pnum]}       ${part_mountpoint[$pnum]}      ${part_fs_type[$pnum]} $fstab_option   $fstab_dump $fstab_fsck" \
                        >> $fstab
            fi
        else
            echo "UUID=${partition_uuid[$pnum]}       ${part_mountpoint[$pnum]}      ${part_fs_type[$pnum]} $fstab_option   $fstab_dump $fstab_fsck" >> $fstab
        fi
    done

    echo "proc      /proc       proc        defaults 0 0" >> $fstab
    echo "sys       /sys        sysfs       defaults 0 0" >> $fstab
}


find_in_dev_by_uuid()
{
    # читаем uuid загрузочного устройства из bootdata
    local source_uuid=$(grep 'uuid=' /proc/bootdata 2>>$INST_LOG | cut -d'=' -f2  | sed "s/'//g")
    if [[ $? == 0 ]] && [ -n "$source_uuid" ]
    then
        # конвертация в /dev/xxx
        SOURCE_DEV=$(blkid -U $source_uuid 2>>$INST_LOG)
        if [[ $? == 0 ]] && [ -n "$SOURCE_DEV" ]
        then
            # считывание в формате /dev/xxxN или /dev/nvmeNpN
            SOURCE_DEV=$(blkid -U $source_uuid 2>>$INST_LOG)
            if [[ $? == 0 ]] && [ -n "$SOURCE_DEV" ]
            then
                # конвертация в /dev/xxx или /dev/nvmeN
                [[ "$SOURCE_DEV" == *nvme* ]] && PART_PREFIX="p" || PART_PREFIX=
                SOURCE_DEV=$(echo $SOURCE_DEV | sed "s/${PART_PREFIX}[[:digit:]]*$//" 2>>$INST_LOG)

                SOURCE_DIR=
                source_type=device
            else
                msg="\nНе удалось определить устройство по UUID"
                dialog_msgbox "$msg" 8 50
            fi
        else
            msg="\nНе удалось определить устройство по UUID"
            dialog_msgbox "$msg" 8 50
            return 1
        fi
    else
        msg="\nНе удалось прочитать UUID загрузочного устройства из bootdata" >> $INST_LOG
        dialog_msgbox "$msg" 8 60
        return 1
    fi
    return 0
}


in_dev_choose()
{
    TITLE=$DEF_TITLE
    if [[ $dialog_min != 0 ]] && [ -z "$SOURCE_DEV" ]; then
        msg="Ошибка: режим без диалога: не задано входное устройство"
        dialog_infobox "$msg"
        do_fatal
    fi

    while true
    do
        ### Выбор источника вручную в диалоге
        if [[ $source_type = default ]] && [[ $dialog_min != 1 ]]
        then
            while true
            do
                rm -f $RESPONSE
                SOURCE_DEV=
                dev_select $DEV_SELECT_INPUT
                ret=$?
                if [[ $ret == 0 ]] && [[ -s $RESPONSE ]]
                   then break
                elif [[ $ret == $DIALOG_ESC ]]    # ESC button
                then
                    do_exit
                    continue
                elif [[ $ret == 3 ]]    # extra button / обновить
                then
                    continue
                elif [[ $ret == 1 ]]    # cancel button
                then
                    exit 0
                elif [[ $ret != 0 ]]
                then
                    msg="\n      Ошибка диалога in_dev_choose"
                    dialog_msgbox "$msg" 7 50
                    exit 1
                fi
            done
            SOURCE_DEV="/dev/`cat $RESPONSE`"
        fi

        in_dev_mount
        ret=$?
        if [[ $dialog_min == 0 ]] && [[ $ret != 0 ]]
        then
            dialog_msgbox "\n     Не удалось смонтировать $SOURCE_DEV\n     см. $INST_LOG" 8 50 && source_type="default"
            continue
        fi

        in_dev_check
        if [[ $? == 0 ]]
        then
            return 0
        else
            continue
        fi
    done
}

in_dev_check()
{
    if [ ! -d "$SOURCE_DIR/images" ] || [ ! -s "$SOURCE_DIR/INFO" ] || [ -z "`cat ${SOURCE_DIR}/INFO | grep "boot image" 2>>$INST_LOG`" ]
    then
        dialog_msgbox "\n    $SOURCE_DEV не является инсталляционным образом" 7 60
#       [[ $source_type != directory ]] && umount -v $SOURCE_DIR >> $INST_LOG 2>&1
        [[ $source_mounted == 1 ]] && umount -v $SOURCE_DIR >> $INST_LOG 2>&1
        [[ $dialog_min == 1 ]] && do_fatal
        source_type="default"
        return 1
    else
        if [[ -f "$SOURCE_DIR/INFO" ]]
        then
            echo -e "\nИнсталляционный образ:" >> $INST_LOG
            cat "$SOURCE_DIR/INFO" >> $INST_LOG
            echo >> $INST_LOG   # отступ
            distr_vers=$(awk -F'=' '/mcst version/ {print $2}' ${SOURCE_DIR}/INFO)
            if [[ $? != 0 ]] || [ -z "$distr_vers" ]
            then
                dialog_msgbox "\nОшибка: не удалось определить версию дистрибутива из ${SOURCE_DIR}/INFO" 7 70
                return 1
            fi
        else
            echo "Ошибка: не найден файл ${SOURCE_DIR}/INFO" >> $INST_LOG
            return 1
        fi
        return 0
    fi
}


in_dev_mount()
{
    if [ "$source_type" == directory ]
    then
        return 0
    ### Источник выбран вручную или задан как устройство
    elif [[ $source_type = default || $source_type = device || $source_type = iso ]]
    then
        dialog_infobox "\n       Монтирование установочного устройства..." 5 60
        SOURCE_DIR=$SOURCE_DIR_DEF
        umount -v $SOURCE_DIR   >> $INST_LOG 2>&1
        mkdir -p $SOURCE_DIR    >> $INST_LOG 2>&1
        [[ $? != 0 ]] && do_fatal

        if [[ $network_flg == 1 ]]
        then
#            mount -v -n -o ro,nolock,vers=3 $SOURCE_DEV $SOURCE_DIR >> $INST_LOG 2>&1
            mount -v -n -o ro,nolock $SOURCE_DEV $SOURCE_DIR >> $INST_LOG 2>&1
            ret=$?
        elif [[ $source_type = iso ]]
        then
            $(lsmod | grep loop &>/dev/null)
            if [[ $? == 1 ]]
            then
                echo "Загрузка модуля ядра loop" >> $INST_LOG
                modprobe -v loop >> $INST_LOG 2>&1
            fi
            mount -v -n -o ro,loop $SOURCE_DEV $SOURCE_DIR >> $INST_LOG 2>&1
            ret=$?
        else
            mount -v -n -o ro ${SOURCE_DEV}${PART_PREFIX}1 $SOURCE_DIR >> $INST_LOG 2>&1 || mount -v -n -o ro $SOURCE_DEV $SOURCE_DIR >> $INST_LOG 2>&1
            ret=$?
        fi

        [[ $ret == 0 ]] && source_mounted=1
        return $ret
    else
        echo "in_dev_mount: Как мы сюда попали?" >> $INST_LOG
        return 1
    fi
}


### $1 - устройство, например, /dev/sda ###
# return
# 0 - не смонтировано
# 1 - смонтировано
# 2 - не подан аргумент
check_dev_mount()
{
    local dev=
    if [ -n "$1" ]
    then
        dev="$1"
    else
        msg="\nОшибка: пустой аргумент в check_dev_mount()"
        dialog_msgbox "$msg" 10 60
        return 2
    fi

    # Проверка устройства на монтирование
    mount | grep "$dev" > $ERR_MSG 2>&1
    if [[ $? == 0 ]]
    then
        # sed вставляет символы переноса строки \n
        msg="\nУстройство $dev уже смонтировано\n`cat $ERR_MSG | sed -e ':a' -e 'N' -e '$!ba' -e 's/\n/\\\n/g'`"
        dialog_msgbox "$msg" 10 50
        return 1
    fi

    return 0
}

target_dev_select()
{
    [[ "$target_type" != "device" ]] && return 0

    TITLE="Разметка диска"
    local ret=

    # Если устройство задано опцией
    if [ -n "$TARGET_DEV" ]; then
        if [ ! -e "$TARGET_DEV" ] && [ "$target_type" != "directory" ]; then
            dialog_msgbox "Целевое устройство \"$TARGET_DEV\" недоступно или задано неверно"
            do_fatal
        fi
    elif [[ $dialog_min == 1 ]] && [ "$auto_net_flg" -ne 1 ] && [ "$format_mode" -ne 0 ]; then
            dialog_msgbox "Не задано целевое устройство"
            do_fatal
    fi

    # Целевое устройство задано опцией без пользовательской разбивки
    if [ "$format_mode" -eq 1 -o "$format_mode" -eq 2 ] && [ -n "$TARGET_DEV" ]; then
        true
    ### Если задана пользовательская разбивка, берем $TARGET_DEV из файла $CONF_FILE
    elif [ "$format_mode" -eq 0 ] && [ -n "$CONF_FILE" ]; then
        TARGET_DEV_CONF=`cat $CONF_FILE | awk -F'#' '{print $1}' | grep -E "target_dev[ \t]{0,}=" | awk -F'=' '{print $2}'| sed -e 's/^[ \t]*//;s/[ \t]*$//'`

        # Если одновременно заданы целевые устройства по -o и в конфиге
        if [ -n "$TARGET_DEV" ] && [ -n "$TARGET_DEV_CONF" ]
        then
            msg="\n   Ошибка: одновременно задано целевое устройство опцией '-o'\n                 и в конфигурационном файле"
            dialog_msgbox "$msg" 7 70
            exit 1
        elif [ -n "$TARGET_DEV_CONF" ]
        then
            TARGET_DEV=$TARGET_DEV_CONF
        fi

        # Force flag
        if [[ `cat $CONF_FILE | awk -F'#' '{print $1}' | grep -E "force_flg[ \t]{0,}=" | awk -F'=' '{print $2}'| sed -e 's/^[ \t]*//;s/[ \t]*$//'` == 1 ]]
        then
            force_flg=1
            [[ $dialog_min != 0 ]] && dialog_infobox "Задан флаг force"
        fi

    elif [[ $dialog_min != 0 ]] && [ "$format_mode" -eq 1 ] && [ -n "$TARGET_DEV" ]; then
        true
    elif [[ $dialog_min != 0 ]] && [ "$format_mode" -eq 1 ]; then
        ### Автоматический выбор:
        ### - перебираем SATA, IDE устройства
        ### - пытаемся почитать устройства
        ### - выбираем первое устройство с удачным чтением как целевое
           msg="\n            Поиск целевого устройства..."
           dialog_infobox "$msg" 5 60

           BLOCK_DEV_LIST=`ls /sys/block`
           echo $BLOCK_DEV_LIST | grep -o -E 'sd[a-z]|hd[a-z]|nvme[0-9]{1,}n[0-9]{1,}|vd[a-z]' > $DEV_LOG

           for dev_short in `cat $DEV_LOG`
           do
               ### Пробное чтение устройства
               check_device_readable /dev/$dev_short
               if [[ $? == 0 ]]
               then
                   TARGET_DEV="/dev/$dev_short"
                   target_type="device"
                   echo -e "\nЦелевое устройство - /dev/${dev_short}\n" >> $INST_LOG
                   break
               else
                   echo "Не удалось пробное чтение с /dev/${dev_short}" >> $INST_LOG
               fi
           done
    else
        ### Выбор устройства в диалоговом режиме
        while true
        do
            rm -f $RESPONSE
            TARGET_DEV=
            PART_PREFIX=
            dev_select $DEV_SELECT_OUTPUT
            local ret=$?
            if [[ $ret == 0 ]] && [[ -s $RESPONSE ]]
            then
                TARGET_DEV="/dev/`cat $RESPONSE`"

                if [[ "$SOURCE_DEV" = "$TARGET_DEV" ]]
                then
                    msg="\nУстройства не могут совпадать. Выберите другой вариант."
                    dialog_msgbox "$msg" 7 60
                    continue
                else
                    break           # Успешное завершение диалога
                fi
            elif [[ $ret == 3 ]]    # extra button / обновить
            then
                continue
            elif [[ $ret == 1 ]]    # cancel button / назад
            then
                return 33
            elif [[ $ret != 0 ]]
            then
                return $ret
            fi
        done

        if [[ $cross_flg == 1 ]] && [ -z "$CROSS_TARGET_DEV" ] && [[ $dialog_min != 1 ]] && [[ $old_fstab_flg == 1 ]]
        then
            cross_dev_select_dialog
            ret=$?
            [[ $ret != 0 ]] && return $ret
        fi
    fi

    [[ "$TARGET_DEV" == *nvme* ]] && PART_PREFIX="p"

    # Если input == output
    if [[ "$SOURCE_DEV" = "$TARGET_DEV" ]]; then
        msg="\nУстройства не могут совпадать. Для повторного выбора перезапустите программу установки."
        dialog_msgbox "$msg" 7 60
        do_fatal
    fi

    # Если не удалось определить целевое устройство
    if [ -z "$TARGET_DEV" ] || [ ! -e "$TARGET_DEV" ]; then
        dialog_msgbox "Не удалось определить целевое устройство" 7 60
        do_fatal
    fi

    if cat /proc/mounts | grep -q "$TARGET_DEV"; then
        dialog_msgbox "\n              Устройство $TARGET_DEV используется\n                  в системе (примонтировано)" 8 60
	    [[ $auto_net_flg == 1 ]] && do_fatal
        TARGET_DEV=
        PART_PREFIX=
        CONF_FILE=
        return 22
    fi
}

### Пробное чтение устройства
### Аргумент вида /dev/xxx
check_device_readable()
{
    local dev=
    local ret=

    if [ -n "$1" ]
    then
        dev=$1
    else
        echo "\nОшибка: пустой параметр в функции check_device_readable()" >> $INST_LOG
        do_datal
    fi

    dd if=$1 of=$DD_LOG bs=1k count=1 >&/dev/null
    ret=$?
    if [[ $ret == 0 ]] && [ -s $DD_LOG ]
    then
        return 0
    else
        return 1
    fi
}

get_target_uuids()
{
    # Задан старый стиль идентификаций партиций
    [[ $old_fstab_flg == 1 ]] && return 0

    if [[ $target_type = device ]]
    then
        for (( pnum=0 ; pnum<=${#part_name[@]} ; pnum++ )); do
            # bug 119787, иногда хватает label вместо uuid
            [ "${part_type[$pnum]}" == "extended" ] || \
            [ -z "${part_mountpoint[$pnum]}" ] || \
            [ "${part_name[$pnum]}" == "<free space>" ] && continue
            partition_uuid[$pnum]=`lsblk -n -o UUID ${TARGET_DEV}${part_n[$pnum]} 2>>$INST_LOG`
            if [ -z "${partition_uuid[$pnum]}" ]
            then
                old_fstab_flg=1
                msg="\n         Не удалось определить UUID раздела ${part_n[$pnum]}\n       Используются метки разделов вида /dev/aaaN"
                dialog_msgbox "$msg" 8 60
                return 1
            fi
        done
    fi
}


# Аргумент - имя файла со списком deb-пакетов в директории ${SOURCE_DIR}/images/debs/
dpkg_unpack_list()
{
    DEB_LIST_OUT=${INST_LOCAL}/DEB_LIST_OUT     # Результирующий список
    :> $DEB_LIST_OUT

    if [ -n "$@" ]
    then
        for list_name in $@
        do
            # Проверка наличия списка
            if [ ! -f "${SOURCE_DIR}/images/debs/${list_name}" ]
            then
                msg="\n  Ошибка: отсутствует список пакетов ${list_name}"
                dialog_msgbox "$msg" 7 50
                return 1
            fi

            cat ${SOURCE_DIR}/images/debs/${list_name} >> $DEB_LIST_OUT
        done
    else
        msg="\n  Ошибка: пустой параметр в dpkg_unpack_list"
        dialog_msgbox "$msg" 7 50
        return 1
    fi

    if [[ ! -d $TARGET_DIR ]]
    then
        echo "$TARGET_DIR не является директорией" >> $INST_LOG
        return 1
    fi

    msg="\n              Распаковка компонент ФС..."
    dialog_infobox "$msg"

    # без этого не будет запускаться dpkg
    mkdir -p ${TARGET_DIR}/var/lib/dpkg/updates     || do_fatal
    mkdir -p ${TARGET_DIR}/var/lib/dpkg/info        || do_fatal
    touch ${TARGET_DIR}/var/lib/dpkg/status         || do_fatal
    touch ${TARGET_DIR}/var/lib/dpkg/available      || do_fatal

    echo 0 >$RESULT
    percent=0
    packs_quantity=`cat ${DEB_LIST_OUT} | wc -l`

    # Проверка наличия пакетов
    i=0
    lost_list=
    for pack in `cat ${DEB_LIST_OUT}`; do
        let i++;
        if [ ! -f ${SOURCE_DIR}/images/debs/${pack}.deb ]
        then
            lost_list="$lost_list ${pack}"
        fi
    done

    if [ -n "$lost_list" ]
    then
        msg="Ошибка: не найдены пакеты $lost_list"
        if [[ $dialog_min == 0 ]]
        then
            echo "$msg" >> $INST_LOG

            echo " --title "Ошибка" --yes-label "Продолжить" --no-label "Выход" --defaultno --yesno \"\n    Не найден пакеты:\n\n" >$DIALOG_FILE
            for name in $lost_list; do
                echo "      $name\n" >> $DIALOG_FILE
            done

            echo "\" 20 60" >> $DIALOG_FILE

            $DIALOG $D_OPT --backtitle "$BACKTITLE" --file $DIALOG_FILE

            if [[ $? == 0 ]]    # Yes-button
            then
                continue
            elif [[ $? == 1 ]]  # Cancel-button
            then
                return 2
            fi
        else
            dialog_infobox "$msg"
        fi
    fi

    if [[ $dialog_min == 0 ]]
    then
        i=0
        for pack in `cat ${DEB_LIST_OUT}`; do
            let i++;
            pack_short_name=$(dpkg --info ${SOURCE_DIR}/images/debs/${pack}*.deb | awk '/Package:/ {print $2}')
            # если пакет в списке ненайденых
            [ -n "$lost_list" ] && [[ `echo $lost_list | grep $pack > /dev/null; echo $?` == 0 ]] && continue

                # прогресс-бар
                # unpack ломает ссылки или проблема была в последовательности установки пакетов в кроссе (ядро было первым)
                (dpkg --unpack --force-all --admindir=${TARGET_DIR}/var/lib/dpkg --root=${TARGET_DIR} ${SOURCE_DIR}/images/debs/${pack}*.deb \
                    >> $INST_LOG 2>&1
#                (   dpkg-deb --fsys-tarfile ${SOURCE_DIR}/images/debs/${pack}*.deb | \
#                    tar -C ${TARGET_DIR} -x -h >> $INST_LOG 2>&1
#                    ret=$((${PIPESTATUS[0]}|${PIPESTATUS[1]}))
#                    if [[ $ret != 0 ]]
#                    then
#                        echo 1 >$RESULT
#                        exit 1
#                    fi
#
#                    dpkg-deb --ctrl-tarfile ${SOURCE_DIR}/images/debs/${pack}*.deb | \
#                    tar -C ${TARGET_DIR}/var/lib/dpkg/info -x -h --transform="flags=r;s,^./,${pack_short_name}.," \
#                    >> $INST_LOG 2>&1
#                    ret=$((${PIPESTATUS[0]}|${PIPESTATUS[1]}))
#                    if [[ $ret == 0 ]]
                    if [[ $? == 0 ]]
                    then
                        echo 1 > $progress
                    else
                        echo 1 > $RESULT
                        exit 1
                    fi
                ) | do_progress_dpkg $percent "\n  Распаковка пакета `echo $pack | awk -F'_' '{print $1 "_" $2}'`"
                [[ `cat $RESULT` == 1 ]] && break

                percent=$[($i*100)/$packs_quantity]
                echo "${pack_short_name}" >> $INSTALLED_LIST
        done | $DIALOG $D_OPT  --backtitle "$BACKTITLE" --title "$TITLE" --gauge "\n  Распаковка пакета" 8 70
        [[ `cat $RESULT` == 1 ]] && return 1

    else # $dialog_min != 0
        for pack in `cat ${DEB_LIST_OUT}`; do
            let i++;
            pack_short_name=$(dpkg --info ${SOURCE_DIR}/images/debs/${pack}*.deb | awk '/Package:/ {print $2}')
            # если пакет в списке ненайденых
            [ -n "$lost_list" ] && [[ $force_flg == 1 ]] && [[ `echo $lost_list | grep $pack > /dev/null; echo $?` == 0 ]] && continue

            if [ ! -f ${SOURCE_DIR}/images/debs/${pack}*.deb ]
            then
                dialog_infobox "Ошибка: не найден пакет ${SOURCE_DIR}/images/debs/${pack}.deb" 7 50
                return 2
            fi
            dialog_infobox "Распаковка $pack..."
            dpkg --unpack --force-all --admindir=${TARGET_DIR}/var/lib/dpkg --root=${TARGET_DIR} ${SOURCE_DIR}/images/debs/${pack}*.deb \
                >> $INST_LOG 2>&1
            [[ $? != 0 ]] && return 1
#            dpkg-deb --fsys-tarfile ${SOURCE_DIR}/images/debs/${pack}*.deb | \
#            tar -C ${TARGET_DIR} -x -h >> $INST_LOG 2>&1
#            ret=$((${PIPESTATUS[0]}|${PIPESTATUS[1]}))
#            [[ $ret != 0 ]] && return 1
#
#            dpkg-deb --ctrl-tarfile ${SOURCE_DIR}/images/debs/${pack}*.deb | \
#            tar -C ${TARGET_DIR}/var/lib/dpkg/info -x -h --transform="flags=r;s,^./,${pack_short_name}.," \
#            >> $INST_LOG 2>&1
#            ret=$((${PIPESTATUS[0]}|${PIPESTATUS[1]}))
#            [[ $ret != 0 ]] && return 1
            echo "${pack_short_name}" >> $INSTALLED_LIST
        done
    fi
    [[ `cat $RESULT` == 1 ]] && return 1

    dialog_infobox "\n   Обработка конфигурационных файлов dpkg..." 5 50
    for config in $(find ${TARGET_DIR}/etc -name "*.dpkg-new" ) ; do
        cp $config ${config%%.dpkg-new}
    done

    if [[ $cross_flg == 1 ]] && [[ $cur_arch != $img_arch ]] && [[ ! -e ${TARGET_DIR}/bin/sh ]]
    then
        ln -s -r ${TARGET_DIR}/bin/bash ${TARGET_DIR}/bin/sh >> $INST_LOG 2>&1
        [[ $? != 0 ]] && msg="\n     Не удалось создать ссылку sh -> bash" && dialog_msgbox "$msg" 7 60
    fi

    return 0
}


# Бутстрап ставим всегда
# Нативно выбираем из метапакетов
# В кроссе - минимум(бутстрап+ядро) или все пакеты
install_func()
{
    # ускоряемся с eatmydata
    TITLE=$DEF_TITLE
    dialog_infobox "\n        Подключение libeatmydata ..." 5 50
    local lib_path=`find /lib/ /usr/lib/ -name "libeatmydata.so" 2>/dev/null | sed q`

    if [ -n "$lib_path" ] && [ -e "$lib_path" ]
    then
        export LD_PRELOAD=$lib_path
        echo "Используется $lib_path" >> $INST_LOG
    fi

    if [[ $cur_arch == $img_arch ]]
    then
        # Установка бутстрапа
        dpkg_unpack_list "DEB_LIST_BOOTSTRAP" || do_fatal

        # перенос библиотеки в chroot
        if [ -n "`echo $LD_PRELOAD`" ]
        then
            mkdir -p ${TARGET_DIR}/tmp/                  >> $INST_LOG 2>&1
            cp -v `echo $LD_PRELOAD` ${TARGET_DIR}/tmp/  >> $INST_LOG 2>&1
            chroot_preload=/tmp/`echo $LD_PRELOAD | awk -F'/' '{print $NF}'`
            echo "chroot_preload=$chroot_preload" >> $INST_LOG
        else
            echo "LD_PRELOAD пуст" >> $INST_LOG
        fi

        if [[ $cross_flg == 0 ]] && [[ $vm_flg == 0 ]]
        then
            # установка модулей ядра в ramfs
            dpkg_unpack_to_initrd "linux-image-`uname -r`" || do_fatal
            dpkg_unpack_to_initrd "kmod" || do_fatal
            # сгенерить modules.dep
            depmod -b / `uname -r` &>> $INST_LOG || do_fatal
        fi

        # Настройка пакетов
        tweaking_func

        # Добавление модулей в стартовый initramfs
        add_modules_to_initrd

        # установка пакетов по конфигу
        if [[ $vm_flg == 1 ]]
        then
            echo "Установка второго диска под виртуальной машиной не поддерживается" >> $INST_LOG
        elif [ -n "$package_list" ]
        then
            # TODO убрать костыли с модулями, прикрутить udev
            dialog_infobox "\n        Загрузка модуля igb..." 5 50
            modprobe igb &>> $INST_LOG || do_fatal

            if [ -n "$DISK2_ISO" ]
            then
                dialog_infobox "\n        Загрузка модуля loop..." 5 50
                modprobe loop &>> $INST_LOG || do_fatal

                dialog_infobox "\n       Монтирование ${DISK2_ISO}..." 5 50
                mkdir -p ${TARGET_DIR}/mnt/cdrom2            &>> $INST_LOG || do_fatal
                mount -v $DISK2_ISO ${TARGET_DIR}/mnt/cdrom2 &>> $INST_LOG || do_fatal
            fi

            # пропись сорсов для apt
            echo "deb [trusted=yes] file:/mnt/cdrom/ ./" > ${TARGET_DIR}/etc/apt/sources.list
            [ -n "$DISK2_ISO" ] && echo "deb [trusted=yes] file:/mnt/cdrom2/ ./"  >> ${TARGET_DIR}/etc/apt/sources.list


            apt_inst "$package_list" || do_fatal
            return 0
        fi

        # Установка ядерных компонент
        [ `echo "$out_kern_meta_list" | wc -w` -gt 0 ] && (apt_inst "$out_kern_meta_list" || do_fatal)

        # Установка ядра по умолчанию, если задано
        [ -n "$default_kernel_meta" ] && (apt_inst "$default_kernel_meta" || do_fatal)

        # проброс ядерных модулей в chroot
        # пригодится при любой установке
        if [[ $vm_flg == 0 ]] && [[ $cross_flg == 0 ]] && [ ! -d ${TARGET_DIR}/lib/modules/`uname -r` ]
        then
            mkdir -p ${TARGET_DIR}/lib/modules/`uname -r` &>> $INST_LOG
            mount -v --bind /lib/modules/`uname -r`  ${TARGET_DIR}/lib/modules/`uname -r` &>> $INST_LOG
        fi

        # Установка метапакетов
        [ `echo "$meta_list" | wc -w` -gt 0 ] && (apt_inst "$meta_list" || do_fatal)

    else
        # Кросс
        # Для интела в кросе (окружение RTC) не ставить ядро, т.к. ломается настройка пакетов dpkg --configure
        # по хорошему и grub исключить
        if [[ $minimal_flg == 1 ]]
        then
            # Минимальная установка
            if [[ $img_arch == x86 || $img_arch == amd64 ]]
            then
                dpkg_unpack_list "DEB_LIST_BOOTSTRAP"             || do_fatal
            else
                dpkg_unpack_list "DEB_LIST_BOOTSTRAP KERN_SIMPLE" || do_fatal
            fi
        else
            # Установка всего (bug 101240)
            if [[ $img_arch == x86 || $img_arch == amd64 ]]
            then
                dpkg_unpack_list "DEB_LIST DEB_LIST_META"             || do_fatal
            else
                dpkg_unpack_list "DEB_LIST DEB_LIST_META KERN_SIMPLE" || do_fatal
            fi
        fi
    fi
}


# Аргумент - список пакетов (метапакетов)
# установка через apt-get
apt_inst()
{
    if [[ $cur_arch != $img_arch ]]
    then
        msg="\nУстановка пакетов через apt-get невозможна на ВК нецелевой архитектуры"
        dialog_msgbox "$msg"
        return 1
    fi

    local chroot_status=
    local pack_list=

    if [ `echo $@ | wc -w` -gt 0 ]
    then
        # Удаление переносов, пробелов и табуляций
        pack_list=`echo $@ | tr '\n' ' ' | sed -e 's/^[ \t]*//;s/[ \t]*$//'`
    else
        msg="\n  Ошибка: отсутствуют аргументы у apt_inst()"
        dialog_msgbox "$msg" 7 50
        return 1
    fi

    rm -f $TARGET_DIR/$PIPE_FILE >> $INST_LOG 2>&1

    echo -e "Установка\n ${pack_list}" >> $INST_LOG

    # Обновление /dev
    # редко на хосте пропадают устройства во время установки
    [[ $cross_flg == 1 ]] && udevadm trigger >> $INST_LOG 2>&1

    # монтирование /dev в chroot
    cat /proc/mounts | grep -q "$TARGET_DIR/dev"
    if [[ $? != 0 ]]
    then
        mount --bind /dev $TARGET_DIR/dev >> $INST_LOG 2>&1
        if [[ $? != 0 ]]
        then
            dialog_msgbox "Не удалось смонтировать /dev в $TARGET_DIR" 8 50
            return 1
        fi
    fi

    mount -v --bind $SOURCE_DIR/images/debs $TARGET_DIR/mnt/cdrom >> $INST_LOG 2>&1
    if [[ $? != 0 ]]
    then
        dialog_msgbox "Не удалось смонтировать $SOURCE_DIR/images/debs в $TARGET_DIR/mnt/cdrom" 8 50
        return 1
    fi

cmd="export LC_ALL=C
mkdir -p /var/cache/apt          >> /var/log/einst_apt.log 2>&1
mkdir -p /etc/apt/apt.conf.d/    >> /var/log/einst_apt.log 2>&1
mkdir -p /etc/apt/preferences.d/ >> /var/log/einst_apt.log 2>&1
export LD_PRELOAD=$chroot_preload
apt-get update &>>/var/log/einst_apt.log || exit 1
apt-get -V -fy install -o Dpkg::Options::=\"--force-confnew\" \
-o APT::Get::AllowUnauthenticated=1 \
-o APT::Get::allow-downgrades=1 \
-o APT::Get::allow-remove-essential=1 \
-o APT::Get::allow-change-held-packages=1 \
-o APT::Status-Fd=4 $pack_list \
&>>/var/log/einst_apt.log 4>$PIPE_FILE || exit 1
exit 0"

    if [[ $dialog_min == 0 ]]
    then
        touch $TARGET_DIR/$PIPE_FILE
        chroot $TARGET_DIR /bin/bash -c "$cmd" &>>$INST_LOG | \
        do_progress_apt | \
        $DIALOG $D_OPT --title "Установка метапакетов" --backtitle "$BACKTITLE" \
        --gauge  "\n                 Подготовка к установке" 8 62
        chroot_status=${PIPESTATUS[0]}
    else
        # список метапакетов в строку
        msg="Установка `echo ${pack_list} | sed -e ':a' -e 'N' -e '$!ba' -e 's/\n/ /g'`..."
        dialog_infobox "$msg"
        chroot $TARGET_DIR /bin/bash -c "$cmd" &>>$INST_LOG | \
        do_progress_apt_no_dialog
        chroot_status=${PIPESTATUS[0]}
    fi

    umount -v ${TARGET_DIR}/mnt/cdrom >> $INST_LOG 2>&1

    # добавление лога apt в основной
    echo -e "\n\nЛог работы apt-get:"       >> $INST_LOG
    cat ${TARGET_DIR}/var/log/einst_apt.log >> $INST_LOG

    if [[ $chroot_status != 0 ]] && [[ $dialog_min == 0 ]]
    then
        while true
        do
            $DIALOG $D_OPT --title "Ошибка apt-get" --ok-label "Лог" --cancel-label "Выход" --extra-button --extra-label "Продолжить" \
            --yesno "\n    Процесс установки завершился с ошибкой\n\nЛог - Посмотреть лог apt-get\nВыход - завершить установку\nПродолжить - возобновить установку\n\nВНИМАНИЕ! Продолжение установки не гарантирует положительного результата! " 14 55
            ret=$?
            if [[ $ret == 0 ]]    # ok-button
            then
                # Смотрим лог
                $DIALOG $D_OPT --exit-label "Продолжить" \
                --textbox ${TARGET_DIR}/var/log/einst_apt.log 30 80 2>$ERR_MSG
                if [[ $? != 0 ]]
                then
                    dialog_msgbox "`cat $ERR_MSG`" 7 60
                fi
                continue
            elif [[ $ret == 1 ]]    # cancel-button
            then
                return 1
            elif [[ $ret == 3 ]]    # extra-button Продолжить
            then
                return 0
            fi
        done
    elif [[ $chroot_status != 0 ]] && [[ $force_flg == 1 ]]
    then
        dialog_infobox "Ошибка игнорируется: код возврата chroot (apt-get) $chroot_status"
        return 0
    elif [[ $chroot_status != 0 ]]
    then
        dialog_infobox "Ошибка chroot (apt-get), код возврата $chroot_status"
        [[ $force_flg == 1 ]] && return 0
         return 1
    fi

    # если успех
    # пробелы в переносы
    echo "${pack_list}"| tr ' ' '\n' >> $INSTALLED_LIST

    rm -v ${TARGET_DIR}/var/log/einst_apt.log >> $INST_LOG
}


sync_func()
{
    TITLE=$DEF_TITLE

    if [[ $dialog_min == 0 ]]
        then
            ### Показываем прогресс-бар в диалоге
            echo 0 > $RESULT

        ( sync
                if [[ $? != 0 ]]
                then
                        echo "Ошибка синхронизации" >>$INST_LOG
                        echo 1 > $progress
                        echo 1 > $RESULT
                        exit 1
                else
                        echo 1 > $progress
                        exit 0
                fi
            ) \
            | do_progress_sync "\n           Синхронизация данных на диске" |
            $DIALOG $D_OPT  --backtitle "$BACKTITLE" --title "$TITLE" --gauge "\n           Синхронизация данных на диске" 8 60

                # Показ 100%
            [[ `cat $RESULT` == 0 ]] && \
                echo 100 | $DIALOG $D_OPT  --backtitle "$BACKTITLE" --title "$TITLE" --gauge "\n           Синхронизация данных на диске" 8 60 \
                && sleep 1

        else
            msg="\n           Синхронизация данных на диске..."
            dialog_infobox "$msg"
            sync
    fi
}

tweaking_func()
{
    # Чтобы при первой загрузке клиента произошла настройка при кросс-установке
    # TODO проверить актуальность действий под условием
    if [[ $cur_arch != $img_arch ]]
    then
        touch ${TARGET_DIR}/etc/mcst.start
        ln -s ../init.d/mountkernfs ${TARGET_DIR}/etc/rcsysinit.d/S91mountkernfs >> $INST_LOG 2>&1
        ln -s ../init.d/mountfs ${TARGET_DIR}/etc/rcsysinit.d/S92mountfs         >> $INST_LOG 2>&1
        ln -s ../init.d/sysctl ${TARGET_DIR}/etc/rcsysinit.d/S93sysctl           >> $INST_LOG 2>&1
        ln -s ../init.d/mcst.start ${TARGET_DIR}/etc/rcsysinit.d/S94mcst.start   >> $INST_LOG 2>&1
        return 0
    fi

    # Обновление /dev
    [[ $cross_flg == 1 ]] && udevadm trigger >> $INST_LOG 2>&1

    # монтирование /dev в chroot
    cat /proc/mounts | grep -q "$TARGET_DIR/dev"
    if [[ $? != 0 ]]
    then
        umount -v $TARGET_DIR/dev >> $INST_LOG 2>&1    # подстраховка
        mount --bind /dev $TARGET_DIR/dev >> $INST_LOG 2>&1
        if [[ $? != 0 ]]
        then
            dialog_msgbox "Не удалось смонтировать /dev в $TARGET_DIR"
            return 1
        fi
    fi

# В диалог выводится mcst.start и setup.d
#mount -t devtmpfs none /dev    &>> /var/log/tweaking.log
cmd="export LC_ALL=C
[ ! -e /bin/sh ] && ln -s -r /bin/bash /bin/sh     &>> /var/log/tweaking.log
mkdir -p /dev/pts              &>> /var/log/tweaking.log
mount -t devpts none /dev/pts  &>> /var/log/tweaking.log
/etc/init.d/mountkernfs start  &>> /var/log/tweaking.log
mkdir -p /run/lock/subsys      &>> /var/log/tweaking.log
export LD_PRELOAD=$chroot_preload
/etc/init.d/sysctl start       &>> /var/log/tweaking.log
/etc/init.d/mcst.start start 2>&1 | tee -a /var/log/tweaking.log
for i in /etc/rc.d/setup.d/01-*.sh; do sh -x \$i 2>&1 | tee -a /var/log/tweaking.log; done;"

    if [[ $dialog_min == 0 ]]
    then
        # Диалоговый режим
        chroot $TARGET_DIR /bin/bash -c "$cmd" 2>&1 | tee -a $INST_LOG | \
        $DIALOG $D_OPT --backtitle "$BACKTITLE" --title "Настройка системы" --progressbox 30 90
        res=${PIPESTATUS[0]}    # chroot return code
    else
        # Без диалога
        msg="Выполняется настройка системы..."
        dialog_infobox "$msg"
        chroot $TARGET_DIR /bin/bash -c "$cmd" >>$INST_LOG 2>&1
        res=$?
    fi

    # Обработка ошибок
    if [[ $res != 0 ]]
    then
        msg="\nОшибка, см. $TARGET_DIR/var/log/tweaking.log"
        dialog_msgbox "$msg" 7 50
    fi
}

# Добавление модулей в стартовый initramfs
add_modules_to_initrd()
{
    # Добавляем необходимые для загрузки c target_dev модули в initramfs
    used_modules=$(udevadm info -a -n ${TARGET_DEV} | grep 'DRIVER' | grep -E -o "\".{1,}\"" | tr -d '"')
    for module in $used_modules; do
        if [ $(grep -c "^${module}$" ${TARGET_DIR}/etc/initramfs-tools/modules) -gt 0 ]; then
            continue
        elif [[ $cross_flg == 1 ]] || [[ $vm_flg == 1 ]]; then
            module_exists=$(find ${TARGET_DIR}/lib/modules/ -name "${module}.ko" 2>> $INST_LOG | wc -l)
            if [ $module_exists -eq 0 ]; then
                continue
            fi
        elif ! lsmod | grep -q "^${module}[[:space:]]"; then
            continue
        fi
        echo $module >> ${TARGET_DIR}/etc/initramfs-tools/modules
        echo "Модуль $module добавлен в /etc/initramfs-tools/modules" >> $INST_LOG
    done

    # если корневой раздел в xfs, добавляем модуль
    if $(awk -F '#' '{print $1}' ${TARGET_DIR}/etc/fstab | grep -q '/ [ \t]*xfs'); then
        echo 'xfs' >> ${TARGET_DIR}/etc/initramfs-tools/modules
        echo "Модуль xfs добавлен в /etc/initramfs-tools/modules" >> $INST_LOG
    fi
}

mkinstlocal()
{
    # Если нет рабочей директории
    if [ ! -d $INST_LOCAL ]
    then
        mkdir -p $INST_LOCAL
        [[ $? != 0 ]] && echo "Аварийное завершение работы" && exit 1
    fi
}

rminstlocal()
{
    if [ -f $INST_LOG ]
    then
        rm -f $INST_LOCAL/* 2> /dev/null
    fi
}

cleanup()
{
    cd $CWD

    # Сброс логов на серевер
    if [ -f "${REMOTE_LOG}" ]
    then
        cp $INST_LOG ${REMOTE_LOG_DIR}/${local_ip}_error_log
        [[ -d ${INST_LOCAL}/hdd_image_dir ]]    && umount ${INST_LOCAL}/hdd_image_dir    >> $INST_LOG 2>&1
        [[ -d ${INST_LOCAL}/lintel_image_dir ]] && umount ${INST_LOCAL}/lintel_image_dir >> $INST_LOG 2>&1

        if [ -f "${TARGET_DIR}/var/log/einst_apt.log " ]
        then
            echo -e "\n\nЛог apt-get:\n" >> ${REMOTE_LOG_DIR}/${local_ip}_error_log
            cat ${TARGET_DIR}/var/log/einst_apt.log >> ${REMOTE_LOG_DIR}/${local_ip}_error_log
        fi
    fi

    # TARGET
    if [[ $target_type = device ]]
    then
        if [ -n "$part_count" ];
        then
            umount_fs
        else
            [[ -d $TARGET_DIR/boot ]] && umount $TARGET_DIR/* >> $INST_LOG 2>&1
            [[ -d $TARGET_DIR ]] && umount $TARGET_DIR >> $INST_LOG 2>&1
        fi
    fi

    # SOURCE
        [[ -d $SOURCE_DIR ]] && umount $SOURCE_DIR >> $INST_LOG 2>&1
        [[ -d ${REMOTE_LOG_DIR} ]] && umount ${REMOTE_LOG_DIR} >> $INST_LOG 2>&1
}

crash_msg()
{
    TITLE="Аварийное завершение"
    msg="\n             Завершение работы по сигналу..."
    dialog_infobox "$msg"
}

module_on()
{
   # Ищем строку с нужным модулем для каждого файла со списком модулей
   # Раскомментируем строку соответственно номеру

    m_list="$@"
    str_num=

    for name in `echo ${m_list}` ;
    do
        for module_list in ${TARGET_DIR}/etc/modules-*
        do
            str_num=$(cat $module_list 2>/dev/null | grep -n -x -e "# $name" | cut -d':' -f1)   # Точное совпадение
            if [ -n "$str_num" ]
            then
                sed -i -e "$str_num s/# //g" $module_list
                echo "Включен модуль ядра ${name} в ${module_list}" >> $INST_LOG
            else
                cat $module_list 2>/dev/null | awk -F'#' '{print $1}' | grep -e "$name" > /dev/null 2>&1   # Модуль уже включен?
                [[ $? != 0 ]] && echo "Модуль ядра ${name} не найден в `echo ${module_list} | sed -e "s%${TARGET_DIR}/etc/sysconfig/%%g"`" >> $INST_LOG
            fi
        done
    done

   return 0
}


# Включение консоли на последовательном порту в inittab
# если есть параметр в boot.conf
mk_serial_console()
{
    local s_port_speed=
    local s_port=
    local s_speed=

    default_label=`cat ${TARGET_DIR}/boot/boot.conf | grep 'default=' | awk -F '=' '{print $2}' 2>>$INST_LOG`
    if [ -z "$default_label" ]
    then
        echo "Не найден параметр default= в целевом boot.conf" >>$INST_LOG
        return 1
    fi

    # Задан ли параметр пользователем или ttyS* в целевом boot.conf
    if [ -n "$serial_port" ]
    then
        s_port_speed=$serial_port
    else
        # sed выводит от дефолтной метки до следующей
        s_port_speed=$(cat ${TARGET_DIR}/boot/boot.conf 2>/dev/null | sed -n "/label=${default_label}/,/label=/p" \
                      | grep 'cmdline=' | grep -o -E 'ttyS[0-9]{1,},[0-9]{1,}')
    fi

    if [ -n "$s_port_speed" ]    # Есть совпадение
    then
        s_port=$(echo $s_port_speed| awk -F ',' '{print $1}')
        s_speed=$(echo $s_port_speed| awk -F ',' '{print $2}')

        # Есть ли раскомментирования строка в inittab
        cat ${TARGET_DIR}/etc/inittab | awk -F '#' '{print $1}' | grep $s_port > /dev/null
        if [[ $? == 0 ]]
        then
            echo "Строка с $s_port_speed в целевом inittab уже присутствует"    >> $INST_LOG
            return 0
        else
            # Задаём параметр
            echo "sp:2345:respawn:/sbin/agetty -L ${s_port} ${s_speed}" >> ${TARGET_DIR}/etc/inittab
            echo "${s_port_speed} задан в целевом inittab"  >> $INST_LOG
            return 0
        fi
    fi
}

# arg - путь до deb-пакета
dpkg_unpack_to_initrd()
{
    if [ -n "$1" ]
    then
        local pack_name=$1
    else
        dialog_msgbox "Пустой параметр в dpkg_unpack_to_initrd" 6 50
        return 1
    fi

    # ищем пакет
    local pack_full_path=`ls ${SOURCE_DIR}/images/debs/${pack_name}*.deb | head -1 2>>$INST_LOG`

    if [ -n "$pack_full_path" ]
    then
        dialog_infobox "\nРаспаковка $pack_name в ramfs" 5 60
#       dpkg -x $pack_full_path / >> $INST_LOG 2>&1
        dpkg-deb --fsys-tarfile $pack_full_path | tar -hx -C / -- >> $INST_LOG 2>&1
        return $?
    else
        dialog_infobox "\n$pack_name не найден в ${SOURCE_DIR}/images/debs" 5 60
        return 1
    fi
}

bin_exist()
{
    local bin_find=

    if [ -n "$1" ]
    then
        local bin=$1
    else
        dialog_infobox "\nПустой аргумент в bin_exist()" 5 60
        return 1
    fi

    bin_find=`find /{bin,sbin} /usr/{bin,sbin} -type f -executable -name "$bin" 2>>$INST_LOG`
    if [ $? != 0 ]
    then
        dialog_infobox "\nОшибка в bin_exist()" 5 60
        return 1
    elif [ -n "$bin_find" ]
    then
        return 0
    else
        echo "$bin не найден в ramfs" >> $INST_LOG
        return 1
    fi
}

date_time_dialog()
{
    local ret=

    # В кроссе настраиваем часовой пояс
    if [[ $cross_flg == 1 ]] || [[ $vm_flg == 1 ]]
    then
        TITLE="Установка часового пояса"
        while true
        do
            tz_set
            ret=$?
            [[ $ret == 0 ]] && return 0    # ОK
            [[ $ret == 3 ]] && continue    # назад из вложенного диалога
            [[ $ret != 0 ]] && return $ret # Ошибка
        done
    fi

    TITLE="Установка даты/времени"

    ### При сетевой установке время берется по ntp
    if [[ $network_flg == 1 ]]
    then
        # ntpdate не отрабатывает, пока сервер не засинкался с собой
#       ntpdate -d ${server_ip} >> $INST_LOG 2>&1

        # Конфиг
        echo "server ${server_ip} iburst prefer" > /etc/ntp.conf
        echo "server ${server_ip} iburst trust" > /etc/chrony.conf

        # Безусловный таймаут 15с, затем в бэкграунд
#       ntpd -d -g -w 10 >> $INST_LOG 2>&1
        # Проверяем, что засинкались с сервером
        # * - признак синка
#       ntpq -p 127.0.0.1 >> $INST_LOG 2>&1     # debug
#       ntpq -p 127.0.0.1 | grep "${server_ip}" | grep * > /dev/null 2>&1


        # Версия 2.4 не работает: Could not get root uid/gid
        # Требуется /etc/passwd с записью рута
        bin_exist "chronyd"
        if [ $? != 0 ]
        then
            dpkg_unpack_to_initrd "chrony"
            ret=$?
        else
            ret=0
        fi

        msg="\n         Получение времени и даты по NTP..."
        dialog_infobox "$msg" 5 60

        if [[ $ret == 0 ]]
        then
            echo "root:x:0:0:root:/root:/bin/bash" > /etc/passwd
            chronyd -4 -d -q >> $INST_LOG 2>&1
        else
            # ntpd будет вечно ждать недоступный сервер :(
            ntpd -dgq >> $INST_LOG 2>&1
        fi
        if [[ $? == 0 ]]
        then
            hwclock --systohc >> $INST_LOG 2>&1
            if [[ $? == 0 ]]
            then
                msg="\n       Время и дата установлены\n       `date`"
                dialog_msgbox "$msg" 7 50
#               date --utc >> $INST_LOG 2>&1
                return 0
            else
                msg="\nНе удалось установить дату/время через hwclock"
                dialog_msgbox "$msg" 6 55
                return 0
            fi
        else
            msg="\n      Не удалось получить дату/время по NTP"
            dialog_msgbox "$msg" 6 55
            return 0
        fi
    fi

    # Без диалога не настраивается
    [[ $dialog_min != 0 ]] && return 0

    while true
    do
        date_set
        ret=$?
        [[ $ret != 0 ]] && return $ret     # Ошибка

        while true
        do
            tz_set
            ret=$?
            [[ $ret == 3 ]] && break        # назад
            [[ $ret != 0 ]] && return $ret  # Ошибка
            while true
            do
                time_set
                ret=$?
                [[ $ret == 0 ]] && return 0     # ОK
                [[ $ret == 3 ]] && break        # назад
                if [[ $ret != 0 ]]; then
                    dialog_msgbox "\n            Ошибка установки времени" 6 55
                    return 0
                fi
            done
        done
    done
}

date_set()
{
   local res=

   # установка даты
    dat=$($DIALOG $D_OPT --stdout --backtitle "$BACKTITLE" --title "Установка даты" \
        --ok-label "Да" --cancel-label "Пропустить" --week-start 1 \
        --calendar "Для перемещения курсора между полями используйте клавишу табуляции, для задания параметра - стрелки" 4 60 2>>$INST_LOG)
    res=$?
    # Пропустить
    if [[ $res == 1 ]]
    then
        echo "Установка даты/времени отменена" >> $INST_LOG
        return 0
    elif [[ $res != 0 ]]    # error
    then
        msg="\n    Не удалось установить дату и время"
        dialog_msgbox "$msg" 7 50
        return 1
    else
        return 0
    fi
}

tz_set()
{
    local res=
    local temp_list=
    TZ_DIR_LIST_TEMP="$INST_LOCAL/tz_dir_list_temp"

    ### часовой пояс ###
    tz=

    msg="\n      Сбор информации о часовых поясах..."
    dialog_infobox "$msg" 5 50

    # для кросса надо распаковать пакет с часовыми поясами в левую директорию
    if [[ $cross_flg == 1 ]]
    then
        TZ_UNPACK_DIR=${INST_LOCAL}/zoneinfo
        mkdir -p ${TZ_UNPACK_DIR} &>> $INST_LOG
        dpkg-deb -X ${SOURCE_DIR}/images/debs/zoneinfo*.deb ${TZ_UNPACK_DIR} &>> $INST_LOG
        [[ $? != 0 ]] && do_fatal
    else
        TZ_UNPACK_DIR=
    fi

    # ищем директории
    file -s $(find ${TZ_UNPACK_DIR}/usr/share/zoneinfo/ -type d )| cut -d':' -f1 | sed "s%/usr/share/zoneinfo/% %g"| cut -d' ' -f2 | sort > $TZ_DIR_LIST_TEMP 2>>$INST_LOG
    temp_list=$(grep "^[[:upper:]]" $TZ_DIR_LIST_TEMP)
    echo -n > $TZ_DIR_LIST
    for item in $temp_list
    do
        echo "${TZ_UNPACK_DIR}/usr/share/zoneinfo/$item" >>$TZ_DIR_LIST
    done
    
    if [[ ${PIPESTATUS[0]} != 0 ]]
    then
        tz="UTC"
        tz_dir="Etc"
        msg="\nОшибка чтения директорий часовых поясов, используется UTC"
        dialog_msgbox "$msg" 7 50
        return 1
    fi

    if [[ $dialog_min != 0 ]]
    then
        # TODO часовой пояс на хосте
        return 0
    fi

    while true
    do
        echo "--radiolist \"\nВыберите регион\nEtc - часовые пояса GMT и UTC\nДля поиска можно ввести начальную букву имени\" 30 50 22 " > $DIALOG_FILE

        for dir in `cat $TZ_DIR_LIST | awk -F'/' '{print $NF}'`
        do
            echo "\"$dir\" \"\" off" >> $DIALOG_FILE
        done

        #### Выбор региона ###
        local extra=
        [[ $cross_flg == 0 ]] && extra='--extra-button --extra-label Назад'

        tz_dir=`$DIALOG $D_OPT --stdout --backtitle "$BACKTITLE" --title "Часовой пояс" --ok-label "Да" \
            --cancel-label "Пропустить" ${extra} --file $DIALOG_FILE 2>>$INST_LOG`
        res=$?
        if [[ $res == 0 ]] && [ -z "${tz_dir}" ]    # Нажат enter без выбора
        then
            continue
        elif [[ $res == 1 ]]    # cancel button
        then
            echo "Установка региона отменена, используется UTC" >> $INST_LOG
            tz="UTC"
            tz_dir="Etc"
            return 0
        elif [[ $res == 3 ]]    # назад
        then
            return $res
        elif [[ $res != 0 ]]
        then
            #msg="\n      Ошибка диалога выбора региона"
            #dialog_msgbox "$msg" 7 50
            return $res
        fi

        # в выбранной директории читаем файлы zoneinfo
        file -s $( ls -d -1 `cat $TZ_DIR_LIST | grep $tz_dir | sed q`/* ) | grep 'timezone data' | cut -d':' -f1 | awk -F '/' '{print $NF;}' | sort >$TZ_LIST 2>>$INST_LOG
        if [[ ${PIPESTATUS[0]} != 0 ]]
        then
            tz="UTC"
            tz_dir="Etc"
            msg="\n  Не удалось получить список часовых поясов, используется UTC"
            dialog_msgbox "$msg" 7 65
            return 0
        else
            msg="\n               Подготовка..."
            dialog_infobox "$msg" 5 50

            echo "--radiolist \"\nВыберите часовой пояс\nДля поиска можно ввести начальную букву имени\" 30 50 22 " > $DIALOG_FILE

            for item in `cat $TZ_LIST`
            do
                echo "\"`echo $item|awk -F '/' '{print $NF;}'`\" \"\" off" >> $DIALOG_FILE
            done

            while true
            do
                ### Выбор часового пояса ###
                # tz в виде 'только название файла'
                # безусловное наличие кнопки
                extra="--extra-button --extra-label Назад"
                tz=`$DIALOG $D_OPT --stdout --backtitle "$BACKTITLE" --title "Часовой пояс" --ok-label "Да" \
                    --cancel-label "Пропустить" ${extra} --file $DIALOG_FILE 2>>$INST_LOG`
                res=$?
                if [[ $res == 0 ]] && [ -z "${tz}" ]    # Нажат enter без выбора
                then
                    continue
                elif [[ $res == 0 ]]    # ok
                then
                    return 0
                elif [[ $res == 1 ]]    # cancel button
                then
                    echo "Установка часового пояса отменена, используется UTC" >> $INST_LOG
                    tz="UTC"
                    tz_dir="Etc"
                    return 0
                elif [[ $res == 3 ]]    # extra button - back
                then
                    break
                elif [[ $res != 0 ]]    # error
                then
                    #msg="\n     Ошибка диалога выбора часового пояса"
                    #dialog_msgbox "$msg" 7 50
                    return $res
                fi
            done
        fi
    done

}

time_set()
{
    local res=

    export TZ="`cat $TZ_DIR_LIST | grep $tz_dir | sed q | sed s%/usr/share/zoneinfo/%%g`/$tz"
    echo TZ=$TZ                 >> $INST_LOG

    ### Установка времени ###
    tim=$($DIALOG $D_OPT --stdout --backtitle "$BACKTITLE" --title "Установка времени" \
        --ok-label "Да" --cancel-label "Пропустить" \
        --extra-button --extra-label "Назад" \
        --timebox "Для перемещения курсора между полями используйте клавишу табуляции, для задания параметра - стрелки\n\nЧасовой пояс $TZ" 6 60 2>>$INST_LOG)
    res=$?
    if [[ $res == 1 ]]
    then
        echo "Установка даты/времени отменена" >> $INST_LOG
        return 0
    elif [[ $res == 3 ]]    # назад
    then
        return $res
    elif [[ $res != 0 ]]    # error
    then
#        echo "Ошибка диалога установки времени" >> $INST_LOG
        return $res
    fi

   day=`   echo -n $dat | cut -d'/' -f1`
   month=` echo -n $dat | cut -d'/' -f2`
   year=`  echo -n $dat | cut -d'/' -f3`
   hour=`  echo -n $tim | cut -d':' -f1`
   minute=`echo -n $tim | cut -d':' -f2`
   second=`echo -n $tim | cut -d':' -f3`

   echo -e "\nУстановлена дата/время: " >> $INST_LOG
   date "${month}${day}${hour}${minute}${year}.${second}" 2>&1 >> $INST_LOG
   res=$?
   if [[ $res != 0 ]]
      then
           return $res
      else
           hwclock --systohc >> $INST_LOG 2>&1
           res=$?
           [[ $res != 0 ]] && return $res || return 0
   fi
}

tz_slink()
{
    # mcstbug 137127
    if [ -z "$tz" ] && [ ! -f ${TARGET_DIR}/etc/localtime ] && [ -f ${TARGET_DIR}/usr/share/zoneinfo/Europe/Moscow ]
    then
        ln -sfv ${TARGET_DIR}/usr/share/zoneinfo/Europe/Moscow ${TARGET_DIR}/etc/localtime >> $INST_LOG 2>&1
        ret=$?
    elif [ -z "$tz" ] && [ -f ${TARGET_DIR}/etc/localtime ]
    then
        ret=0
    elif [ -n "$tz" ]
    then
        if [[ $cross_flg == 1 ]]
        then
            ln -sfv `cat $TZ_DIR_LIST | grep $tz_dir | sed q | sed "s%${TZ_UNPACK_DIR}%%"`/$tz \
                ${TARGET_DIR}/etc/localtime >> $INST_LOG 2>&1
            ret=$?
        else
            chroot ${TARGET_DIR} /bin/bash -c "ln -sf `cat $TZ_DIR_LIST |
                   grep $tz_dir | sed q`/$tz /etc/localtime" >> $INST_LOG 2>&1
            ret=$?
        fi
    else
        ret=1
    fi

    if [[ $ret != 0 ]]
    then
        export TZ="UTC"
        msg="\nОшибка установки часового пояса в целевой ФС, используется UTC"
        dialog_msgbox "$msg" 7 55
    else
        echo "Ссылка /etc/localtime установлена" >> $INST_LOG
        ls -l ${TARGET_DIR}/etc/localtime  >> $INST_LOG 2>&1
    fi
}

# 1 аргумент - имя пользователя
password_dialog()
{
    local dialog_ret=
    local ret_code=

    [[ -z "$1" || -z "$2" ]]  && return 0
    local user_var=$1
    local ret_var=$2

    local user=
    local pass=
    local pass2=

    [[ "$1" == root ]] && user="root" || user="${!1}"

    if [[ $cur_arch != $img_arch ]] || [[ $dialog_min != 0 ]]
    then
        [[ "$user" == "root" ]] && show_def_root_pass
        return 0
    fi

    if [[ "$user" == "skip" ]]
    then
        user=
        pass=
        export "$user_var=$user"
        export "$ret_var=$pass"
        return 0
    fi

    [[ -z "$user" ]]  && return 33

    TITLE="Установка пароля для $user"

    if [[ $cur_arch != $img_arch ]] || [[ $dialog_min != 0 ]] || [[ $auto_net_flg == 1 ]]
    then
        pass=
    else
        while true; do
            pass=$($DIALOG $D_OPT   --stdout --backtitle "$BACKTITLE" --title "$TITLE" \
                                  --ok-label "Да" --cancel-label "Пропустить" \
                                  --extra-button --extra-label "Назад" --insecure \
                                  --passwordbox "\n    Введите пароль для пользователя ${user}\n " 11 50 )
            dialog_ret=$?
            if [[ $dialog_ret == 3 ]]    # назад
            then
                pass=
                return 33
            elif [[ $dialog_ret == 1 ]]
            then # Пользователь отказался
                pass=
                ret_code=0
                break
            elif [[ $dialog_ret != 0 ]]
            then
                return $dialog_ret
            fi

            if [ -z "$pass" ]
            then
                msg="\n         Пустой пароль недопустим"
                dialog_msgbox "$msg" 7 50
                continue
            fi

            pass2=$($DIALOG $D_OPT --stdout --backtitle "$BACKTITLE" --title "$TITLE" \
                                  --ok-label "Да" --cancel-label "Пропустить" \
                                  --extra-button --extra-label "Назад" --insecure \
                                  --passwordbox "\n              Повторите пароль \n " 11 50 )
            dialog_ret=$?
            if [[ $dialog_ret == 3 ]]    # назад
            then
                pass=
                return 33
            elif [[ $dialog_ret == 1 ]]
            then # Пользователь отказался
                pass=
                ret_code=0
                break
            elif [[ $dialog_ret != 0 ]]
            then
                return $dialog_ret
            fi

            if [[ "$pass" == "$pass2" ]]
            then
                ret_code=0
                break
            else
                msg="\n             Пароли не совпали"
                dialog_msgbox "$msg" 7 50
                continue
            fi
        done
    fi

    if [[ "$dialog_ret" != $DIALOG_ESC ]] && [[ -z "$pass" ]] && [[ "$user" == "root" ]]
    then
        show_def_root_pass
        return 0
    fi

    export "$ret_var=$pass"
    return $ret_code
}

show_def_root_pass()
{
    msg="\n    ВНИМАНИЕ! Установлен пароль по умолчанию \"f2line.\""
    dialog_msgbox "$msg" 7 60
    return 0
}

password_set()
{
    local user="$1"
    local pass="$2"

    [[ -z "$user" ]] && return

    # для рута в неинтерактивном режиме вставляем готовый хэш стандартного пароля рута
    if [[ $cur_arch != $img_arch ]] || [[ $dialog_min != 0 ]] || [[ -z "$pass" ]] && [[ "$user" == "root" ]]
    then
        sed -i -e 's/root::/root:Y4SW3wFiZ4dkY:/' $TARGET_DIR/etc/shadow 2>>$INST_LOG
        if [[ $? == 0 ]]
        then
           #chmod 600 $TARGET_DIR/etc/shadow >>$INST_LOG 2>&1
           echo -e "\nУстановлен пароль для root по умолчанию\n" >> $INST_LOG
           return 0
        else
            echo -e "\nОшибка установки пароля root" >> $INST_LOG
            msg="\n        Не удалось установить пароль по умолчанию"
            dialog_msgbox "$msg" 7 60
            return 1
        fi

    elif [[ $cur_arch == $img_arch ]]
    then
    # нативный режим
    [[ -z "$pass" ]] && return

    cmd="
passwd $user << EOF
$pass
$pass

EOF
"
        chroot $TARGET_DIR /bin/bash -c "$cmd" >>$INST_LOG 2>&1
        if [[ $? != 0 ]]
        then
            msg="\n    Ошибка установки пароля $user"
            dialog_msgbox "$msg" 7 50
        fi
    else
            echo -e "\nКак мы сюда попали при установке пароля?" >> $INST_LOG
            return
    fi
}


fstab_create()
{
    mkdir -p $TARGET_DIR/etc/

    if [ "$target_type" == directory ];
    then
cat > $TARGET_DIR/etc/fstab << EOF
/dev/sda1       /boot  ext2 defaults   1 2
/dev/sda2       /      ext4 defaults   1 1
EOF
        else
                generate_fstab
                cp $fstab $TARGET_DIR/etc/fstab
        fi
    [[ $? != 0 ]] && return 1

        echo -e "\n########## /etc/fstab: ##########"   >> $INST_LOG
        cat $TARGET_DIR/etc/fstab >> $INST_LOG
        echo -e   "######### End of fstab ##########\n" >> $INST_LOG

        return 0
}


cross_dev_select_dialog()
{
    dev_short=$($DIALOG $D_OPT --stdout --backtitle "$BACKTITLE" --title "$TITLE" \
    --ok-label "Выбрать" --cancel-label "Прервать" \
    --radiolist "\n  Как определяется устройство установки в целевом ВК: \n " 18 60 8 \
        'hda' '/dev/hda' on  \
        'hdb' '/dev/hdb' off \
        'hdc' '/dev/hdc' off \
        'hdd' '/dev/hdd' off \
        'sda' '/dev/sda' off \
        'sdb' '/dev/sdb' off \
        'sdc' '/dev/sdc' off \
        'sdd' '/dev/sdd' off \
        'nvme0n1' '/dev/nvme0n1' off \
        'vda' '/dev/vda' off )

    [[ $? != 0 ]] && return 1

    CROSS_TARGET_DEV="/dev/${dev_short}"
    [[ "$CROSS_TARGET_DEV" == *nvme* ]] && PART_PREFIX="p" || PART_PREFIX=

    return 0
}

hardcore_tune()
{
    # Для x86* включаем звук
    # mcstbug 117211
    if [[ $img_arch == x86 || $img_arch == amd64 ]]
    then
        module_on "snd_pcm_oss"
    fi

    # Для e8c/e8c2 включаем модуль ядра для отображения температуры процессорных ядер
    # mcstbug 126561
    if [[ $img_arch == e2k-8c || $img_arch == e2k-8c2 ]]
    then
        module_on "l_pcs"
    fi

    # mcstbug 140892
    # настройка cmdline для серверов с платами Intel 10G Ethernet
    if [[ $cross_flg == 0 ]] && lspci | grep "Ethernet controller:" | grep -q "Intel.*10-Gigabit"
    then
        local str=
        local i=1
        local port_num=
        # считаем количество портов контроллера
        port_num=$(lspci | grep "Ethernet controller:" | grep -c "Intel.*10-Gigabit")
        str="ixgbe.allow_unsupported_sfp=$(while [ $i -lt $port_num ]; do echo -n '1,'; let i++; done ; echo -n 1)"

        # добавляем в конец cmdline
        sed -i "/cmdline=/ s/$/ ${str}/" ${TARGET_DIR}/boot/boot.conf &>>$INST_LOG

        echo -e "\nОбновление cmdline в boot.conf:" >>$INST_LOG
        cat ${TARGET_DIR}/boot/boot.conf >>$INST_LOG
    else
        echo "Ethernet controller: Intel 10-Gigabit не найден" >> $INST_LOG
    fi
}

kill_chroot_procs()
{
    local pid
    dpkg_unpack_to_initrd "lsof"

    # убийство чрутовых процессов
    echo "lsof ${TARGET_DIR}:" >> $INST_LOG
    lsof ${TARGET_DIR} &>> $INST_LOG
    local pids=$(lsof ${TARGET_DIR} 2>/dev/null | grep -v lsof | awk '{print $2}' |grep -o [0-9].* | uniq)
    for pid in $pids
    do
        kill -9 $pid
    done
}

copy_repo_on_target()
{
    [[ $copy_repo_flg == 0 ]] && return 0

    TITLE="Копирование репозитория на целевой диск"

    cd ${SOURCE_DIR}/images/debs/
    echo 0 >$RESULT
    percent=0
    packs_quantity=$(ls -1 *.deb Packages.gz | wc -l)

    mkdir -p ${TARGET_DIR}/${REPO_DIR} >> $INST_LOG 2>&1 || return 1

    if [[ $dialog_min == 0 ]]
    then
        i=0
        for pack in $(ls -1 *.deb Packages.gz); do
            let i++;
            (cp -aP $pack ${TARGET_DIR}/${REPO_DIR} >> $INST_LOG 2>&1
             if [[ $? != 0 ]]
             then
                 echo 1 > $RESULT
                 exit 1
             fi
            ) | do_progress_cp $percent "\n  $pack"
            [[ `cat $RESULT` == 1 ]] && break

            percent=$[($i*100)/$packs_quantity]
        done | $DIALOG $D_OPT  --backtitle "$BACKTITLE" --title "$TITLE" --gauge "\n" 8 65
        [[ `cat $RESULT` == 1 ]] && return 1

    else # $dialog_min != 0
        dialog_infobox "$TITLE"
        for pack in $(ls -1 *.deb Packages.gz); do
            dialog_infobox "Копирование $pack..."
            cp -aP $pack ${TARGET_DIR}/${REPO_DIR} >> $INST_LOG 2>&1
            [[ $? != 0 ]] && return 1
        done
    fi

    cd $CWD

    (echo "deb [trusted=yes] file:${REPO_DIR} ./" > ${TARGET_DIR}/etc/apt/sources.list.d/disk1.list) 2>> $INST_LOG || return 1
}

finish()
{
    TITLE=$DEF_TITLE
    cd $CWD

    rm -fv $TARGET_DIR/$PIPE_FILE ${TARGET_DIR}/tmp/libeatmydata.so* >> $INST_LOG 2>&1

    # Отмонтирование системных директорий в chroot'е
    if [[ $cur_arch == $img_arch ]] && [[ $hdd_clone_flg != 1 ]]
    then
        [[ $cross_flg == 0 ]] && kill_chroot_procs

        dialog_infobox "\n  Отмонтирование системных директорий в chroot'е" 5 55
        cmd="umount -vlf /sys /proc /dev/pts /run"
        chroot $TARGET_DIR /bin/bash -c "$cmd" >> $INST_LOG 2>&1
        umount -vlf $TARGET_DIR/dev/pts        >> $INST_LOG 2>&1
        umount -vlf $TARGET_DIR/dev            >> $INST_LOG 2>&1
    fi

    if [[ $hdd_clone_flg == 1 ]]
    then
        [[ -d ${INST_LOCAL}/hdd_image_dir ]]    && umount -v ${INST_LOCAL}/hdd_image_dir    >> $INST_LOG 2>&1
        [[ -d ${INST_LOCAL}/lintel_image_dir ]] && umount -v ${INST_LOCAL}/lintel_image_dir >> $INST_LOG 2>&1
    else
        rm -f ${TARGET_DIR}/var/log/tweaking.log
        # Копирование лога на целевой диск
        # Дальнейшее в лог на целевом устройстве не попадает
        [ -f "$DEBUG_LOG" ] && cp $DEBUG_LOG ${TARGET_DIR}/var/log/einst_debug.log
        cp $INST_LOG ${TARGET_DIR}/var/log/einst_os.log
        [[ $? != 0 ]] && dialog_infobox "Ошибка копирования инсталляционного лога на целевой диск" 5 55 || sync
    fi

    if [[ "$target_type" = device ]]
    then
        dialog_infobox "\n      Размонтирование целевой ФС..." 5 45
        umount_fs
        [[ $? != 0 ]] && do_fatal
        # очистка временных файлов с разбивкой
        rm -rf ${INST_LOCAL}/part_edit
    fi

    # отмонтирование cgroups и прочих сервисов
    umount -v $(mount | grep "${TARGET_DIR}" | awk '{print $3}')

    if [[ $source_type = default || $source_type = device || $source_type = iso  ]]
    then
        if [[ $source_mounted == 1 ]]
        then
            msg="\n       Размонтирование установочного устройства..."
            dialog_infobox "$msg"
            umount -v $SOURCE_DIR >>$INST_LOG 2>&1
            [[ $? != 0 ]] && do_fatal
        fi
    fi

    if [[ $network_flg == 1 ]]
    then
        dialog_infobox "\n         Работа завершена\n\n      Ожидание перезагрузки..." 7 40
        # Дальше в цикле ожидание признака перезагрузки
        local ret_cp=1
        while true;do
            ### Копирование полного лога на сервер
            ### является признаком готовности клиента
            ### При сильной загрузке сети, может не пройти с первого раза
            if [[ $ret_cp != 0 ]]
            then
                cp $INST_LOG ${REMOTE_LOG_DIR}/${local_ip}_full_log
                ret_cp=$?
            fi
            sync    # проявление файла-признака ребута

            ### Проверка признака автоматической перезагрузки
            if [ -f ${REMOTE_LOG_DIR}/${local_ip}_reboot ] && $(grep -qEo "\<1\>" ${REMOTE_LOG_DIR}/${local_ip}_reboot)
            then
                # посылка признака на сервер
                echo 0 > ${REMOTE_LOG_DIR}/${local_ip}_reboot
                sync

                dialog_infobox "\n     Перезагрузка ВК..." 5 30
                reboot -f
            fi
            sleep 1
            echo -n '.'
        done
    elif [[ "$target_type" != directory ]] && [[ $cross_flg == 0 ]] && [[ $vm_flg == 0 ]]
    then
        ### перезагрузка после выхода из диалога
        [[ $dialog_min == 0 ]] && \
        dialog_msgbox "\n                      Работа завершена.\n\n            Для перезагрузки ВК нажмите \"Продолжить\"\n\nЛог установки в файле $INST_LOG" 10 65

        dialog_infobox "\n     Перезагрузка ВК..." 5 30
        reboot -f

    elif [[ "$target_type" == directory ]]
    then
        msg="\n                     Работа завершена\n\nЛог установки в файле\n$TARGET_DIR/var/log/einst_os.log"
        dialog_infobox "$msg" 7 65
    else
        msg="\n                     Работа завершена\n\nЛог установки в файле ${INST_LOG}"
        dialog_infobox "$msg" 6 65
    fi

    # Чистка рабочей директории на хостовой машине
   if [[ $cross_flg == 1 ]]
   then
       cd $INST_LOCAL
       for file in $(ls -Ilog -Idebug_log .) # все, кроме лога
       do
           [ -f $file ] && rm -f $file && continue
           [ -d $file ] && [ ! "`ls -A $file`" ] && rm -r $file && continue
       done
       [ -d "$TZ_UNPACK_DIR" ] && rm -rf $TZ_UNPACK_DIR
       cd /
   fi

    # Корректная очистка терминала в посл. порту
    if [[ -n "$SERIAL_PORT" ]]
    then
        clear
        reset
    fi
}

add_groups()
{
    [[ $cur_arch != $img_arch ]] && return 0

cmd="for gr in \$(echo ${groups} | sed 's/,/ /g'); do
grep -q \${gr}: /etc/group || groupadd \${gr}
done
"
    chroot $TARGET_DIR /bin/bash -c "$cmd" >>$INST_LOG 2>&1
    [[ $? != 0 ]] && msg="\n     Ошибка добавления пользовательских групп" && dialog_msgbox "$msg" 7 60 && \
        echo "$msg" >> $INST_LOG
}

new_user_dialog()
{
    [[ $cur_arch != $img_arch ]] || [[ $dialog_min != 0 ]] || [[ $auto_net_flg == 1 ]] && return 0

    local return_code=0
    local user_name=

    if [ -n "$1" ]; then
        local ret_var=$1
        user_name="${!1}"
        [ -z "$user_name" ] && user_name="user"
    else
        return 1
    fi

    TITLE="Добавление нового пользователя"

    # Диалоговый режим
    while true
    do
        # Имя пользователя
        user_name=`$DIALOG $D_OPT --backtitle "$BACKTITLE" --title "$TITLE" \
                --ok-label "Да" --cancel-label "Пропустить" \
                --extra-button --extra-label "Назад" \
                --stdout \
                --inputbox "                  Введите имя пользователя.\n\nДопустимы строчные буквы, цифры, дефис, нижнее подчёркивание.\nИмя должно начинаться с буквы или нижнего подчёркивания." 11 65 ${user_name}`

        ret=$?
        if [[ $ret == 3 ]]    # назад
        then
            user_name=
            return_code=33
            break
        # Выбран выход из диалога
        elif [[ $ret == 1 ]]
        then
            user_name="skip"
            return_code=0
            break
        elif [[ $ret != 0 ]]
        then
            return $ret
        fi

        if [ -n "$user_name" ]
        then
            return_code=0
            break
        else
            continue
        fi
    done

    export "$ret_var=$user_name"
    return $return_code
}

# 1 аргумент - имя пользователя
add_user()
{
    [[ $cur_arch != $img_arch ]] && return 0

    local user=
    [[ -n "$1" ]] && user="$1" || return 0

    cmd="mkdir -p /home"
    chroot $TARGET_DIR /bin/bash -c "$cmd" >>$INST_LOG 2>&1

    cmd="useradd -m -d /home/${user} -G ${groups} ${user}"
    chroot $TARGET_DIR /bin/bash -c "$cmd" >>$INST_LOG 2>&1
    if [[ $? != 0 ]]
    then
        msg="\n   Ошибка добавления пользователя $user"
        dialog_msgbox "$msg" 7 50
        return 1
    fi

    return 0
}

set_graphic_init()
{
    if [[ $graphic_flg == 1 ]]
    then
        sed -i -E 's/id:[[:digit:]]:/id:5:/g' $TARGET_DIR/etc/inittab >>$INST_LOG 2>&1
        [[ $? != 0 ]] && msg="\n  Не удалось включить графический режим по умолчанию" && dialog_msgbox "$msg" 7 60
    fi
}

set_inst_flags()
{
    [[ $dialog_min != 0 ]] && return 0

    TITLE=$DEF_TITLE
    local answer=
    local i=
    local ret=

    echo " --stdout --ok-label \"Да\" --separate-output --no-cancel
    --backtitle \"$BACKTITLE\" --title \"$TITLE\"
    --checklist" > $DIALOG_FILE

    echo -n '"                       Выберите пункты для установки" 11 80 6
    "graphic"   "Графический режим по умолчанию (при установке X-сервера)"    "off"
    "old_fstab" "Использование меток разделов вида /dev/aaaN вместо UUID"   "off"
    "cmdline"   "Редактировать cmdline ядра после установки"    "off"
    "repo"      "Копирование репозитория на целевой диск"    ' >> $DIALOG_FILE
    ([[ $copy_repo_flg == 1 ]] && echo \"on\" || echo \"off\") >> $DIALOG_FILE

    answer=`$DIALOG $D_OPT --file $DIALOG_FILE`
    ret=$?

    [[ "$ret" != 0 ]] && return $ret

    if [ -n "$answer" ]
    then
        for i in $answer
        do
            case $i in
            "old_fstab") old_fstab_flg=1            ;;
            "graphic")  graphic_flg=1               ;;
            "cmdline")  cmdline_flg=1               ;;
            "repo")     copy_repo_flg=1             ;;
            *)          echo "Unknown parametr $i"  ;;
            esac
        done

        # отключаем копирование репо, если не задано
        $(echo "$answer" | grep -q "repo" >> $INST_LOG 2>&1)
        [[ $? != 0 ]] && copy_repo_flg=0
    fi

    return 0
}

hdd_clone()
{
    local lintel_opts=
    local ret_cont=5

    [[ $network_flg != 1 ]] && return $ret_cont

    # проверяем опции в конфигурационном файее
    if [ -f "${REMOTE_LOG_DIR}/conf/install.cfg" ]
    then
        CONF_FILE="${REMOTE_LOG_DIR}/conf/install.cfg"

        # TODO возможно зависание при отсутствии файла, надо бы сократить пайп
        hdd_image="`cat $CONF_FILE | awk -F'#' '{print $1}' | grep -E "hdd_image[ \t]{0,}=" | awk -F'=' '{print $2}' | sed -e 's/^[ \t]*//;s/[ \t]*$//'`"
        if [ -n "$hdd_image" ]
        then
            hdd_clone_flg=1
            dialog_min=1
            dialog_infobox "Задан образ ФС для клонирования по сети $hdd_image"
        else
            # образ не задан, продолжаем обычную установку
            return $ret_cont
        fi
    else
        return $ret_cont
    fi

    # читаем остальные флаги
    hdd_image_target="`cat $CONF_FILE | awk -F'#' '{print $1}' | grep -E "hdd_image_target[ \t]{0,}=" | awk -F'=' '{print $2}' | sed -e 's/^[ \t]*//;s/[ \t]*$//'`"
    lintel_image="`cat $CONF_FILE | awk -F'#' '{print $1}' | grep -E "lintel_image[ \t]{0,}=" | awk -F'=' '{print $2}' | sed -e 's/^[ \t]*//;s/[ \t]*$//'`"
    lintel_image_target="`cat $CONF_FILE | awk -F'#' '{print $1}' | grep -E "lintel_image_target[ \t]{0,}=" | awk -F'=' '{print $2}' | sed -e 's/^[ \t]*//;s/[ \t]*$//'`"

    if [ -z "$hdd_image_target" ] # задан образ ФС, не задана цель
    then
        dialog_infobox "Не задано целевое устройство для записи клона ФС"
        do_fatal
    else
        ### Пробное чтение устройства
        check_device_readable $hdd_image_target
        if [[ $? != 0 ]]
        then
            dialog_infobox "Не удалось пробное чтение с целевого устройства для записи клона ФС $hdd_image_target"
            do_fatal
        fi
    fi

    if [ -n "$lintel_image" ]
    then
        dialog_infobox "Задан образ Lintel для клонирования по сети $lintel_image"

        if [ -z "$lintel_image_target" ]
        then
            dialog_infobox "Не задано целевое устройство для записи образа Lintel, используется /dev/hdc"
            lintel_image_target="/dev/hdc"
        fi

        ### Пробное чтение устройства
        check_device_readable $lintel_image_target
        if [[ $? != 0 ]]
        then
            dialog_infobox "Не удалось пробное чтение с целевого устройства для записи Lintel $lintel_image_target"
            do_fatal
        fi
        lintel_opts="-d $lintel_image_target -l ${INST_LOCAL}/lintel_image_dir/$lintel_image"
    fi

    # монтирование NFS директорий
    dialog_infobox "Монтирование образа ФС для клонирования..."
    mkdir -p ${INST_LOCAL}/hdd_image_dir >> $INST_LOG 2>&1 || do_fatal
    mount -v -n -o rw,nolock $server_ip:/inst.server/hdd_image_dir  ${INST_LOCAL}/hdd_image_dir >> $INST_LOG 2>&1
    [[ $? != 0 ]] && do_fatal

    if [ -n "$lintel_image" ]
    then
    dialog_infobox "Монтирование образа Lintel..."
    mkdir -p ${INST_LOCAL}/lintel_image_dir >> $INST_LOG 2>&1 || do_fatal
    mount -v -n -o rw,nolock $server_ip:/inst.server/lintel_image_dir  ${INST_LOCAL}/lintel_image_dir >> $INST_LOG 2>&1
    [[ $? != 0 ]] && do_fatal
    fi

    # Ставим недостающее
    bin_exist "which"
    if [ $? != 0 ]
    then
        dpkg_unpack_to_initrd "which" || do_fatal
    fi

    bin_exist "pigz"
    if [ $? != 0 ]
    then
        dpkg_unpack_to_initrd "pigz"  || do_fatal
    fi

    # вызываем прогу клонирования локально
    $CLONE_HDD -L $lintel_opts -g $hdd_image_target ${INST_LOCAL}/hdd_image_dir/$hdd_image 2>&1 | tee -a $INST_LOG | tee -a $REMOTE_LOG
    ret=${PIPESTATUS[0]}

    return $ret
}


check_devtree()
{
    [[ $img_arch == x86 || $img_arch == amd64 ]] && return 0
    [[ $vm_flg == 1 || $cross_flg == 1 ]] && return 0
    local dt_model=
    local dt_path="/sys/firmware/devicetree/base/model"
    local ret=0

    if [ -f "${dt_path}" ]
    then
        dt_model=$(cat ${dt_path} | tr '\0' '\n')
        ret=${PIPESTATUS[0]}
    else
        ret=1
    fi

    if [[ $ret != 0 ]] || [ -z "${dt_model}" ]
    then
        msg="\n      Внимание: не прошито device tree\n       возможна некорректная работа ВК"
        dialog_msgbox "$msg" 8 50
        return 1
    else
        echo "Обнаружено device tree, dt_model = ${dt_model}" >> $INST_LOG
        return 0
    fi
}

check_fruid()
{
    [[ $img_arch == x86 || $img_arch == amd64 ]] && return 0
    [[ $vm_flg == 1 || $cross_flg == 1 ]] && return 0
    local ret=0

    dialog_infobox "\n         Считывание FRUID..." 5 40
    mb_name=$(fruid_print | grep 'Board product name' | awk '{print $NF}' 2>>$INST_LOG)
    ret=${PIPESTATUS[0]}

    if [[ $ret != 0 ]] || [ -z "${mb_name}" ]
    then
        msg="\n          Внимание: не прошит FRUID\n       возможна некорректная работа ВК"
        dialog_msgbox "$msg" 8 50
        return 1
    else
        BACKTITLE="$BACKTITLE ${mb_name}"
        echo "FRUID:"   >> $INST_LOG
        fruid_print     &>>$INST_LOG
        return 0
    fi
}
