#!/bin/bash

###############################
#                             #
# Функции выбора мета-пакетов #
#                             #
###############################

dstr_meta_pack_choose()
{
    local ret=

    # результирующие списки для установки
    meta_list=              # метапакеты дистрибутива

    [ -z "$serverup_flg" ] && serverup_flg=0

    TITLE="Выбор пакетов"

    # Эта ветка работает в инсталляторе
    if [[ $auto_net_flg == 1 ]]
    then
        if [ -f "${REMOTE_LOG_DIR}/conf/meta_list" ]
        then
            meta_list=`cat ${REMOTE_LOG_DIR}/conf/meta_list`
            return 0
        else
            dialog_msgbox "Не найдены списки пакетов в ${REMOTE_LOG_DIR}/conf"
            # поставится все по умолчанию
        fi
    fi

    META_LIST_FULL="${SOURCE_DIR}/images/debs/DEB_LIST_META"        # Полный список метапакетов дистрибутива
    META_KERN_LIST="${SOURCE_DIR}/images/debs/DEB_LIST_KERNEL_META" # Метапакеты ядра

    # Выпилить bootstrap и планшетные пакеты из DEB_LIST_META
    cat $META_LIST_FULL | grep -v 'bootstrap' | grep -v 'meta-tablet' > $META_LIST

    if [[ $cur_arch == $img_arch ]] || [[ $serverup_flg == 1 ]]
    then # нативная установка
        # жестко конфигурируемый список пакетов в нативном бездиалоговом режиме
        if [ -n "$package_list" ]
        then
            return 0
        # Нативная установка в интерактивном режиме
        elif [[ $dialog_min == 0 ]]
        then
            # Выбор метапакетов дистрибутива
            meta_list_dialog
            return $?
        else # dialog_min == 1
            # без диалога ставим все
            meta_list=`cat $META_LIST | awk -F'_' '{print $1;}'` || do_fatal
        fi
    else
        # Кросс
        if [ $dialog_min == 0 ]
        then
            $DIALOG $D_OPT --title "$TITLE" --backtitle "$BACKTITLE" \
            --ok-label "Да" --extra-button --extra-label "Назад" --cancel-label "Нет" \
            --yesno "\nУстановить все пакеты дистрибутива?\n\nДа - установить все возможные пакеты\nНет - минимальная установка" 10 50
            ret=$?
            if [[ $ret == 0 ]]    # Yes/ok-button
            then
                # Установка всего
                minimal_flg=0
            elif [[ $ret == 1 ]]  # cancel-button
            then
                # Минимальная установка
                minimal_flg=1
            elif [[ $ret == 3 ]]  # Назад/extra-button
            then
                return 33
            else
                return $ret
            fi
        else
            # Без диалога - установка всего
            minimal_flg=0
        fi
    fi

    return 0
}

kern_meta_pack_choose()
{
    local ret=

    # результирующие списки для установки
    out_kern_meta_list=     # метапакеты ядра
    default_kernel_meta=    # Дефолтный метапакет ядра, ставится последним

    local numa_flg=0            # 0 - одна нода или отсутствует поддержка, 1 - больше одной ноды

    TITLE="Выбор пакетов"

    # Эта ветка работает в einst при автоустановке по сети
    if [[ $auto_net_flg == 1 ]]
    then
        if [ -f "${REMOTE_LOG_DIR}/conf/out_kern_meta_list" ] && [ -f "${REMOTE_LOG_DIR}/conf/default_kernel_meta" ]
        then
            out_kern_meta_list=`cat ${REMOTE_LOG_DIR}/conf/out_kern_meta_list`
            default_kernel_meta=`cat ${REMOTE_LOG_DIR}/conf/default_kernel_meta`
            return 0
        else
            dialog_msgbox "Не найдены списки пакетов в ${REMOTE_LOG_DIR}/conf"
            # поставится все по умолчанию
        fi
    fi

    META_KERN_LIST="${SOURCE_DIR}/images/debs/DEB_LIST_KERNEL_META" # Метапакеты ядра

    # нативная установка или задание списка пакетов для сервера сетевой установки
    # для кросса пакеты ставятся по спику KERN_SIMPLE
    if [[ $cur_arch == $img_arch ]] || [[ $serverup_flg == 1 ]]
    then
        # жестко конфигурируемый список пакетов в нативном бездиалоговом режиме
        if [ -n "$package_list" ]
        then
            return 0
        else
            while true
            do
                ### Задание умолчаний ###
                # для сетевой установки (запуск serverup)
                # дефолтное ядро с нумой
                if [ $serverup_flg -eq 1 ]
                then
                    default_kernel_meta="meta-kern-simple"

                # кросс установка при совпадении архитектур хоста и цели
                # куда вставят целевой жесткий диск не известно
                # поэтому дефолтное ядро с нумой
                elif [[ $cross_flg == 1 ]]
                then
                    default_kernel_meta="meta-kern-simple"

                # Проверка numa
                elif [ -d /sys/devices/system/node ] && [ `ls -d /sys/devices/system/node/node[0-9] | wc -l` -gt 1 ]
                then
                    numa_flg=1
                    echo "Обнаружено более одной numa node" >> $INST_LOG
                    default_kernel_meta="meta-kern-simple"
                else
                    numa_flg=0
                    echo "Отсутствует поддержка numa или найдена одна numa node" >> $INST_LOG
                    set_nn_image_default || default_kernel_meta="meta-kern-simple"
                fi

                # добавление ядерных утилит в список по умолчанию
                #[[ `grep "meta-kernel-utils" $META_KERN_LIST 2>>$INST_LOG >/dev/null; echo $?` == 0 ]] && \
                #out_kern_meta_list="meta-kernel-utils"

                depends=`for item in $default_kernel_meta $out_kern_meta_list; do \
                        dpkg --info ${SOURCE_DIR}/images/debs/${item}_*.deb 2>>$INST_LOG \
                        | grep 'Depends:' | sed -e 's# Depends: ##' | tr '\n' ','; done`

                depends=`echo $depends | sed 's#\(.*\),#\1#'`

                # Нативная установка в интерактивном режиме
                # Дефолтная установка ядра или выбираем
                # показываем пользователю, какие пакеты собираемся ставить
                if [[ $dialog_min == 0 ]]; then
                    unset dialog_opt dialog_extra dialog_init
                    local extra=
                    [[ $vm_flg == 1 ]] && extra="Пропустить"
                    dialog_extra+=("--help-button" "--help-label" "Назад")
                    dialog_call --yesno "Установить ядро и компоненты по умолчанию?\n\n${depends}" "Да" $extra "Выбрать"
                    ret=$?
                    if [[ $ret == 0 ]]      # ok
                    then
                        break
                    elif [[ $ret == 2 ]]    # help-button - назад
                    then
                        # сброс умолчаний
                        default_kernel_meta=
                        out_kern_meta_list=
                        return 33
                    elif [[ $ret == 3 ]]    # Extra-button - пропустить
                    then
                        # сброс умолчаний
                        default_kernel_meta=
                        out_kern_meta_list=
                        break
                    elif [[ $ret == 1 ]]    # No-button - выбрать вручную
                    then
                        # сброс умолчаний
                        default_kernel_meta=
                        out_kern_meta_list=
                        # выбор пакетов ядра
                        kern_meta_dialog
                        ret=$?
                        if [[ $ret == 0 ]]      # ok
                        then
                            break
                        elif [[ $ret == 1 ]]    # Cancel-button - назад
                        then
                            # сброс выбора
                            default_kernel_meta=
                            out_kern_meta_list=
                            continue
                        else
                            return $ret
                        fi
                    else
                        return $ret
                    fi
                # без диалога используем умолчания
                else
                    break
                fi
            done
        fi
    fi

    return 0
}

set_nn_image_default()
{
    if [[ `grep "meta-linux-image-nn_" $META_KERN_LIST 2>>$INST_LOG >/dev/null; echo $?` == 0 ]]
    then
        default_kernel_meta="meta-linux-image-nn"
        return 0
    else
        echo "Пакет meta-linux-image-nn не найден в списке $META_KERN_LIST" >> $INST_LOG
        return 1
    fi
}

meta_list_dialog()
{
    local ret=

    state=on    # Все выбрано по умолчанию

    list_height=`cat $META_LIST | wc -l`    || do_fatal
    list_height=$(($list_height+5))
    [ $list_height -gt 36 ] && list_height=36
    [ $list_height -lt 10 ] && list_height=10

    whole_height=$(($list_height+4))

    while true
    do
    echo " --stdout --ok-label \"Да\" --separate-output" > $DIALOG_FILE
    [[ $serverup_flg == 1 ]] && echo " --cancel-label \"Выход\"" >> $DIALOG_FILE || echo " --cancel-label \"Назад\"" >> $DIALOG_FILE
    echo " --extra-button --extra-label \"Снять/выбрать все\"
        --help-button --help-label \"О пакете\"
        --backtitle \"$BACKTITLE\" --title \"$TITLE\"
        --checklist
        \"Выбор метапакетов для установки\n\n\n    bootstrap устанавливается всегда (минимальная установка)\" $whole_height 100 $list_height">> $DIALOG_FILE

        while IFS='' read -r meta_pack_name || [[ -n "$meta_pack_name" ]];
        do
            echo \"`echo $meta_pack_name | awk -F'_' '{print $1;}'`\"    \"`dpkg --info ${SOURCE_DIR}/images/debs/${meta_pack_name}.deb 2>>$INST_LOG | \
                grep 'Description:' | sed -e 's/Description: //'`\"  `echo $state` >> $DIALOG_FILE
        done < $META_LIST

        meta_list=`$DIALOG $D_OPT --file $DIALOG_FILE`
        ret=$?
        case $ret in
            0) break     ;; # да
            1) return 33 ;; # назад
            2) show_pack_info "`echo $meta_list| awk '{print $2}'`" ; continue ;; # help button
            3) [[ "$state" == on ]] && state=off || state=on; continue ;; # снять/выбрать все
            *) return $ret ;;
        esac
    done

    return 0
}


show_pack_info()
{
    name=$1
    if [ -n "$name" ]
    then
        $DIALOG $D_OPT --cr-wrap --backtitle "$BACKTITLE" --title "Информация о пакете ${name}" \
                --msgbox "Используйте клавиши стрелки, PageUp/Down, End, Home для прокрутки\n\n`dpkg --info ${SOURCE_DIR}/images/debs/${name}_*.deb 2>>$INST_LOG`" 25 75
    else
        dialog_msgbox "     Ошибка: пустой параметр в show_pack_info()"
    fi
}


kern_meta_dialog()
{
    local ret=
    local state=

    list_height=`cat $META_KERN_LIST | wc -l`    || do_fatal
    list_height=$(($list_height+5))
    [ $list_height -gt 36 ] && list_height=36
    [ $list_height -lt 10 ] && list_height=10

    whole_height=$(($list_height+4))

while [ -z "$out_kern_meta_list" ]
do
    echo " --stdout --ok-label \"Да\" --separate-output --cancel-label \"Назад\"
        --backtitle \"$BACKTITLE\" --title \"$TITLE\"
        --help-button --help-label \"О пакете\" --help-status
        --checklist
        \"\nВыберите метапакеты ядра для установки\" $whole_height 120 $list_height" > $DIALOG_FILE

    while IFS='' read -r meta_pack_name || [[ -n "$meta_pack_name" ]]; do
        [[ "$meta_pack_name" == "meta-kern-simple"* ]] && continue   # Убираем из списка
	if [[ "$out_kern_meta_list" == *"$meta_pack_name"* ]]; then 
		state=on 
	else 
		state=off 
	fi
        # отбрасываем версию и приставку, чтобы влезло в диалог
        echo \"`echo $meta_pack_name | awk -F'_' '{print $1;}' | sed 's/^meta-//g'`\"    \
            \"`dpkg --info ${SOURCE_DIR}/images/debs/${meta_pack_name}.deb 2>>$INST_LOG | \
            grep 'Description:' | sed -e 's/[ ]*Description: Linux[ ]*//'`\"    \"`echo $state`\" >> $DIALOG_FILE
    done < $META_KERN_LIST
    [[ $? != 0 ]] && do_fatal

    # получили список с выбранными ядрами
    out_kern_meta_list=`$DIALOG $D_OPT --file $DIALOG_FILE`
    ret=$?
    if [ "$ret" -eq 2 ]; then
    # показываем help
    # + приставка в названии
        show_pack_info "`echo $out_kern_meta_list | awk '{print $2}' | sed 's/^/meta-/g'`"
        out_kern_meta_list=""
        continue # help button
    elif [[ "$ret" != 0 ]]; then
        return $ret
    fi

    if [ -z "$out_kern_meta_list" ]; then
        # не выбрано ни одного пакета
        continue
    else
        # добавляем приставку обратно
        local item=
        local tmp_list=
        for item in $out_kern_meta_list; do
            tmp_list+="$(echo $item | sed 's/^/meta-/g') "
        done
        out_kern_meta_list=$tmp_list
    fi

    # Если выбрано одно ядро (image) - оно дефолтное
    local n="`for thing in $out_kern_meta_list; do echo $thing | grep image; done | wc -l`"
    if [ "$n" -gt 1 ]
    then
        choose_def_kern
        if [[ $? -ne 0 ]]; then
            out_kern_meta_list=
            continue
        fi
        # Вырезаем из списка дефолтное ядро - оно ставится отдельно последним
        out_kern_meta_list=`echo $out_kern_meta_list | sed -e "s/${default_kernel_meta}//"`
    elif [ "$n" -lt 1 ]
    then
        msg="\n   Не задан ни один образ ядра (image) для установки!"
        dialog_msgbox "$msg" 7 60
        out_kern_meta_list=
        continue
    fi
done
}


# Выбор дефолтного ядрёного метапакета
choose_def_kern()
{
    local ret=

while [ -z "$default_kernel_meta" ]
do
    echo " --stdout --backtitle \"$BACKTITLE\" --title \"$TITLE\" --cancel-label \"Назад\" --no-items --radiolist \"\nВыберите ядро для загрузки по умолчанию\" 8 60 0" >$DIALOG_FILE
    for item in `for thing in $out_kern_meta_list; do echo $thing | grep image; done`
    do
        echo "\"$item\" off " >>$DIALOG_FILE
    done
        echo  >>$DIALOG_FILE

    default_kernel_meta=`$DIALOG $D_OPT --file $DIALOG_FILE`
    ret=$?
    if [[ $ret == 1 ]]  # Назад
    then return $ret
    fi
done

return 0
}


