#!/bin/bash

###################################
#                                 #
# Функции вывода сообщений и лога #
#                                 #
###################################

# Некоторые глобалы здесь из-за включения файла в разные модули
OK_LABEL="Продолжить"   # для dialog_msgbox
declare -a dialog_opt
declare -a dialog_extra

do_fatal()
{
    if [ -n "$1" ]
    then
        msg="\nОшибка в процедуре ${FUNCNAME[1]}:\n$1\nлог в $INST_LOG"
    else
        msg="\nОшибка в процедуре ${FUNCNAME[1]}\nлог в $INST_LOG"
    fi
    dialog_infobox "$msg"
    cleanup

    exit 1
}

# нажатие ESC в диалоге
do_exit()
{
    local dialog_ret=

    if [[ "$dialog_min" == 0 ]]
    then
        while true; do
            unset dialog_opt dialog_extra dialog_init dialog_extra
            dialog_extra+=("--defaultno")
            dialog_call --yesno "Прервать установку? Лог - Посмотреть лог установки" "Прервать" "Лог" "Продолжить"
            dialog_ret=$?
            if [[ "$dialog_ret" == 0 ]]     # выход
            then
                cleanup
                exit 0
            elif [[ "$dialog_ret" == 3 ]]   # лог
            then
                $DIALOG $D_OPT --exit-label "Назад" \
                        --textbox ${INST_LOG} 30 80 \
                        2>>$INST_LOG
                continue
            elif [[ "$dialog_ret" == 1 ]]   # продолжить установку
            then
                return 0
            elif [[ "$dialog_ret" == $DIALOG_ESC ]]   # ещё раз нажат ESC
            then
                continue
            else
                return $dialog_ret
            fi
        done
    else
        exit 0
    fi
}

### Вывод сообщений в диалог без ожидания пользователя
### в консоль/последовательный порт
### в лог
dialog_infobox()
{
    typeset m="$1"
    typeset x=$2
    typeset y=$3
    [[ -z "$m" ]] && m="Internal ERROR: empty message"
    [[ -z "$x" ]] && x=5
    [[ -z "$y" ]] && y=60


    ### Убираем пустые строки, пробелы и табуляции в начале и конце строки
    crop_msg=`echo -e "$m" | sed '/^$/d;s/^[ \t]*//;s/[ \t]*$//'`

    if [[ $dialog_min == 0 ]]
    then
        $DIALOG $D_OPT --backtitle "$BACKTITLE" --title "$TITLE" --no-collapse --infobox "$m" $x $y \
                2>>$INST_LOG
    elif [[ $dialog_min == 1 ]]
    then
        console_serial_message
    else
        echo "$crop_msg" >> $INST_LOG
    fi

    remote_message

    [[ -f $INST_LOG ]] && echo -e "$crop_msg" >> $INST_LOG
}

### Вывод сообщений в диалог с ожиданием пользователя
### в консоль/последовательный порт
### в лог
dialog_msgbox()
{
    typeset m="$1"
    typeset x=$2
    typeset y=$3

    [[ -z "$m" ]] && m="Internal ERROR: empty message"
    [[ -z "$x" ]] && x=5
    [[ -z "$y" ]] && y=60

    ### Убираем пустые строки, пробелы и табуляции в начале и конце строки
    crop_msg=`echo -e "$m" | sed '/^$/d;s/^[ \t]*//;s/[ \t]*$//'`

    if [[ $dialog_min == 0 ]]
    then
        $DIALOG $D_OPT --backtitle "$BACKTITLE" --title "$TITLE" \
        --ok-label $OK_LABEL --no-collapse --stdout --msgbox "$m" $x $y \
        2>>$INST_LOG
    elif [[ $dialog_min == 1 ]]
    then
        console_serial_message
    else
        echo "$crop_msg" >> $INST_LOG
    fi

    remote_message

    [[ -f $INST_LOG ]] && echo -e "$crop_msg" >> $INST_LOG
}

dialog_call()
{
    dialog_type="$1"
    text="$2"
    ok_label="$3"
    if [ $# -eq 5 ]; then
        dialog_extra+=("--extra-button" "--extra-label" "$4")
        cancel_label="$5"
    else
        cancel_label="$4"
    fi
    crop_msg=`echo -e "$text" | sed '/^$/d;s/^[ \t]*//;s/[ \t]*$//'`
    #longest str search
    width=$(($(echo -e "$text" | awk '{ print length, $0 }' | sort -n -s -r | cut -d" " -f1 | head -1) +5))
    [ $width -lt 60 ]  && width=$(($width+26))
    [ $width -gt 180 ] && width=180
    height=$(echo -e $text | wc -l)
    #center align
    if [ $height -eq 1 ]; then
        len=$(expr length "$text")
        text=$(echo; for i in `seq 1 $((($width-2-$len)/2))`; do echo -n " "; done; echo "$text")
    else
        text="\n$text\n"
    fi
    height=$(($height+6))
    if [ "$dialog_type" = "--menu" ] || [ "$dialog_type" = "--radiolist" ]; then
        menu_height=$((${#dialog_opt[*]}/2))
        height=$(($height+$menu_height))
    fi
    if [[ $dialog_min == 0 ]]
    then
        export IFS=$'\n'
            case $dialog_type in
                --menu|--radiolist )
                    height=$(($height+1))
                    val=$($DIALOG $D_OPT --stdout --no-tags --backtitle "$BACKTITLE" --title "$TITLE" --ok-label "$ok_label" \
                        --cancel-label "$cancel_label" ${dialog_extra[@]} $dialog_type "$text" $height $width $menu_height "${dialog_opt[@]}" \
                        2>>$INST_LOG)
                    ret=$?
                    echo "$val"
                ;;
                --inputbox )
                    height=$(($height+1))
                    val=$($DIALOG $D_OPT --stdout --no-tags --backtitle "$BACKTITLE" --title "$TITLE" --ok-label "$ok_label" \
                        --cancel-label "$cancel_label" ${dialog_extra[@]} $dialog_type "$text" $height $width "$dialog_init" \
                        2>>$INST_LOG)
                    ret=$?
                    echo "$val"
                ;;
                --yesno )
                    [ -z "$ok_label" ] && ok_label="Да"
                    [ -z "$cancel_label" ] && cancel_label="Нет"
                    # dialog bug with button labels in yesno widget when use extra button
                    # replace yes/no to ok/cancel
                    if [ -z "${dialog_extra[1]}" ] || [[ "${dialog_extra[@]}" == *"--help-button"* ]] && [[ "${dialog_extra[@]}" != *"--extra-button"* ]]
                    then
                        $DIALOG $D_OPT --stdout --no-collapse --backtitle "$BACKTITLE" --title "$TITLE" \
                                --yes-label "$ok_label" --no-label "$cancel_label" ${dialog_extra[@]} $dialog_type "$text" $height $width \
                                2>>$INST_LOG
                    else
                        $DIALOG $D_OPT --stdout --no-collapse --backtitle "$BACKTITLE" --title "$TITLE" \
                                --ok-label "$ok_label" --cancel-label "$cancel_label" ${dialog_extra[@]} $dialog_type "$text" $height $width \
                                2>>$INST_LOG
                    fi
                    ret=$?
                ;;
                --infobox )
                    height=$(($height-2))
                    $DIALOG $D_OPT --stdout --backtitle "$BACKTITLE" --title "$TITLE" --no-collapse --infobox "$text" $height $width \
                            2>>$INST_LOG
                    ret=$?
                ;;
                --msgbox )
                    $DIALOG $D_OPT --stdout --backtitle "$BACKTITLE" --title "$TITLE" --no-collapse --msgbox "$text" $height $width \
                            2>>$INST_LOG
                    ret=$?
                ;; 
                * )
                    do_fatal "Ошибка в вызове функции dialog_call"
                ;;
            esac
        export IFS=$' \t\n'
    elif [[ $dialog_min == 1 ]]
    then
        console_serial_message "$text"
    else
        echo "$text" >> $INST_LOG
    fi
    remote_message

    [[ -f $INST_LOG ]] && echo -e "$crop_msg" >> $INST_LOG
    return $ret
}

# Вывод в консоль и в последовательный порт
console_serial_message()
{
    [ -n "$1" ] && crop_msg=`echo -e $1 | sed '/^$/d;s/^[ \t]*//;s/[ \t]*$//'`

    ### Вывод в консоль с начала строки (из-за бага apt?)
    echo -e "\r$crop_msg"

    ### Вывод сообщений в последовательный порт при автоматической установке по сети
    if [[ -n "$SERIAL_PORT" ]] && [[ $auto_net_flg == 1 ]]
    then
        echo -e "$crop_msg" >> $SERIAL_PORT
    fi
    [[ $? != 0 ]] && do_fatal
}

# Вывод на сервер установки (serverup)
remote_message()
{
    ### Вывод сообщений в nfs шару на сервере сетевой установки
    if [ -n "$REMOTE_LOG" ]
    then
        [ -n "$1" ] && crop_msg=`echo -e $1 | sed '/^$/d;s/^[ \t]*//;s/[ \t]*$//'`
        echo -e "$crop_msg" >> $REMOTE_LOG
    fi
}


