# Глобальные переменные
declare -a eth_list
declare -a eth_name_list
declare -a ip_list
declare -a prefix_list
declare -a service_list
declare hostname
declare gateway
declare dns_servers
declare dns_search

# Служебные функции
net_dec_to_hex()
{
    printf "%02x" "$(printf "%d" "$(echo $1 | cut -d '.' -f 1)")"
    printf "%02x" "$(printf "%d" "$(echo $1 | cut -d '.' -f 2)")"
    printf "%02x" "$(printf "%d" "$(echo $1 | cut -d '.' -f 3)")"
    printf "%02x\n" "$(printf "%d" "$(echo $1 | cut -d '.' -f 4)")"
}

net_hex_to_dec()
{
    addr=$(printf "%08x" "0x$1")
    printf '%d.' "0x${addr:0:2}"
    printf '%d.' "0x${addr:2:2}"
    printf '%d.' "0x${addr:4:2}"
    printf '%d\n' "0x${addr:6:2}"
}

# Проверяет, поднят ли линк на интерфейсе $1
# is_link_up <iface>
is_link_up()
{
    ip link show $1 | grep DOWN  >>$INST_LOG 2>&1
    return $?
}

# Возвращает маску сети
# net_prefix_to_mask <prefix>
net_prefix_to_netmask()
{
    [ -z "$1" ] && return 1
    zero=$((32-$1))
    bin_netmask=$(printf '1%.0s' $(seq 1 $1);printf '0%.0s' $(seq 1 $((32-$1))))
    echo -n "$((2#${bin_netmask:0:8}))."
    echo -n "$((2#${bin_netmask:8:8}))."
    echo -n "$((2#${bin_netmask:16:8}))."
    echo $((2#${bin_netmask:24:8}))
}

# Проверка ipv4 адреса
net_check_ip()
{
    [ -z "$1" ] && return 1
    ip=$1
    if echo $ip |  grep -q -o -x -E '[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}' &>/dev/null; then
        return 0
    fi
    dialog_call --msgbox "Значение должно быть в формате nnn.nnn.nnn.nnn"
    return 1
}

# Возвращает broadcast адрес
# net_get_broadcast <ip> <prefix>
net_get_broadcast()
{
    if [ -z $1 ] || [ -z $2 ]; then
        echo
        return
    fi
    ip=$(net_dec_to_hex "$1")
    netmask=$(net_prefix_to_netmask $2)
    netmask=$(net_dec_to_hex "$netmask")
    netmask=$(printf "%08x" $((0x$netmask ^ 0xffffffff)))
    echo $(net_hex_to_dec "$(printf "%08x" $((0x$ip | 0x$netmask)))")
}

# Возвращает адрес сети
# net_get_network_addr <ip> <netmask>
net_get_network_addr()
{
    [ -z "$1" ] && return 1
    [ -z "$2" ] && return 1
    ip=$(net_dec_to_hex "$1")
    netmask=$(net_dec_to_hex "$2")
    echo $(net_hex_to_dec "$(printf "%08x" $((0x$ip & 0x$netmask)))")
}

# Проверяем, к какому интерфейсу принадлежит адрес
get_addr_iface()
{
    [ -z "$1" ] && return 1
    addr="$1"

    for ((i=0; i<${#eth_list[@]}; i=i+1)); do
        iface_netmask=$(net_prefix_to_netmask ${prefix_list[$i]})
        iface_network=$(net_get_network_addr ${ip_list[$i]} $iface_netmask)
        [ -z "$iface_network" ] && continue
        addr_network=$(net_get_network_addr $addr $iface_netmask)
        if [ "$addr_network" = "$iface_network" ]; then
            echo ${eth_list[$i]}
            break
        fi
    done
}

# $1 - подаем интересующий сетевой интерфейс
get_current_net_conf()
{
    ip=
    prefix=
    gateway=

    local ret=0
    local iface=
    [ -n "$1" ] && iface=$1 || return 1

    # Определение ip-адреса
    ip=`ifconfig $iface | grep -o -E 'inet addr:[0-9.]{,}' | cut -d ':' -f2 2>> $INST_LOG`
    [[ $? != 0 ]] || [[ -z "$ip" ]] && ret=1 && echo "Не удалось определить ip-адрес из $iface" >> $INST_LOG

    # Определение маски сети
    prefix=`ip addr show label $iface | grep 'inet ' | awk '{print $2}' | awk -F'/' '{print $2}' 2>> $INST_LOG`
    [[ $? != 0 ]] || [[ -z "$prefix" ]] && ret=1 && echo "Не удалось определить префикс из $iface" >> $INST_LOG

    # Определение шлюза
    gateway=`route | grep default | awk '{print $2}' 2>> $INST_LOG`
    [[ $? != 0 ]] || [[ -z "$gateway" ]] && echo "Не удалось определить шлюз по умолчанию из $iface" >> $INST_LOG

    if [[ $ret == 0 ]]
    then
        echo "$ip;$prefix;$gateway"
    fi

    return $ret
}

apply_net_config_common()
{
    ### Настройка /etc/sysconfig/network
    rm -f $TARGET_DIR/etc/sysconfig/network
    [ -z "$hostname" ] && hostname="$default_hostname"
    cat > $TARGET_DIR/etc/sysconfig/network << EOF
NETWORKING=yes
HOSTNAME=$hostname
#NISDOMAIN=
EOF

    echo -e "\n/etc/sysconfig/network:"     >> $INST_LOG
    cat $TARGET_DIR/etc/sysconfig/network   >> $INST_LOG

    ### Настройка /etc/resolv.conf
    rm -f $TARGET_DIR/etc/resolv.conf
    if [ -n "$dns_servers" ]
    then
        if [ -n "$dns_search" ]
        then
            cat >> $TARGET_DIR/etc/resolv.conf  << EOF
search $dns_search
EOF
        fi
        for dns_addr in $dns_servers
        do
            cat >> $TARGET_DIR/etc/resolv.conf      << EOF
nameserver $dns_addr
EOF
        done
        echo -e "\n/etc/resolv.conf:"   >> $INST_LOG
        cat $TARGET_DIR/etc/resolv.conf >> $INST_LOG
    else
        # заводим шаблон
        echo "#search mydomain.ru hisdomain.ru" >> $TARGET_DIR/etc/resolv.conf
        echo "#nameserver 1.1.1.11"             >> $TARGET_DIR/etc/resolv.conf
        echo "#nameserver 2.2.2.22"             >> $TARGET_DIR/etc/resolv.conf
    fi


    echo -e "\n/etc/hosts:"     >> $INST_LOG
    cat $TARGET_DIR/etc/hosts   >> $INST_LOG

    # добавляем шлюз
    if [ -n "$gateway" ]; then
        gateway_iface="$(get_addr_iface $gateway)"
        if [ -z "$gateway_iface" ]; then
            echo "Задан недостижимый адрес шлюза: $gateway. Пропускаю" >> $INST_LOG
        else
            echo "GATEWAY=$gateway" >> $TARGET_DIR/etc/sysconfig/network-devices/ifconfig.${gateway_iface}/ipv4
        fi
    fi
}

apply_net_config_iface()
{
    eth="$1"
    [ -n "$2" ] && service="$2"
    ### Настройка /etc/sysconfig/network-devices/ifconfig.$eth/ipv4
    mkdir -p $TARGET_DIR/etc/sysconfig/network-devices/ifconfig.$eth >> $INST_LOG 2>&1
    [[ $? != 0 ]] && do_fatal "Ошибка в apply_net_config_iface()"
    if [ $service = "static" ]
    then 
        if [ -z "$ip" ] || [ -z "$prefix" ]; then
            dialog_msgbox "Ошибка: не могу назначить сетевой адрес интерфейсу $eth.";
            return 1
        fi
        bcast=$(net_get_broadcast $ip $prefix)
        rm -f $TARGET_DIR/etc/sysconfig/network-devices/ifconfig.$eth/ipv4
        cat > $TARGET_DIR/etc/sysconfig/network-devices/ifconfig.$eth/ipv4 << EOF
TYPE=Ethernet
CHECK_LINK=yes
ONBOOT=yes
SERVICE=ipv4-static
IP=$ip
BROADCAST=$bcast
PREFIX=$prefix
EOF
    elif [[ $service == dynamic ]]
    then
        ip=     # Зачистка ip-адреса
        rm -f $TARGET_DIR/etc/sysconfig/network-devices/ifconfig.$eth/ipv4
        cat > $TARGET_DIR/etc/sysconfig/network-devices/ifconfig.$eth/ipv4 << EOF
TYPE=Ethernet
ONBOOT=yes
SERVICE=dhclient
EOF
    fi

    echo -e "\n/etc/sysconfig/network-devices/ifconfig.$eth/ipv4:"   >> $INST_LOG
    cat $TARGET_DIR/etc/sysconfig/network-devices/ifconfig.$eth/ipv4 >> $INST_LOG


    ### Настройка /etc/hosts для xfce4
    if [[ -n "$ip" ]]
    then
        cat $TARGET_DIR/etc/hosts | grep $ip > /dev/null 2>&1
        if [[ $? != 0 ]]
        then
            echo "$ip   $hostname"      >> $TARGET_DIR/etc/hosts
        fi
    fi

    return 0
}

network_init()
{
    [[ $cross_flg == 1 ]] || [[ $network_flg == 0 ]] && return 0

    # Сразу поднимаем все интерфейсы, избавляемся от лишних sleep
    for iface in $(ls /sys/class/net); do
        ifconfig $iface up >>$INST_LOG 2>&1
    done


    dialog_infobox "\n  Включение сетевых интерфейсов..." 5 40
    sleep 5

    ### Узнаем server_ip (в формате SERVER_IP=192.168.0.1/255.255.255.0)
    if [ -z "$server_ip" ]; then
        for i in `cat $cmdline | tr ' ' '\n'`; do
            if echo $i | grep -q "SERVER_IP="; then
            server_ip=${i##*SERVER_IP=}
            server_mask=$(echo $server_ip | cut -d '/' -f2)
            server_ip=$(echo $server_ip | cut -d '/' -f1)
            server_network=$(net_get_network_addr $server_ip $server_mask)
            echo "server_network=$server_network"  >> $INST_LOG
            fi
        done
    else
        server_mask="255.255.255.0"
        server_network=$(net_get_network_addr $server_ip $server_mask)
        echo "server_network=$server_network"  >> $INST_LOG
    fi


    ### Получаем список всех сетевых интерфейсов кроме sit0
    ifaces=`ifconfig -a | grep HWaddr |cut -d' ' -f1 | grep -v 'sit0'`

    # Если сетевых интерфейсов не нашлось
    if [ `for i in $ifaces; do echo $i; done | wc -l` -lt 1 ]
    then
        msg="Сетевые интерфейсы не обнаружены"
        dialog_infobox "$msg"
        do_fatal
    fi

    ### Если $eth уже задан, рассматриваем только один интерфейс
    [[ -n "$eth" ]] && ifaces="$eth"

    ### В автоматическом режиме поднимаем dhcp-клиент на каждом сетевом интерфейсе по-очереди
    if [[ $auto_net_flg == 1 ]]
    then
        ### clean ####
    mkdir -p /var/lib/dhclient
        :>/var/lib/dhclient/dhclient.leases
        for i in $ifaces; do
            is_link_up $i
            if [[ $? == 0 ]]; then
                dialog_msgbox "\n        Нет линка на интерфейсе $i" 6 40
                ifconfig $i down
                eth=""
                err=1
                continue
            else
                msg="\n            Попытка получения ip адреса на интерфейсе $i по dhcp..."
                dialog_infobox "$msg"
                dhclient -1 -v $i >> $DHCLIENT_LOG 2>&1
                err=$?
            fi
            if [[ $err -eq 0 ]]; then
                answer=$(ifconfig $i | grep "inet addr")
                local_ip=$(echo $answer | grep -o "addr:[0-9.]*" | cut -d':' -f2)
                local_mask=$(echo $answer | grep -o "Mask:[0-9.]*" | cut -d':' -f2)
                local_network=$(net_get_network_addr $local_ip $local_mask)
                echo "local_network=$local_network" >> $INST_LOG

                # server_ip = dhcp server ip
                if [ -z "$server_ip" ] || [ -z "$server_mask" ]; then
                    msg="Предупреждение: не задан SERVER_IP в cmdline. Будет использован адрес dhcp-сервера."
                    dialog_infobox "$msg" 7 60
                    server_network=$local_network
                    server_mask=$local_mask
                    server_ip=`cat $DHCLIENT_LOG | grep DHCPACK | grep -o -E "[0-9.]{,}"`
                    if [ -n "$server_ip" ]; then
                        echo -e "ip адрес сервера = ${server_ip}/${server_mask}\n" >> $INST_LOG
                    else
                        echo "Ошибка получения ip адреса сервера" >> $INST_LOG
                        do_fatal
                    fi
                fi

                ### Выходим из цикла, если мы в одной подсети с сервером
                if [[ "$server_network" = "$local_network" ]]; then
                   eth=$i;
                   break
                else
                    dialog_msgbox "Пропускаю интерфейс $i: получен ответ от dhcp-сервера, но сервер установки находится в другой подсети (    server_addr=$server_ip/$server_mask : local_addr=$local_ip/$local_mask)" 9 60
                    eth=""
                fi
            ### Пропускаем интерфейс, если нет ответа от DHCP-сервера
            elif [[ $err -eq 2 ]]; then
                eth=""
                continue;
            else
                msg="Ошибка dhcp клиента. Лог $DHCLIENT_LOG"
                dialog_infobox "$msg"
                do_fatal
            fi
        done
        if [ -z "$eth" ]; then
             msg="Ошибка: не могу найти сервер установки $server_ip"
             dialog_infobox "$msg"
             do_fatal
        fi


        mac_addr=`ifconfig $eth | grep -o "HWaddr [0-9A-F:]*" | awk '{print $NF}'`
        msg="mac адрес клиента ${eth} = ${mac_addr}"
        dialog_infobox "$msg"
        msg="ip адрес клиента = ${local_ip}"
        dialog_infobox "$msg"
        msg="ip адрес сервера = ${server_ip}"
        dialog_infobox "$msg"
        return 0
    fi
    # end of auto_net_inst part


    while true; do
        [[ $dialog_min == 1 ]] && break
        ### Выбираем сетевой интерфейс, если он не единственный
        [ "$(echo $ifaces | wc -w)" -eq 1 ] && eth="$ifaces"
        while [[ -z "$eth" ]]; do
            for i in $ifaces; do
                eth="$eth $i $i off"
            done
            eth=$($DIALOG $D_OPT --stdout --no-tags --backtitle "$BACKTITLE"  --title "$TITLE" \
                --ok-label "Выбрать" --cancel-label "Прервать" \
                --radiolist "Выберите сетевой интерфейс:" 10 60 0 $eth)
            ret=$?
            if [[ $ret == $DIALOG_ESC ]]; then  # ESC
                do_exit && continue
            elif [[ $ret != 0 ]]; then  # Нажата кнопка "Прервать"
                exit 0
            fi
        done

        ### проверка линка
        is_link_up $eth
        if [[ $? == 0 ]]; then
            dialog_msgbox "\n        Нет линка на $eth" 6 40
            ifconfig $eth down
            eth=
            continue
        fi
        # Задаем server_ip
        while [ -z "$server_ip" ]; do
            server_mask="255.255.255.0"
            server_ip=$($DIALOG $D_OPT --stdout --backtitle "$BACKTITLE" --ok-label 'Да' \
                    --cancel-label "Прервать" --form "Введите ip сервера" \
                    12 60 0 "ip:" 2 1 "`echo $local_ip | grep -o -E '[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.'`" 2 17 17 0)
            ret=$?
            if [[ $ret == $DIALOG_ESC ]]; then  # ESC
                do_exit && continue
            elif [[ $ret != 0 ]]; then  # Нажата кнопка "Прервать"
                exit 0
            fi
            if [ -z "$server_ip" ]; then
                continue
            elif [[ "$local_ip" = "$server_ip" ]]; then
                dialog_msgbox "\n    Введенный адрес совпадает с адресом клиента" 7 60
                local_ip=""
                continue
            fi
            server_network="$(net_get_network_addr $server_ip $server_mask)"
        done

        ### Выбираем static или DHCP
        while [[ -z "$service" ]]; do
            service="static Статическая off dynamic DHCP off"
            service=$($DIALOG $D_OPT --stdout --no-tags --backtitle "$BACKTITLE"  --title "$TITLE" \
                --ok-label "Выбрать" --cancel-label "Прервать" \
                --radiolist "Выберите тип конфигурации сети:" 10 60 0 $service)
            ret=$?
            if [[ $ret == $DIALOG_ESC ]]; then  # ESC
                do_exit && continue
            elif [[ $ret != 0 ]]; then  # Нажата кнопка "Прервать"
                exit 0
            fi
        done

        ### Вводим ip или получаем адрес по DHCP
        while [[ -z "$local_ip" ]]; do
            if [[ "$service" = "static" ]]; then
                local_ip=$($DIALOG $D_OPT --stdout --backtitle "$BACKTITLE" --ok-label 'Да' \
                    --cancel-label "Прервать" --form "Введите локальный ip \n      (Напр. 192.168.0.10[/255.255.255.0])" \
                    12 80 0 "ip:" 2 1 "`echo $server_ip | grep -o -E '[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.'`" 2 17 32  0)
                ret=$?
                if [[ $ret == $DIALOG_ESC ]]; then  # ESC
                    do_exit && continue
                elif [[ $ret != 0 ]]; then  # Нажата кнопка "Прервать"
                    exit 0
                fi

                #Проверка на корректность
                local_ip=$(echo "$local_ip" | sed 's/[[:space:]]//g')
                net_check_ip "$local_ip" || local_ip=""

                if [[ -z "$local_ip" ]]; then
                    continue
                elif [[ "$local_ip" = "$server_ip" ]]; then
                    dialog_msgbox "\n    Введенный адрес совпадает с адресом сервера" 7 60
                    local_ip=""
                    continue
                fi
                local_mask="$(echo $local_ip/ | cut -d'/' -f2)"
                local_ip="$(echo $local_ip | cut -d'/' -f1)"
            else
                dialog_infobox "\n          Получение ip-адреса по dhcp ($eth)..." 5 60
                dhclient -1 -v $eth >> $DHCLIENT_LOG 2>&1
                if [[ $? -ne 0 ]]; then
                    msg="\n    Ошибка: не удалось получить IP-адрес по DHCP"
                    dialog_msgbox "$msg" 7 60
                    do_fatal
                fi
                answer=$(ifconfig $eth | grep "inet addr")
                local_ip=$(echo $answer | grep -o "addr:[0-9.]*" | cut -d':' -f2)
                local_mask=$(echo $answer | grep -o "Mask:[0-9.]*" | cut -d':' -f2)
            fi

        done
        # Проверка на корректность введенного адреса
        ifconfig $eth "$server_ip" netmask $server_mask up >>$INST_LOG 2>&1
        if [[ $? != 0 ]]; then
            dialog_msgbox "\n    Проверьте правильность параметров сервера\n     $server_ip/$server_mask" 7 60
            server_ip=""
            continue
        fi

        [ -z "$local_mask" ] && local_mask="$server_mask"
        local_network=$(net_get_network_addr $local_ip $local_mask)
        ifconfig $eth "$local_ip" netmask $local_mask up >>$INST_LOG 2>&1
        if [[ $? != 0 ]]; then
            dialog_msgbox "\n    Проверьте правильность параметров $eth\n     $local_ip/$local_mask" 7 60
            local_ip=""
            continue
        fi


        ### Проверка на одинаковую подсеть с сервером
        if [[ "$server_network" != "$local_network" ]]; then
            dialog_msgbox "\n    Сервер установки находится в другой подсети\n    server_addr=$server_ip/$server_mask\n    local_addr=$local_ip/$local_mask" 9 60
            [[ "$service" = "dynamic" ]] && do_fatal
            local_ip=""
            continue
        fi

        break
    done

    # mac-адрес клиента в лог
    mac_addr=`ifconfig $eth | grep -o "HWaddr [0-9A-F:]*" | awk '{print $NF}'`
    echo "mac адрес клиента ${eth} = ${mac_addr}" >> $INST_LOG

    return 0
}


network_func()
{
    local nret=

    if [[ $network_flg == 1 ]]
    then
        # Поиск hostname в dhcpd.conf по mac-адресу
        DHCPD_CONF=${REMOTE_LOG_DIR}/conf/dhcpd.conf
        if [ -f "$DHCPD_CONF" ] && [ -n "${mac_addr}" ]
        then
            hostname_tmp=`sed "/host /h;/${mac_addr}/!d;x;/host /!d;q" $DHCPD_CONF | awk -F 'host ' '{print $2}'  | awk -F '{' '{print $1}' | sed 's/ *$//' 2>>$INST_LOG`
            [ -n "$hostname_tmp" ] && hostname=`echo $hostname_tmp | sed 's/ //g'`  # Удаление пробелов
        else
            # Генерация имени хоста
            hostname_tmp="`uname -i`_`echo ${local_ip} | cut -d'.' --output-delimiter=- -f3,4`"
            hostname="`echo $hostname_tmp | sed 's/ /_/g'`"  # замена пробелов на подчеркивание
        fi
    fi

    ### Настройка сети при автоматической установке
    if [[ $auto_net_flg == 1 ]]
    then
        # Проверка флага динамической настройки
        grep -q 'DHCLIENT' $cmdline > /dev/null
        if [[ $? = 0 ]]
        then
            eth_list+=("$eth")
            service_list+=("dynamic")
        else
            # иначе статическая настройка
            current_conf=$(get_current_net_conf $eth)
            eth_list+=("$eth")
            service_list+=("static")
            ip_list+=("$(echo $current_conf | cut -d ';' -f 1)")
            prefix_list+=("$(echo $current_conf | cut -d ';' -f 2)")
            gateway=$(echo $current_conf | cut -d ';' -f 3)
        fi
        return 0
    fi
}

set_net_ifaces()
{
    local answer=
    local value=
    local dret=

    TITLE="Добавить/удалить сетевые интерфейсы"

    if [[ $cross_flg == 0 ]]
    then
        # Настройка каждого найденного интерфейса
        # Игнорируем локальный и виртуальный интерфейсы
        eth_list=($(ls /sys/class/net  | grep -v 'lo' | grep -v 'sit'))

        # Если пользовтель не принимает участия в конфигурации сети, 
        # ставим dhclient на все интерфейсы с линком
        if [[ $dialog_min != 0 ]]
        then
            for ((i=0; i<${#eth_list[@]}; i=i+1)); do
                is_link_up ${eth_list[$i]}
                [ $? -eq 1 ] && service_list[$i]="dynamic"
            done
            return 0
        fi
    # в кроссе добавляем интерфейсы руками
    else
        [[ $dialog_min != 0 ]] && return 0

        while true; do
            dialog_iface_empty_ret=
            if [ ${#eth_list[@]} -gt 0 ] && [[ "${eth_list[@]}" != "" ]]
            then
                unset dialog_opt dialog_extra dialog_init dialog_opt
                for ((i=0; i<${#eth_list[@]}; i=i+1)); do
                    dialog_opt+=("${eth_list[$i]}" "${eth_list[$i]}" )
                done
                dialog_extra+=("--help-button" "--help-label" "Назад")
                dialog_iface=$(dialog_call --menu "Выберите интерфейс:" "Добавить" "Удалить" "Далее")
                dialog_iface_ret=$?

                # вычисление индекса
                for ((i=0; i<${#eth_list[@]}; i=i+1)); do
                    [ "${eth_list[$i]}" == "$dialog_iface" ] && break
                done
            # нет интерфейсов
            else
                unset dialog_opt dialog_extra dialog_init dialog_extra
                dialog_extra+=("--help-button" "--help-label" "Назад")
                dialog_iface=$(dialog_call --yesno "Добавить сетевой интерфейс?" "Да" "Пропустить")
                dialog_iface_ret=$?
                dialog_iface_empty_ret=$dialog_iface_ret
            fi

            # Добавить
            if [ $dialog_iface_ret -eq 0 ]; then
                unset dialog_opt dialog_extra dialog_init
                dialog_init="eth"
                value=$(dialog_call --inputbox "Введите название сетевого интерфейса" "Принять" "Отмена")
                [ $? -ne 0 ] && continue

                tmp_eth=$(echo "$value" | sed 's/[[:space:]]//g')

                # проверка на пустой ввод
                [[ -z "$tmp_eth" ]] && continue

                # проверка на дублирование
                dret=0
                for ((j=0; j<${#eth_list[@]}; j=j+1)); do
                    if [ "${eth_list[$j]}" == "$tmp_eth" ]
                    then
                        dret=1
                        dialog_call --msgbox "Дублирование интерфейса $tmp_eth"
                        break
                    fi
                done
                [[ "$dret" == 1 ]] && continue || eth_list+=("$tmp_eth")

            # Удалить
            elif [ $dialog_iface_ret -eq 3 ]; then
                unset 'eth_list[$i]'
                unset 'eth_name_list[$i]'
                unset 'service_list[$i]'
                unset 'ip_list[$i]'
                unset 'prefix_list[$i]'
                # remove index gap
                eth_list=("${eth_list[@]}")
                eth_name_list=("${eth_name_list[@]}")
                service_list=("${service_list[@]}")
                ip_list=("${ip_list[@]}")
                prefix_list=("${prefix_list[@]}")

            # Далее/Пропустить
            elif [ $dialog_iface_ret -eq 1 ]; then
                [[ "$dialog_iface_empty_ret" == 1 ]] && return 0 || break
            # назад (help button)
            elif [ $dialog_iface_ret -eq 2 ]; then
                return 33
            else
                return $dialog_iface_ret
            fi
        done
    fi
}

set_net_conf()
{
    TITLE="Задать параметры сетевых интерфейсов"

    # настройка каждого интерфейса
    if [ ${#eth_list[@]} -gt 0 ]
    then
        for ((i=0; i<${#eth_list[@]}; i=i+1)); do
            if [[ $cross_flg == 0 ]]
            then
                # Получаем модель адаптера
                #01:01.0 Ethernet controller: Intel Corporation MFE/M (rev 01) -> Intel Corporation MFE/M (rev 01)
                eth_name_list[$i]=`lspci -s $(ls -l /sys/class/net/ | grep ${eth_list[$i]} | grep -o -E '[0-9a-Z.:]{12,12}/net' \
                    | cut -d'/' -f1) | awk -F': ' '{print $2}'`

                current_conf=$(get_current_net_conf "${eth_list[$i]}")
            else
                # задание элементов массива для корректного редактирования интерфейсов
                eth_name_list[$i]=""
                current_conf=
            fi

            if [ -n "$current_conf" ]
            then
                service_list[$i]="static"
                ip_list[$i]=$(echo $current_conf | cut -d ';' -f 1)
                prefix_list[$i]=$(echo $current_conf | cut -d ';' -f 2)
                [ -z "$gateway" ] && gateway=$(echo $current_conf | cut -d ';' -f 3)    # не привязан к интерфейсу, один на всех
            else
                # задание элементов массива для корректного редактирования интерфейсов
                [ -z "${service_list[$i]}" ] && service_list[$i]="dynamic"
                [ -z "${ip_list[$i]}" ]      && ip_list[$i]=""
                [ -z "${prefix_list[$i]}" ]  && prefix_list[$i]=""
            fi
        done

        # без диалога выходим
        [[ $dialog_min != 0 ]] && return 0

        while true; do
            unset dialog_opt dialog_extra dialog_init
            dialog_extra+=("--column-separator" "|")
            for ((i=0; i<${#eth_list[@]}; i=i+1)); do
                dialog_opt+=("${eth_list[$i]}" "${eth_list[$i]} | ${service_list[$i]} | $([ -n "${ip_list[$i]}" ] && echo "${ip_list[$i]}/${prefix_list[$i]}") | ${eth_name_list[$i]}")
            done
            dialog_iface=$(dialog_call --menu "Выберите интерфейс для настройки:" "Изменить" "Назад" "Сохранить")
            dialog_iface_ret=$?

            for ((i=0; i<${#eth_list[@]}; i=i+1)); do
                [ "${eth_list[$i]}" = "$dialog_iface" ] && break
            done

            tmp_service="${service_list[$i]}"
            tmp_ip="${ip_list[$i]}"
            tmp_prefix="${prefix_list[$i]}"
            # Выбрать
            if [ $dialog_iface_ret -eq 0 ]; then
                answer_ret=0
                while true; do
                        unset dialog_opt dialog_extra dialog_init
                        # STATIC
                        if [[ $tmp_service == "static" ]]; then
                            dialog_opt+=("service"       "service     : $tmp_service")
                            dialog_opt+=("ip"            "ip          : $tmp_ip")
                            dialog_opt+=("prefix"        "prefix      : $tmp_prefix")

                        # DYNAMIC or OFF
                        elif [[ $tmp_service == "dynamic" ]] || [[ $tmp_service == "disable" ]]
                        then
                             dialog_opt+=("service"      "service  : $tmp_service")
                        else
                            do_fatal "Неизвестный или пустой параметр service = \"$tmp_service\""
                        fi
                        answer=$(dialog_call --menu "Редактирование интерфейса $dialog_iface. Выберите параметр для изменения." "Редактировать" "Сохранить" "Отмена")
                        answer_ret=$?

                        # Сохранить
                        if [ $answer_ret -eq 0 ]; then
                            # Обработка выбранного параметра
                            case $answer in
                            service )
                                # в кроссе не предлагаем выключать интерфейс
                                unset dialog_opt dialog_extra dialog_init
                                if [ $tmp_service = "static" ]; then
                                    dialog_opt+=("static"        "Статическая")
                                    dialog_opt+=("dynamic"       "Динамическая")
                                    [[ $cross_flg == 0 ]] && dialog_opt+=("disable" "Выключено")
                                else
                                    dialog_opt+=("dynamic"       "Динамическая")
                                    dialog_opt+=("static"        "Статическая")
                                    [[ $cross_flg == 0 ]] && dialog_opt+=("disable" "Выключено")
                    
                                fi
                                value=$(dialog_call --menu "Выберите тип конфигурации" "Да" "Отмена")
                                [ $? -ne 0 ] && continue

                                tmp_service=$value
                            ;;
                            ip )
                                unset dialog_opt dialog_extra dialog_init
                                dialog_init=$tmp_ip
                                value=$(dialog_call --inputbox "Введите IP v4 адрес" "Принять" "Отмена")
                                [ $? -ne 0 ] && continue
                        
                                #Проверка на корректность
                                value=$(echo "$value" | sed 's/[[:space:]]//g')
                                if [ -n "$value" ]; then
                                    net_check_ip "$value" || continue
                                fi
                                tmp_ip=$value
                            ;;
                            prefix )
                                unset dialog_opt dialog_extra dialog_init
                                dialog_init=$tmp_prefix
                                value=$(dialog_call --inputbox "Введите значение префикса сети от 0 до 32" "Принять" "Отмена")
                                [ $? -ne 0 ] && continue

                                #Проверка на корректность
                                value=$(echo "$value" | sed 's/[[:space:]]//g')
                                echo $value |  grep -q -o -x -E '[0-9]{1,2}' || [ -z "$value" ] &>/dev/null
                                if [ $? -ne 0 ] || [[ -n "$value" && "$value" -gt 32 ]]
                                then
                                dialog_msgbox "Значение должно быть в формате nn"
                                    continue    # выход
                                fi
                                tmp_prefix=$value
                            ;;
                           esac

                    # Отмена
                    elif [ $answer_ret -eq 1 ]
                    then
                        # восстановление параметров
                        if [ -z "$tmp_ip" ] || [ -z "$tmp_prefix" ] && [ $tmp_service = "static" ]
                        then
                            #tmp_service="${service_list[$i]}"
                            tmp_ip="${ip_list[$i]}"
                            tmp_prefix="${prefix_list[$i]}"
                        fi
                        break
                    # Сохранить
                    elif [ $answer_ret -eq 3 ]
                    then
                        if [ $tmp_service = "static" ] # && ( [ -n "$tmp_ip" ] || [ -n "$tmp_prefix" ] )
                        then
                            if [ -z "$tmp_ip" ]; then
                                dialog_msgbox "\n                 Введите ip" 7 50
                                continue
                            elif [ -z "$tmp_prefix" ]; then
                                dialog_msgbox "\n               Введите prefix" 7 50
                                continue
                            fi
                        fi
                        break;
                    else
                        return $answer_ret
                    fi
                done

                # Сохранить в диалоге настройки интерфейса
                if [ $answer_ret -eq 3 ]; then
                    service_list[$i]=$tmp_service
                    if [ "$tmp_service" = "static" ]; then
                        ip_list[$i]=$tmp_ip
                        prefix_list[$i]=$tmp_prefix
                    else
                        ip_list[$i]=""
                        prefix_list[$i]=""
                    fi
                fi
            # продолжить
            elif [ $dialog_iface_ret -eq 1 ]; then
                    return 0
            # назад
            elif [ $dialog_iface_ret -eq 3 ]; then
                return 33
            else
                return $dialog_iface_ret
            fi

        done
    elif [[ $cross_flg == 0 ]]
    then
        dialog_msgbox "Сетевые интерфейсы не обнаружены"
        return 0
    else
        # если вернулись в процедуру вверх по стеку
        # и интерфейсов нет - идём назад
        # иначе вперёд
        [[ $stack_ret == 33 ]] && return 33 || return 0
    fi
}

set_host_gw_dns()
{
    [[ $auto_net_flg == 1 ]] || [[ $dialog_min != 0 ]] && return 0

    # Настройка в диалоговом режиме
    TITLE="Задать параметры сети"
    local answer=
    local value=
    local static_flg=0

    # есть интерфейс со статической настройкой?
    for ((n=0; n<${#eth_list[@]}; n=n+1)); do
        if [[ "${service_list[$n]}" == static ]] ; then
            static_flg=1
            break
        fi
    done

    tmp_hostname="$hostname"
    [ $static_flg == 1 ] && tmp_gateway="$gateway" || tmp_gateway=
    tmp_dns_servers="$dns_servers"
    tmp_dns_search="$dns_search"

    while true; do
        unset dialog_opt dialog_extra dialog_init
        dialog_extra+=("--help-button" "--help-label" "Назад")
        dialog_opt+=("hostname"      "hostname    : $tmp_hostname")
        # если все интерфейсы настраиваются динамически или выключены
        # не выводим этот пункт меню
        [[ $static_flg == 1 ]] && dialog_opt+=("gateway"       "gateway     : $tmp_gateway")
        dialog_opt+=("dns_servers"   "dns_servers : $tmp_dns_servers")
        dialog_opt+=("dns_search"    "dns_search  : $tmp_dns_search")
        answer=$(dialog_call --menu "Редактирование сетевых настроек. Выберите параметр для изменения." "Редактировать" "Сохранить" "Пропустить")
        answer_ret=$?

        # Редактировать
        if [ $answer_ret -eq 0 ]; then
            case $answer in
            hostname )
                unset dialog_opt dialog_extra dialog_init
                dialog_init=$tmp_hostname
                value=$(dialog_call --inputbox "Введите имя ВК" "Принять" "Отмена")
                [ $? -ne 0 ] && continue

                # Ограничения длины hostname в Linux 64 байта
                if [ $(echo "$value" | wc -m) -ge 64 ]; then
                    dialog_call --msgbox "$answer не должен превышать 64 символов"
                    continue    # выход
                fi

                if [ -n "$value" ]; then
                    [[ $(echo -n "$value" | wc -m) != $(echo -n "$value" | grep -o -e '[a-Z0-9,_,-]' | wc -w) ]] && \
                        dialog_call --msgbox  "Допустимые символы A-Z a-z 0-9 - _" && \
                        continue
                    tmp_hostname="$value"
                fi
                ;;
            gateway )
                unset dialog_opt dialog_extra dialog_init
                dialog_init=$tmp_gateway
                value=$(dialog_call --inputbox "Введите адрес шлюза" "Принять" "Отмена")
                [ $? -ne 0 ] && continue

                #Проверка на корректность
                value=$(echo "$value" | sed 's/[[:space:]]//g')
                if [ -n "$value" ]; then
                    net_check_ip "$value" || continue
                fi

                tmp_gateway="$value"
                ;;
            dns_servers )
                unset dialog_opt dialog_extra dialog_init
                dialog_init=$tmp_dns_servers
                value=$(dialog_call --inputbox "Введите IP-адреса DNS серверов через пробел" "Принять" "Отмена")
                [ $? -ne 0 ] && continue

                #Проверка на корректность
                for v in $(echo $value | tr ',' ' ') ; do
                    v=$(echo "$v" | sed 's/[[:space:]]//g')
                    if [ -n $v ]; then
                        net_check_ip "$v" || continue 2
                    fi
                done

                tmp_dns_servers="$value"
                ;;
            dns_search )
                unset dialog_opt dialog_extra dialog_init
                dialog_init=$tmp_dns_search
                value=$(dialog_call --inputbox "Введите имя домена. Например \"mydomain.ru\"" "Принять" "Отмена")
                [ $? -ne 0 ] && continue

                # не более 64 символов
                if [ $(echo "$value" | wc -m) -ge 64 ]; then
                    dialog_call --msgbox "$answer не должен превышать 64 символов"
                    continue    # выход
                fi

                tmp_dns_search="$value"
                ;;
            esac
        # Сохранить
        elif [ $answer_ret -eq 3 ]; then
            hostname="$tmp_hostname"
            dns_servers="$tmp_dns_servers"
            dns_search="$tmp_dns_search"
            if [ -n "$tmp_gateway" ] && [ -z "$(get_addr_iface $tmp_gateway)" ]; then
                dialog_msgbox "\n     Шлюз недостижим ни на одном сетевом интерфейсе" 7 60
                continue
            fi
            gateway="$tmp_gateway"
            return 0
         # Пропустить
        elif [ $answer_ret -eq 1 ]; then
            hostname=
            gateway=
            dns_servers=
            dns_search=
            return 0
        # Назад
        elif [ $answer_ret -eq 2 ]; then
            hostname="$tmp_hostname"
            gateway="$tmp_gateway"
            dns_servers="$tmp_dns_servers"
            dns_search="$tmp_dns_search"
            return 33
        else
            return $answer_ret
        fi
    done
}


apply_network_settings()
{
    for ((i=0; i<${#eth_list[@]}; i=i+1)); do
        eth=${eth_list[$i]}
        service=${service_list[$i]}
        ip=${ip_list[$i]}
        prefix=${prefix_list[$i]}
        bcast=$(net_get_broadcast $ip $prefix)

        [ -n "$service" ] && [[ "$service" != "disable" ]] && apply_net_config_iface "$eth"
    done
    apply_net_config_common

}
