#!/bin/bash

PWD=`pwd`

# должка быть выделенная директория, т.к. чистится целиком
INITRD_REPACK_DIR="/tmp/inst_ird-$(head -n100 /dev/urandom | tr -dc _a-zA-Z0-9 | head -c8)"
NEW_INITRD="/tmp/initrd-$(head -n100 /dev/urandom | tr -dc _a-zA-Z0-9 | head -c8).img"

INITRD=$1
EINST_DIR=$2
DEB_DIR=$3

deb_flg=0

usage()
{
    echo ""
    echo "usage: $0 {initrd} {einst dir} [deb dir]"
    echo ""
    echo "Обновление установочного initrd заданным инсталлятором и deb-пакетами"
    echo ""
    echo "initrd - инсталляционный initrd для перепаковки"
    echo ""
    echo "einst dir - директория с инсталлятором ОС,"
    echo "в которой лежат скрипты einst, serverup и т.д."
    echo ""
    echo "deb dir (опционально) - директория с deb-пакетами"
    echo "для установки в initrd"
}

clean()
{
    # не корень ФС?
    echo "$INITRD_REPACK_DIR" | grep -q -x -E '^[/]+'
    if [[ $? == 0 ]]
    then
        echo "Ошибка: задан корневой раздел как рабочая директория"
        exit 1
    fi

    if [ -d $INITRD_REPACK_DIR ]
    then
        echo "Удаление рабочей директории..."
        rm -rf $INITRD_REPACK_DIR || exit 1
        return 0
    else
        return 1
    fi
}

init()
{
    if [ -z "$EINST_DIR" ] || [ ! -d $EINST_DIR ]
    then
        echo "Ошибка: не подан каталог с инсталлятором ОС"
        usage
        exit 1
    elif [ ! -f $EINST_DIR/einst ] && [ ! -f $EINST_DIR/serverup ]
    then
        echo "Ошибка: не найден инсталлятор в $EINST_DIR"
        exit 1
    else
        # полный путь
        EINST_DIR=`readlink -f $EINST_DIR`
    fi
    
    if [ -z "$INITRD" ]
    then
        echo "Ошибка: пустой параметр INITRD"
        exit 1
    elif [ ! -f $INITRD ]
    then
        echo "Ошибка: не найден $INITRD"
        exit 1
    else
        INITRD=$(readlink -f $INITRD)
    fi

    if [ -n "$DEB_DIR" ]
    then
        if [ ! -d $DEB_DIR ]
        then
            echo "$DEB_DIR не является директорией или не существует"
            exit 1
        elif [ ! "`ls -A $DEB_DIR/*.deb 2>/dev/null`" ]
        then
            echo "deb-пакеты не найдены в $DEB_DIR"
            exit 1
        else
            deb_flg=1
            DEB_DIR=`readlink -f $DEB_DIR`
        fi
    else
        deb_flg=0
        echo "Директория с deb-пакетами не задана"
        # продолжаем, это опционально
    fi

    echo "Рабочая директория - $INITRD_REPACK_DIR"
    mkdir -p $INITRD_REPACK_DIR || exit 1

    `which pigz &>/dev/null`
    [[ $? == 0 ]] && zip_prog="pigz" || zip_prog="gzip"
}

unpack()
{
    echo "Распаковка $INITRD ..."
    cd $INITRD_REPACK_DIR       || exit 1
    $zip_prog -d < $INITRD | cpio -i || exit 1
}

pack()
{
    echo "Копирование инсталлятора ОС..."
    rm -rf $INITRD_REPACK_DIR/inst/*               || exit 1
    cp -r $EINST_DIR/* $INITRD_REPACK_DIR/inst/    || exit 1

    # установка пакетов
    if [ $deb_flg == 1 ]
    then
        for f in $DEB_DIR/*.deb
        do
            echo "Распаковка $f ..."
            dpkg -x $f $INITRD_REPACK_DIR               || exit 1
        done
    fi

    # пауза
#    echo "Нажмите ENTER для продолжения" && read

    echo "Запаковка $NEW_INITRD ..."
    find . | cpio -o -H newc | $zip_prog > $NEW_INITRD  || exit 1
}


# MAIN #
if [[ $# == 0 ]]
then
    usage
    exit 0
fi

clean
init
unpack
pack
clean
echo "Работа завершена"

